<?php

/**
 * @mainpage 简介
 *
 * 本SDK包括访问AbleCloud云端服务的API，以及AbleCloud与微信公众号对接的API。
 *
 * - 子目录 lib/ablecloud
 *
 *  AbleCloud云端服务API。
 *
 * - 子目录 lib/bridge
 *
 *  AbleCloud与第三方云平台对接的工具。与微信对接：ACBridgeWeChat。
 *
 * - 子目录 demo
 *
 *  微信公众号与AbleCloud对接demo。
 *
 * - 子目录 docs
 *
 *  API参考文档。首页名字：index.html。
 *
 * - 适用PHP版本
 *
 *  本SDK使用了PHP v5.6及其后续版本才支持的不定长参数。使用v5.6之前版本PHP的开发者可以修改文件 ablecloud/services/ACStoreScanner.php 第45行及第123行，分别去掉行中函数select及函数groupBy参数列表里的'...'符号，并在调用此两行所对应的函数时使用字符串数组作为参数。
 *
 * - 使用说明
 *
 *   + 配置 ablecloud/ACConfig.php 中的开发者信息
 *
 *   + 引用文件 ablecloud/ACClient.php
 *
 *     在调用 session_start() 方法之前引用 ablecloud/ACClient.php ，或者直接引用 bridge/ACBridgeWeChat.php （针对开发微信公众号后台应用的场景。该文件会引用 ablecloud/ACClient.php ），并且调用方法 ACClient\::init() 完成初始化。
 *
 *     该步骤执行的初始化设置包括 session_set_save_handler() 等。
 *
 *   + 访问外网
 *
 *     - 如果代码运行在AbleCloud云平台的托管环境中，需要通过AbleCloud云平台的代理服务器来访问外网。
 *
 *       AbleCloud云平台PHP运行环境支持使用PHP Curl访问外网，但要按如下代码配置代理服务器：
 *
 *       <PRE>
 *       $ch = curl_init();
 *       // ......
 *       $defaultProxy = 'http://xxx.xxx.xxx.xxx:1234'; // 若缺省条件下不使用代理，可以设置 $defaultProxy 为 NULL 。
 *       curl_setopt($ch, CURLOPT_PROXY, ACConfig::getProxyURL($defaultProxy));
 *       // ......
 *       curl_close($ch);
 *       </PRE>
 *
 *     - 如果代码不是运行在AbleCloud云平台的托管环境中，访问外网时不受上述条件限制。
 *
 *       此时，`ACConfig::getProxyURL($defaultProxy)` 将返回调用者设置的地址，因此上例中的代码在非AbleCloud云平台托管环境下也适用。
 */

require_once(dirname(__FILE__) . '/ACConfig.php');
require_once(dirname(__FILE__) . '/services/ACAccountMgr.php');
require_once(dirname(__FILE__) . '/services/ACBindMgr.php');
require_once(dirname(__FILE__) . '/services/ACFileMgr.php');
require_once(dirname(__FILE__) . '/services/ACOtaMgr.php');
require_once(dirname(__FILE__) . '/services/ACNotificationMgr.php');
require_once(dirname(__FILE__) . '/services/ACStore.php');
require_once(dirname(__FILE__) . '/services/ACTimerTaskMgr.php');
require_once(dirname(__FILE__) . '/services/ACAnalysisMgr.php');
require_once(dirname(__FILE__) . '/services/ACWarehouseMgr.php');
require_once(dirname(__FILE__) . '/services/ACProductMgr.php');
require_once(dirname(__FILE__) . '/services/ACFeedbackMgr.php');
require_once(dirname(__FILE__) . '/services/ACWeatherMgr.php');
require_once(dirname(__FILE__) . '/services/ACPHPSessionMgr.php');
require_once(dirname(__FILE__) . '/services/ACSMSMgr.php');
require_once(dirname(__FILE__) . '/services/ACOSSQueryMgr.php');

/**
 * AbleCloud服务的客户端。
 */
class ACClient
{
    /**
     * 通用的以HTTP POST方法访问AbleCloud云端服务（包括开发者自定义的服务）。
     * @param $request ACRequest ACRequest 对象。描述访问目标及参数。
     * @return ACResponse 返回 ACResponse 对象，表示远程服务的应答消息。
     */
    public static function sendToService($request)
    {
        return ACHttpClient::doRequest($request, ACClient::getContext());
    }

    /**
     * AbleCloud帐号服务。
     * @return ACAccountMgr 返回 ACAccountMgr 对象。
     */
    public static function getAccountMgr()
    {
        return new ACAccountMgr('zc-account', 1, ACClient::getContext());
    }

    /**
     * AbleCloud文件存储服务。
     * @return ACFileMgr    返回 ACFileMgr 对象。
     */
    public static function getFileMgr()
    {
        return new ACFileMgr('zc-blobstore', 1, ACClient::getContext());
    }

    /**
     * AbleCloud设备管理服务。
     * @return ACBindMgr 返回 ACBindMgr 对象。
     */
    public static function getBindMgr()
    {
        return new ACBindMgr('zc-bind', 1, ACClient::getContext());
    }

    /**
     * AbleCloud设备OTA服务。
     * @return ACOtaMgr 返回 ACOtaMgr 对象。
     */
    public static function getOtaMgr()
    {
        return new ACOtaMgr('zc-ota', 1, ACClient::getContext());
    }

    /**
     * AbleCloud消息推送服务。
     * @return ACNotificationMgr 返回 ACNotificationMgr 对象。
     */
    public static function getNotificationMgr()
    {
        return new ACNotificationMgr('zc-inform', 1, ACClient::getContext());
    }

    /**
     * AbleCloud短信服务。
     * @return ACSMSMgr 返回 ACSMSMgr 对象。
     */
    public static function getSMSMgr()
    {
        return new ACSMSMgr('zc-inform', 1, ACClient::getContext());
    }
    
    /**
     * AbleCloud数据存储服务。
     * @return ACStore 返回 ACStore 对象。
     */
    public static function getStore()
    {
        return new ACStore('zc-store', 1, ACClient::getContext());
    }

    /**
     * AbleCloud定时任务管理服务。
     * @return ACTimerTaskMgr 返回 ACTimerTaskMgr 对象。
     */
    public static function getTimerTaskMgr()
    {
        return new ACTimerTaskMgr('zc-timer-task', 1, ACClient::getContext());
    }

    /**
     * AbleCloud数分分析服务。
     * @return ACAnalysisMgr 返回 ACAnalysisMgr 对象。
     */
    public static function getAnalysisMgr()
    {
        return new ACAnalysisMgr('zc-queryengine', 1, ACClient::getContext());
    }

    /**
     * AbleCloud设备入库服务。
     * @return ACWarehouseMgr 返回 ACWarehouseMgr 对象。
     */
    public static function getWarehouseMgr()
    {
        return new ACWarehouseMgr('zc-warehouse', 1, ACClient::getContext());
    }

    /**
     * AbleCloud产品服务。
     * @return ACProductMgr 返回 ACProductMgr 对象。
     */
    public static function getProductMgr()
    {
        return new ACProductMgr('zc-product', 1, ACClient::getContext());
    }

    /**
     * AbleCloud用户反馈服务。
     * @return ACFeedbackMgr 返回 ACFeedbackMgr 对象。
     */
    public static function getFeedbackMgr()
    {
        return new ACFeedbackMgr('zc-feedback', 1, ACClient::getContext());
    }

    /**
     * AbleCloud天气服务。
     * @return ACWeatherMgr 返回 ACWeatherMgr 对象。
     */
    public static function getWeatherMgr()
    {
        return new ACWeatherMgr('zc-pm25', 1, ACClient::getContext());
    }

    /**
     * AbleCloud PHP Session服务。
     * @return ACPHPSessionMgr  返回 ACPHPSessionMgr 对象。
     */
    public static function getPHPSessionMgr()
    {
        return new ACPHPSessionMgr('zc-session', 1, ACClient::getContext());
    }

    /**
     * AbleCloud运营支持服务。
     * @return ACOSSQueryMgr    返回 ACOSSQueryMgr 对象。
     */
    public static function getOSSQueryMgr()
    {
        return new ACOSSQueryMgr('zc-oss-query', 1, ACClient::getContext());
    }

    /**
     * 取访问AbleCloud远程服务的环境信息。
     * @return ACContext 返回 ACContext 对象，表示访问AbleCloud远程服务的环境信息。
     */
    public static function getContext()
    {
        return new ACContext(
            ACConfig::getDeveloperId(),
            ACConfig::getAccessKey(),
            ACConfig::getSecretKey(),
            ACConfig::getMajorDomain(),
            ACConfig::getRuntimeMode(),
            ACConfig::getRouterUrl()
        );
    }

    /**
     * 初始化 ACClient 。配置好 ACConfig 之后再调用该方法完成初始化。
     */
    public static function init() {
        if (ACConfig::isACRuntime()) {
            // 注册session处理函数
            session_set_save_handler(ACClient::getPHPSessionMgr(), true);
        }
    }
}
