<?php

/**
 * AbleCloud服务配置信息。
 */
class ACConfig {
    /// 运行模式：test（测试模式）；production（生产模式）。
    public static $RuntimeMode = 'test';
    /// AbleCloud开发者帐号ID。整数。
    public static $DeveloperId = 0;
    /// 开发者的AK/SK密钥对中的AK。字符串。
    public static $AccessKey = '';
    /// 开发者的AK/SK密钥对中的SK。字符串。
    public static $SecretKey = '';
    /// 本地服务对应的主域的名字。
    public static $MajorDomain = '';
    /// AbleCloud远程服务的访问入口地址，如：http://test.ablecloud.cn:5000。
    public static $RouterUrl = 'http://test.ablecloud.cn:5000';
    /// 访问云端API的超时时长（单位为秒）。取值小于或等于0时忽略该值。
    public static $RequestTimeout = 300;

    /**
     * 检查当前运行环境是否为AbleCloud云端环境。
     * @return bool 返回TRUE表示是AbleCloud云端环境，否则表示其它环境。
     */
    public static function isACRuntime() {
        $v = getenv('WEB_RUNTIME');
        return (is_string($v) && strcmp($v, 'AC_PHP_ENGINE') == 0);
    }

    /**
     * 取运行模式：test（测试模式）；production（生产模式）。
     * @return string   运行模式。
     */
    public static function getRuntimeMode() {
        if (self::isACRuntime())
            return 'production';
        else
            return self::$RuntimeMode;
    }

    /**
     * 取AbleCloud开发者帐号ID。
     * @return int  开发者ID。
     */
    public static function getDeveloperId() {
        $v = getenv('zc_service_developer_id');
        if ($v === FALSE) {
            return self::$DeveloperId;
        } else {
            return intval($v);
        }
    }

    /**
     * 取开发者的AK/SK密钥对中的AK。
     * @return string   开发者的AK/SK密钥对中的AK。
     */
    public static function getAccessKey() {
        $v = getenv('zc_service_access_key');
        if ($v === FALSE) {
            return self::$AccessKey;
        } else {
            return strval($v);
        }
    }

    /**
     * 取开发者的AK/SK密钥对中的SK。
     * @return string   开发者的AK/SK密钥对中的SK。
     */
    public static function getSecretKey() {
        $v = getenv('zc_service_secret_key');
        if ($v === FALSE) {
            return self::$SecretKey;
        } else {
            return strval($v);
        }
    }

    /**
     * 取开发者的主域名。
     * @return string   开发者的主域名。
     */
    public static function getMajorDomain() {
        $v = getenv('zc_service_major_domain');
        if ($v === FALSE) {
            return self::$MajorDomain;
        } else {
            return strval($v);
        }
    }

    /**
     * 取AbleCloud远程服务的访问入口地址，如：http://test.ablecloud.cn:5000。
     * @return string   AbleCloud远程服务的访问入口地址。
     */
    public static function getRouterURL() {
        if (self::isACRuntime()) {
            $addresses = '';
            // 先从文件中获取地址集合
            $filePath = self::getACMetaInfoFilePath();
            if (file_exists($filePath)) {
                $c = file_get_contents($filePath);
                if ($c !== FALSE) {
                    $json = json_decode($c, true);
                    $addresses = $json['routerAddresses'];
                }
            }
            // 再从环境变量中取
            if ($addresses === FALSE || empty($addresses)) {
                $addresses = getenv('zc_router_addr');
            }
            // 从多个地址中随机选择一个
            if ($addresses === FALSE || empty($addresses)) {
                return self::$RouterUrl;
            } else {
                $urls = explode(',', $addresses);
                $n = count($urls);
                if ($n < 1) {
                    return self::$RouterUrl;
                } else {
                    $i = rand(0, $n - 1);
                    return $urls[$i];
                }
            }
        } else {
            return self::$RouterUrl;
        }
    }

    /**
     * 访问云端API的超时时长（单位为秒）。取值小于或等于0时忽略该值。
     * @return int      访问云端API的超时时长。
     */
    public static function getRequestTimeout() {
        return self::$RequestTimeout;
    }

    /**
     * 取AbleCloud云端环境正向代理服务器的地址。
     *
     * 代码运行在AbleCloud云端环境时需要通过代理服务器访问外部网络。该方法用于获取AbleCloud云端环境的代理服务器地址。
     * 同时，可通过参数 $default 设置缺省的代理服务器地址。在非AbleCloud云端环境下，该方法将直接返回该参数设置的地址。
     *
     * @param $default null|string  缺省的代理服务器地址。该地址应该包含代理服务器的端口号。缺省值为 NULL 。
     * @return null|string      正向代理服务器的地址。无需代理时返回NULL。
     */
    public static function getProxyURL($default = NULL) {
        if (self::isACRuntime()) {
            $addresses = '';
            // 先从文件中获取地址集合
            $filePath = self::getACMetaInfoFilePath();
            if (file_exists($filePath)) {
                $c = file_get_contents($filePath);
                if ($c !== FALSE) {
                    $json = json_decode($c, true);
                    $addresses = $json['proxyAddresses'];
                }
            }
            // 再从环境变量中取
            if ($addresses === FALSE || empty($addresses)) {
                $addresses = getenv('zc_proxy_addr');
            }
            // 从多个地址中随机选择一个
            if ($addresses === FALSE || empty($addresses)) {
                return $default;
            } else {
                $urls = explode(',', $addresses);
                $n = count($urls);
                if ($n < 1) {
                    return $default;
                } else {
                    $i = rand(0, $n - 1);
                    return $urls[$i];
                }
            }
        } else {
            return $default;
        }
    }

    /**
     * 取AC-MetaInfo文件的路径。
     * @return string
     */
    private static function getACMetaInfoFilePath() {
        return getenv('META_FILE_PATH') . '/ac-metaInfo';
    }
}
