<?php

require_once(dirname(__FILE__) . '/ACRequest.php');
require_once(dirname(__FILE__) . '/ACResponse.php');
require_once(dirname(__FILE__) . '/ACDeveloperSignature.php');
require_once(dirname(__FILE__) . '/ACUserSignature.php');

/**
 * 访问AbleCloud远程服务的HTTP客户端。
 */
class ACHttpClient
{
    /**
     * 访问AbleCloud远程服务。
     * @param $request    ACRequest    访问AbleCloud服务的请求信息。
     * @param $context    ACContext    访问AbleCloud服务的环境信息。
     * @return            ACResponse    返回一个ACResponse对象，表示远程服务的响应消息。
     */
    public static function doRequest($request, $context)
    {
        $url = self::makeACURL($request, $context);
        $headers = self::makeHeaders($request, $context);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $timeout = intval(ACConfig::getRequestTimeout());
        if ($timeout > 0) {
            curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
        }
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $request->getPayload());
        curl_setopt($ch, CURLOPT_HEADER, true);    // 获取响应消息的头信息。

        $response = curl_exec($ch);
        if (curl_errno($ch) == 0) {
            $info = curl_getinfo($ch);
            $rawHeader = trim(substr($response, 0, $info['header_size']));
            if ($info['header_size'] < strlen($response))
                $rawBody = substr($response, $info['header_size']);
            else
                $rawBody = '';
            $header = self::http_parse_headers($rawHeader);
            if (isset($header['X-Zc-Msg-Name']) && $header['X-Zc-Msg-Name'] === 'X-Zc-Ack') {
                $response = new ACResponse(0, $rawBody);
            } else {
                $body = NULL;
                if (!is_null($rawBody)) {
                    $body = json_decode($rawBody, true);
                }
                if (is_array($body) && is_numeric($body['errorCode'])) {
                    $errorCode = intval($body['errorCode']);
                    $errorMessage = '';
                    if (array_key_exists('error', $body)) {
                        $errorMessage = strval($body['error']);
                    }
                    if (array_key_exists('description', $body)) {
                        $errorMessage .= ' - ' . strval($body['description']);
                    }
                    $response = new ACResponse($errorCode, $errorMessage);
                } else {
                    $response = new ACResponse(-1, $rawBody);
                }
            }
        } else {
            $response = new ACResponse(-1, curl_error($ch));
        }

        curl_close($ch);

        return $response;
    }

    /**
     * 计算随机字符串。
     * @param $len    int    表示要生成的字符串的长度。
     * @return        string    返回长度为$len的随机字符串。
     */
    public static function nonce($len)
    {
        $c = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        $size = strlen($c);
        $str = '';
        srand((double)microtime() * 1000000);
        for ($i = 0; $i < $len; $i++) {
            $str .= $c[rand() % $size];
        }
        return $str;
    }

    /**
     * 计算AbleCloud服务的访问地址。
     * @param $request    ACRequest    是访问AbleCloud服务的请求消息。
     * @param $context    ACContext    是访问AbleCloud服务的环境信息。
     * @return            ACResponse    返回对应的AbleCloud服务的访问地址。
     */
    public static function makeACURL($request, $context)
    {
        $url = $context->getRouterAddress() . '/' . $request->getServiceName() . '/v' . $request->getServiceVersion() . '/' . $request->getMethodName();
        $keys = $request->getParameterKeys();
        foreach ($keys as $index => $key) {
            if ($index == 0)
                $url .= '?';
            else
                $url .= '&';
            $val = $request->getParameterValue($key);
            if (is_array($val)) {
                foreach ($val as $i => $v) {
                    if ($i > 0)
                        $url .= '&';
                    $url .= $key . '[]=' . $v;
                }
            } else {
                $url .= $key . '=' . $val;
            }
        }
        return $url;
    }

    /**
     * 计算访问AbleCloud云端服务时HTTP请求的头信息。
     * @param $request    ACRequest    表示访问AbleCloud服务的请求消息。
     * @param $context    ACContext    访问AbleCloud服务的环境信息。
     * @return            array        返回包含了HTTP请求头信息的数组。
     */
    protected static function makeHeaders($request, $context)
    {
        // 一般头信息及开发者签名
        $contentType = $request->getPayloadFormat();
        $contentLength = $request->getPayloadSize();
        $nonce = self::nonce(16);
        $timeout = 60;
        $default_timezone = date_default_timezone_get();
        date_default_timezone_set("UTC");
        $timestamp = time();    // in second
        date_default_timezone_set($default_timezone);
        $subDomain = $request->getSubDomain();
        $developer = $context->getDeveloper();
        $majorDomain = $context->getMajorDomain();
        $traceId = $request->getTraceId();
        $developerSignature = ACDeveloperSignature::signature($developer, $context, $subDomain, $request->getMethodName(), $timestamp, $timeout, $nonce);
        $headers = array(
            "Content-type: $contentType",
            "Content-Length: $contentLength",
            "X-Zc-Major-Domain: $majorDomain",
            "X-Zc-Sub-Domain: $subDomain",
            "X-Zc-Timestamp: $timestamp",
            "X-Zc-Timeout: $timeout",
            "X-Zc-Nonce: $nonce",
            "X-Zc-Developer-Id: " . $developer->getId(),
            "X-Zc-Access-Key: " . $developer->getAccessKey(),
            "X-Zc-Developer-Signature: $developerSignature",
            "X-Zc-Trace-Id: $traceId"
        );
        // X-Zc-Device-OS, X-Zc-Device-Version, X-Zc-Device-Name, X-Zc-Device-ID
        $defaultHeaderValue = "unknown";
        $hasDeviceOS = FALSE;
        $hasDeviceVersion = FALSE;
        $hasDeviceName = FALSE;
        $hasDeviceID = FALSE;
        foreach ($context->getExtHttpHeaders() as $key => $val) {
            array_push($headers, strval($key) . ': ' . strval($val));
            if (!$hasDeviceOS && strcasecmp($key, 'X-Zc-Device-OS') == 0) {
                $hasDeviceOS = TRUE;
            }
            if (!$hasDeviceVersion && strcasecmp($key, 'X-Zc-Device-Version') == 0) {
                $hasDeviceVersion = TRUE;
            }
            if (!$hasDeviceName && (strcasecmp($key, 'X-Zc-Device-Name') == 0 || strcasecmp($key, 'X-Zc-Operation-Type') == 0)) {
                $hasDeviceName = TRUE;
            }
            if (!$hasDeviceID && strcasecmp($key, 'X-Zc-Device-ID') == 0) {
                $hasDeviceID = TRUE;
            }
        }
        if (!$hasDeviceName) {
            // 尝试从$_SERVER['HTTP_USER_AGENT']中提取信息：微信，操作系统
            $agentInfo = $_SERVER['HTTP_USER_AGENT'];
            // 微信
            $pos = strripos($agentInfo, 'MicroMessenger');
            if ($pos !== FALSE) {
                array_push($headers, "X-Zc-Device-Name: weixin");
                array_push($headers, "X-Zc-Operation-Type: weixin");
                $wechatInfo = substr($agentInfo, $pos);
                $s = explode('/', $wechatInfo);
                if (!$hasDeviceVersion && is_array($s) && count($s) >= 2) {
                    array_push($headers, "X-Zc-Device-Version: " . $s[1]);
                    $hasDeviceVersion = TRUE;
                }
            }
        }
        if (!$hasDeviceVersion) {
            array_push($headers, "X-Zc-Device-Version: " . $defaultHeaderValue);
        }
        if (!$hasDeviceID) {
            array_push($headers, "X-Zc-Device-ID: " . $defaultHeaderValue);
        }
        if (!$hasDeviceOS) {
            // 操作系统
            // 尝试从$_SERVER['HTTP_USER_AGENT']中提取信息：操作系统
            $agentInfo = $_SERVER['HTTP_USER_AGENT'];
            if (stripos($agentInfo, 'android') !== FALSE) {
                array_push($headers, "X-Zc-Device-OS: android");
            } else if (stripos($agentInfo, 'iphone') !== FALSE) {
                array_push($headers, "X-Zc-Device-OS: ios");
            } else {
                array_push($headers, "X-Zc-Device-OS: " . $defaultHeaderValue);
            }
        }
        // 非开发者用户签名
        $user = $request->getUser();
        if ($user != NULL) {
            $userSignature = ACUserSignature::signature($user, $timestamp, $timeout, $nonce);
            $extHeaders = array(
                "X-Zc-User-ID: " . $user->getId(),
                "X-Zc-User-Signature: $userSignature"
            );
            $headers = array_merge($headers, $extHeaders);
        }

        return $headers;
    }

    /**
     * 解析HTTP响应消息的头信息。
     * @param $raw_headers    string    待解析的原始HTTP响应头信息。
     * @return                array    返回包含头信息的数组。
     */
    protected static function http_parse_headers($raw_headers)
    {
        $headers = array();
        $key = '';
        foreach (explode("\n", $raw_headers) as $i => $h) {
            $h = explode(':', $h, 2);
            if (isset($h[1])) {
                if (!isset($headers[$h[0]]))
                    $headers[$h[0]] = trim($h[1]);
                elseif (is_array($headers[$h[0]])) {
                    $headers[$h[0]] = array_merge($headers[$h[0]], array(trim($h[1])));
                } else {
                    $headers[$h[0]] = array_merge(array($headers[$h[0]]), array(trim($h[1])));
                }
                $key = $h[0];
            } else {
                if (substr($h[0], 0, 1) == "\t")
                    $headers[$key] .= "\r\n\t" . trim($h[0]);
                elseif (!$key)
                    $headers[0] = trim($h[0]);
                trim($h[0]);
            }
        }
        return $headers;
    }
}