<?php

/**
 * 访问AbleCloud远程服务的请求消息。
 */
class ACRequest
{
    /**
     * 构造函数。
     * @param $serviceName        string    是拟访问的远程服务的名字。
     * @param $methodName        string    是拟访问的远程服务的方法的名字。
     * @param $serviceVersion    int        是拟访问的远程服务的主版本号。
     * @param $subDomain        string    是拟访问的远程服务所属的子域的名字。缺省值为空字符串，表示访问主域级别的服务。不为空字符串时，表示访问该子域所对应的服务。
     */
    function __construct($serviceName, $methodName, $serviceVersion, $subDomain = '') {
        $this->serviceName = $serviceName;
        $this->serviceVersion = $serviceVersion;
        $this->methodName = $methodName;
        $this->subDomain = empty($subDomain) ? '' : strval($subDomain);
        $this->parameters = array();
        $this->traceId = ACRequest::newTraceId();
    }

    /**
     * 设置本次请求所关联的用户。该用户是AbleCloud平台中开发者所提供服务的用户。
     * @param $user ACUser|NULL    AbleCloud平台中开发者所提供服务的用户。NULL表示清除设置的用户信息。
     */
    public function setUser($user) {
        $this->user = $user;
    }

    /**
     * 取当前设置的用户信息。
     * @return ACUser|NULL    返回NULL或ACUser对象。
     */
    public function getUser() {
        return $this->user;
    }

    /**
     * 取拟访问的远程服务的名字。
     * @return string    返回拟访问的远程服务的名字。
     */
    public function getServiceName() {
        return $this->serviceName;
    }

    /**
     * 取拟访问的远程服务的主版本号。
     * @return int    返回拟访问的远程服务的主版本号。
     */
    public function getServiceVersion() {
        return $this->serviceVersion;
    }

    /**
     * 取拟访问的远程服务的方法名。
     * @return string    返回拟访问的远程服务的方法名。
     */
    public function getMethodName() {
        return $this->methodName;
    }

    /**
     * 取拟访问的远程服务所属的子域的名字。
     * @return string     回拟访问的远程服务所属的子域的名字。
     */
    public function getSubDomain() {
        return $this->subDomain;
    }

    /**
     * 添加请求的参数。这些参数是键值对，将会以查询字符串的方式置于访问远程服务的URL中传递给远程服务。
     * @param $key        string    参数的名字。
     * @param $value    mixed    参数的值。对同名的参数多次赋值时，取最后一次设置的值。
     */
    public function addParameter($key, $value) {
        $this->parameters[$key] = $value;
    }

    /**
     * 取设置的所有参数的名字。
     * @return array    返回参数名字所形成的数组。
     */
    public function getParameterKeys() {
        $keys = array();
        foreach ($this->parameters as $k => $v)
            array_push($keys, $k);
        return $keys;
    }

    /**
     * 取参数的值。
     * @param $key    string        参数的名字。
     * @return        mixed|NULL    返回指定参数的值。参数不存在时返回NULL。
     */
    public function getParameterValue($key) {
        return $this->parameters[$key];
    }

    /**
     * 消息内容要么是JSON格式的要么是二进制格式的。取最后一次设置的内容为最终内容。本方法用于设置JSON格式的消息内容。
     * @param $payload        string    JSON格式的内容。
     * @param $contentType    string    HTTP请求的Content-Type的值，缺省为'application/x-zc-object'。
     */
    public function setPayloadAsJSON($payload, $contentType = 'application/x-zc-object') {
        $this->payloadFormat = $contentType;
        $this->payload = $payload;
    }

    /**
     * 消息内容要么是JSON格式的要么是二进制格式的。取最后一次设置的内容为最终内容。本方法用于设置二进制格式的消息内容。
     * @param $payload string    string类型的数据，表示二进制格式的内容。
     */
    public function setPayloadAsStream($payload) {
        $this->payloadFormat = 'application/octet-stream';
        $this->payload = $payload;
    }

    /**
     * 取请求所包含的数据的格式：application/x-zc-object或application/octet-stream，分别表示JSON格式和二进制数据格式。
     * @return string    返回请求所包含的数据的格式。
     */
    public function getPayloadFormat() {
        return $this->payloadFormat;
    }

    /**
     * 取请求所包含数据的大小。
     * @return int    返回请求所包含数据的字节数。
     */
    public function getPayloadSize() {
        if (is_string($this->payload))
            return strlen($this->payload);
        else
            return 0;
    }

    /**
     * 取设置的请求所包含的数据。
     * @return string    返回以string数据类型表示的数据。
     */
    public function getPayload() {
        return $this->payload;
    }

    /**
     * 取当前的Trace-Id。
     * @return string    当前的Trace-Id。
     */
    public function getTraceId() {
        return $this->traceId;
    }

    /**
     * 设置Trace-Id。
     * @param $traceId string    是符合RFC 4122 - Section 4.4规范的UUID的十六进制表示形式，用来作为请求的Trace-Id。
     */
    public function setTraceId($traceId)
    {
        $this->traceId = $traceId;
    }

    /**
     * 生成一个Trace-ID。
     * @return string    是符合RFC 4122 - Section 4.4规范的UUID的十六进制表示形式。
     */
    public static function newTraceId() {
        $data = openssl_random_pseudo_bytes(16);
        // RFC 4122 - Section 4.4
        $data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
        $data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10
        return bin2hex($data);
    }

    private $user = NULL;        ///< 发起访问的用户。
    private $serviceName;        ///< 拟访问的远程服务的名字。
    private $serviceVersion;    ///< 拟访问的远程服务的主版本号。
    private $methodName;        ///< 拟访问的远程服务的方法名。
    private $subDomain;            ///< 拟访问的远程服务所属的子域的名字。
    private $parameters;        ///< 访问远程服务时使用的参数。
    private $payloadFormat = 'application/x-zc-object';        ///< 请求所包含的数据的格式：application/x-zc-object或application/octet-stream。分别表示JSON格式和二进制数据格式。
    private $payload = NULL;    ///< 请求所包含的数据。
    private $traceId;           ///< Trace-ID。
}
