<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACUser.php');

/**
 * AbleCloud帐号服务。
 */
class ACAccountMgr extends ACService {
	/**
	 * 构造函数。
	 * @param $name		string		AbleCloud帐号服务的名字。
	 * @param $version	int			AbleCloud帐号服务的主版本号。
	 * @param $context	ACContext	表示访问该远程服务所依赖的环境信息。
	 */
	function __construct($name, $version, $context) {
		parent::__construct($name, $version, $context);
	}

	/// @name 用户帐号注册与登录。
	//@{
	/**
	 * 获取用户注册的验证码。
	 * @param $account  string  是用户的登录名：email地址或手机号。
	 * @param $timeout  int     是验证码的有效时长。单位为秒。
	 * @return          string  操作成功时返回字符串形式的验证码。操作失败时返回空字符串，并且可以调用getLastError()方法获取错误信息。
	 */
	public function getVerifyCode($account, $timeout) {
		$request = new ACRequest($this->getName(), 'getVerifyCode', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('account' => $account, 'timeout' => intval($timeout))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['verifyCode'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 向指定的手机号或电子邮箱发送验证码。
	 * @param $account	string	接收验证码的手机号或电子邮箱地址。
	 * @param $template	int		是短信模板的代码。
	 * @param $timeout	int		是验证码的有效时长，单位是秒。
	 * @return			bool	操作成功时返回TRUE；操作失败时返回FALSE，并且可调用getLastError()后去错误信息。
	 */
	public function sendVerifyCode($account, $template, $timeout) {
		$request = new ACRequest($this->getName(), 'sendVerifyCode', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('account' => $account, 'template' => intval($template), 'timeout' => intval($timeout))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 检查验证码是否有效。
	 * @param $account		string	与验证码匹配的用户帐号：Email地址或者手机号码。
	 * @param $verifyCode	string	待检查的验证码。
	 * @return 				bool	返回TRUE表示验证码有效，返回FALSE表示无效。返回NULL表示操作失败，此时可调用方法getLastError()获取错误信息。
	 */
	public function checkVerifyCode($account, $verifyCode) {
		$request = new ACRequest($this->getName(), 'checkVerifyCode', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('account' => $account, 'verifyCode' => $verifyCode)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return boolval($json['valid']);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 注册用户帐号。
	 * @param $name			string	用户的显示名。
	 * @param $email		string	新用户的邮箱。$email与$phone不能都为空字符串。
	 * @param $phone		string	新用户的手机号码。$email与$phone不能都为空字符串。
	 * @param $password		string	新用户的登录密码。
	 * @param $verifyCode	string	注册新用户时所使用的验证码。
	 * @param $enableTokenExpiration	bool	是否使用用户TOKEN过期的机制。缺省为不使用。
	 * @return				ACUser|NULL	用户注册成功时返回一个ACUser对象，表示新用户的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function register($name, $email, $phone, $password, $verifyCode, $enableTokenExpiration = FALSE) {
		$request = new ACRequest($this->getName(), 'register', $this->getVersion());
		$data = array('email' => $email, 'phone' => $phone, 'password' => $password, 'verifyCode' => $verifyCode, 'nickName' => $name);
		$request->setPayloadAsJSON(json_encode($data));
		if ($enableTokenExpiration) {
			$this->getContext()->addExtHttpHeader('X-ZC-Version', '1');
		}
		$response = ACHttpClient::doRequest($request, $this->getContext());
		$this->getContext()->addExtHttpHeader('X-ZC-Version', '');
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return new ACUser(intval($json['userId']),
				$json['token'],
				$json['nickName'],
				array_key_exists('refreshToken', $json) ? $json['refreshToken'] : '',
				array_key_exists('tokenExpire', $json) ? $json['tokenExpire'] : '',
				array_key_exists('refreshTokenExpire', $json) ? $json['refreshTokenExpire'] : '',
				$email,
				$phone
			);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 按登录名和密码登录，取用户的信息。
	 * @param $account	string		用户的登录名。
	 * @param $password	string		用户登录密码。
	 * @param $enableTokenExpiration	bool	是否使用用户TOKEN过期的机制。缺省为不使用。
	 * @return			ACUser|NULL	返回一个ACUser对象，表示该用户的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function login($account, $password, $enableTokenExpiration = FALSE) {
		$request = new ACRequest($this->getName(), 'login', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('account' => $account, 'password' => $password)));
		if ($enableTokenExpiration) {
			$this->getContext()->addExtHttpHeader('X-ZC-Version', '1');
		}
		$response = ACHttpClient::doRequest($request, $this->getContext());
		$this->getContext()->addExtHttpHeader('X-ZC-Version', '');
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return new ACUser(intval($json['userId']),$json['token'], $json['nickName'],
				array_key_exists('refreshToken', $json) ? $json['refreshToken'] : '',
				array_key_exists('tokenExpire', $json) ? $json['tokenExpire'] : '',
				array_key_exists('refreshTokenExpire', $json) ? $json['refreshTokenExpire'] : '',
				$json['email'],
				$json['phone']);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 按登录名和密码取用户的信息。
	 * @param $account	string		用户的登录名。
	 * @param $password	string		用户登录密码。
	 * @param $enableTokenExpiration	bool	是否使用用户TOKEN过期的机制。缺省为不使用。
	 * @return			ACUser|NULL	返回一个ACUser对象，表示该用户的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function getUser($account, $password, $enableTokenExpiration = FALSE) {
		return $this->login($account, $password, $enableTokenExpiration);
	}

	/**
	 * 根据用户的Id查找用户信息。
	 * @param $userId	int		用户的ID。
	 * @return			ACUser | NULL	返回一个ACUser对象，表示该用户的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function getAccountInfo($userId) {
		$request = new ACRequest($this->getName(), 'getAccount', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('userId' => intval($userId))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			if (is_array($json) && array_key_exists('account', $json)) {
				$obj = $json['account'];
				return new ACUser(intval($userId), '', $obj['nickName'], '', '', '', $obj['email'], $obj['phone']);
			} else {
				$this->resetLastError(-1, 'Unknown response from getProfilesByUserList: ' . json_encode($json));
				return NULL;
			}
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 根据电话或Email查找用户ID。
	 * @param $account	string	待查询的用户的帐号：Email或者电话号码。
	 * @return			int		查询成功时返回的用户ID为正整数。返回值小于0时表示查询失败，并且可调用getLastError()方法获取错误信息。
	 */
	public function getIdByAccount($account) {
		$request = new ACRequest($this->getName(), 'getIdByAccount', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('account' => $account)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			if (is_array($json) && array_key_exists('userId', $json)) {
				return intval($json['userId']);
			} else {
				$this->resetLastError(-1, 'Unknown response from getProfilesByUserList: ' . json_encode($json));
				return -1;
			}
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return -1;
		}
	}

	/**
	 * 检查帐号是否已经存在。
	 * @param $login	string	表示用户的登录名，如邮箱或者手机号。
	 * @return			bool	返回TRUE表示存在该帐号；不存在该帐号时返回FALSE；操作失败时也返回FALSE。
	 * 							返回值为FALSE时，应该调用getLastError()方法获取错误信息，并检查其errCode值：errCode为0时，表示操作成功；否则表示操作失败。
	 */
	public function checkAccountExist($login) {
		$request = new ACRequest($this->getName(), 'checkExist', $this->getVersion());
		$request->addParameter('account', $login);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['exist'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 检查用户的总数。
	 * @return int	返回用户总数。返回值小于0时表示操作失败，可调用getLastError()方法获取错误消息。
	 */
	public function getAccountCount() {
		$request = new ACRequest($this->getName(), 'getAccountCount', $this->getVersion());
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return intval($json['count']);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return -1;
		}
	}

	/**
	 * 查询用户列表。
	 * @param $offset	int	查询的记录偏移量。取值应该为非负整数。
	 * @param $limit	int	限制本次调用查询的记录的最大数目。取值范围是闭区间[1, 100]。
	 * @return			array	成功时返回ACUser对象的数组。失败时返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function listAllAccounts($offset, $limit) {
		$request = new ACRequest($this->getName(), 'listAllAccounts', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('offset' => $offset, 'limit' => $limit)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$users = array();
			foreach ($json['accounts'] as $k => $v) {
				$u = new ACUser(intval($v['userId']), '', $v['nickName'], '', '', '', $v['email'], $v['phone']);
				array_push($users, $u);
			}
			return $users;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 依据用户的ID批量获取用户的帐号信息。
	 * @param $userIds		array	拟查询的用户ID的列表。
	 * @return 				array	成功时返回ACUser对象的数组。失败时返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function getAccountsByIds($userIds) {
		$request = new ACRequest($this->getName(), 'getAccountsByIds', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('userIds' => $userIds)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$users = array();
			foreach ($json['accounts'] as $k => $v) {
				$u = new ACUser(intval($v['userId']), '', $v['nickName'], '', '', '', $v['email'], $v['phone']);
				array_push($users, $u);
			}
			return $users;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}
	//@}

	/// @name 第三方平台用户
	//@{
	/**
	 * 使用第三方帐号注册用户。
	 * @param $openId	string		第三方帐号的OpenID。
	 * @param $provider	string		第三方帐号的来源。如"weixin"。
	 * @param $unionId	string		对来自微信平台的用户，是其在微信平台对应的UnionID。如果不提供该参数，则无法识别同一个用户关注开发者的多个微信公众号的情况。
	 * @param $enableTokenExpiration	bool	是否使用用户TOKEN过期的机制。缺省为不使用。
	 * @return 			ACUser|NULL	注册成功后返回一个ACUser对象，表示新用户的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function registerByOpenId($openId, $provider, $unionId = '', $enableTokenExpiration = FALSE) {
		return $this->getUserByOpenId($openId, $provider, $unionId, $enableTokenExpiration);
	}

	/**
	 * 按OpenID和帐号来源取用户的信息。
	 * @param $openId 	string		用户的OpenID。
	 * @param $provider	string		用户的来源。如"weixin"等。
	 * @param $unionId	string		对来自微信平台的用户，是其在微信平台对应的UnionID。
	 * @param $enableTokenExpiration	bool	是否使用用户TOKEN过期的机制。缺省为不使用。
	 * @return			ACUser|NULL	返回一个ACUser对象，表示该用户的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function getUserByOpenId($openId, $provider, $unionId = '', $enableTokenExpiration = FALSE) {
		$request = new ACRequest($this->getName(), 'loginWithOpenId', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('openId' => $openId, 'provider' => $provider, 'unionId' => $unionId)));
		if ($enableTokenExpiration) {
			$this->getContext()->addExtHttpHeader('X-ZC-Version', '1');
		}
		$response = ACHttpClient::doRequest($request, $this->getContext());
		$this->getContext()->addExtHttpHeader('X-ZC-Version', '');
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$user = new ACUser((int)$json['userId'], $json['token'], $json['nickName'], '', '', '', $json['email'], $json['phone']);
			if (array_key_exists('tokenExpire', $json)) {
				$user->setToken($json['token'], $json['tokenExpire']);
			}
			if (array_key_exists('refreshToken', $json) && array_key_exists('refreshTokenExpire', $json)) {
				$user->setRefreshToken($json['refreshToken'], $json['refreshTokenExpire']);
			}
			return $user;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}
	
	public function getAccountByOpenId($openId, $provider,$enableTokenExpiration = FALSE)
	{
		  $request = new ACRequest($this->getName(), 'getAccountByOpenId', $this->getVersion());
	    $request->setPayloadAsJSON(json_encode(array('openId' => $openId, 'provider' => $provider, 'unionId' => "")));
	  	if ($enableTokenExpiration) {
			$this->getContext()->addExtHttpHeader('X-ZC-Version', '1');
		}
		$response = ACHttpClient::doRequest($request, $this->getContext());
		$this->getContext()->addExtHttpHeader('X-ZC-Version', '');
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$user = new ACUser((int)$json['userId'], '', $json['nickName'], '', '', '', $json['email'], $json['phone']);
			return $user;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	    /*
	       resp.PutInt("userId", account.Uid())resp.PutString("nickName", account.NickName())resp.PutString("email", account.Email())resp.PutString("phone", account.Phone())
	    */
	   // $user = new ACUser((int)$json['userId'], $json['token'], $json['nickName'], '', '', '', $json['email'], $json['phone']);
	}
	
	

	/**
	 * 获取用户在第三方平台上的OpenID。
	 * @param $userId	int		是用户在AbleCloud平台上的ID。
	 * @param $provider	string	标识第三方平台。如'weixin'表示微信平台。
	 * @return			string	返回一个字符串，表示用户在指定的第三方平台中对应的OpenID。如果返回值为空字符串，表示操作失败。此时，可调用getLastError()方法获取错误信息。
	 */
	public function getUserOpenId($userId, $provider) {
		$request = new ACRequest($this->getName(), 'listAllOpenIds', $this->getVersion());
		$request->setUser(new ACUser($userId, ''));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			foreach ($json['openIds'] as $v) {
				if (strcasecmp($v['provider'], $provider) == 0) {
					return $v['openId'];
				}
			}
			$this->resetLastError(-1, 'Can not find OpenID from the provider.');
			return '';
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return '';
		}
	}
	//@}

	/**
	 * 修改用户的手机号码。
	 * @param $user			ACUser	是要修改手机号的用户。
	 * @param $phone		string	字符串，是用户的新手机号码。
	 * @param $verifyCode	string	是验证码。
	 * @param $password		string	是用户的密码。
	 * @return				bool	操作成功返回TRUE；操作失败则返回FALSE，并且可调用getLastError()方法获取错误信息。
	 */
	public function changePhone($user, $phone, $verifyCode, $password) {
		$request = new ACRequest($this->getName(), 'changePhone', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('phone' => $phone, 'password' => $password, 'verifyCode' => $verifyCode, 'backup' => false)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 修改用户显示名称。
	 * @param $user			ACUser	是要修改显示名的用户。
	 * @param $nickName		string	用户的新名字。
	 * @return				bool	操作成功返回TRUE；操作失败则返回FALSE，并且可调用getLastError()方法获取错误信息。
	 */
	public function changeNickName($user, $nickName) {
		$request = new ACRequest($this->getName(), 'changeNickName', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('nickName' => $nickName)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 更新用户的Token。
	 * @param $user	ACUser	要更新其Token的用户。操作成功后直接更新该对象保存的信息。
	 * @return		bool	返回TRUE表示操作成功，并且会更新$user对象的信息；返回FALSE表示操作失败，可以调用getLassError()获取错误信息。
	 */
	public function updateUserToken($user) {
		$oldToken = $user->getToken();
		$oldTokenExpiration = $user->getTokenExpiration();
		$refreshToken = $user->getRefreshToken();
		$refreshTokenExpiration = $user->getRefreshTokenExpiration();
		$user->setToken($refreshToken, $refreshTokenExpiration);
		$request = new ACRequest($this->getName(), 'updateAccessToken', $this->getVersion());
		$request->setUser($user);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$user->setToken($json['token'], $json['tokenExpire']);
			return TRUE;
		} else {
			$user->setToken($oldToken, $oldTokenExpiration);
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * @deprecated 请使用resetPasswordByAccount方法。
	 *
	 * 重设用户的密码。
	 * @param $userId		int		要更新其密码的用户的ID。
	 * @param $account		string	要更新其密码的用户的登录名。
	 * @param $password		string	用户的新密码。
	 * @param $verifyCode	string	更新用户密码的验证码。
	 * @return				bool	操作成功返回TRUE；操作失败则返回FALSE，并且可调用getLastError()方法获取错误信息。
	 */
	public function resetPassword($userId, $account, $password, $verifyCode) {
		$request = new ACRequest($this->getName(), 'resetPassword', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('userId' => $userId, 'account' => $account, 'password' => $password, 'verifyCode' => $verifyCode)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 重设用户的密码。
	 * @param $account		string	要更新其密码的用户的登录名。
	 * @param $password		string	用户的新密码。
	 * @param $verifyCode	string	更新用户密码的验证码。
	 * @return				bool	操作成功返回TRUE；操作失败则返回FALSE，并且可调用getLastError()方法获取错误信息。
	 */
	public function resetPasswordByAccount($account, $password, $verifyCode) {
		return $this->resetPassword(0, $account, $password, $verifyCode);
	}

	/// @name 用户的扩展属性
	//@{
	/**
	 * 设置用户的扩展属性。
	 * @param $user		ACUser	ACUser对象，表示要设置其扩展属性的用户。
	 * @param $profile	array	是由键值对组成的关联数组，表示用户的扩展属性值。
	 * @return			bool	操作成功时返回TRUE，否则返回FALSE。操作失败时可以调用getLastError()方法获取错误信息。
	 */
	public function setUserProfile($user, $profile) {
		$request = new ACRequest($this->getName(), 'setUserProfile', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('userProfile' => $profile)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 获取用户的扩展属性。
	 * @param $user	ACUser		表示要获取其扩展属性的用户。
	 * @return		array|NULL	操作成功时返回一个由键值对组成的关联数组，表示该用户的扩展属性。操作失败时返回NULL，并且可调用getLastError()方法获取错误信息。
	 */
	public function getUserProfile($user) {
		$request = new ACRequest($this->getName(), 'getUserProfile', $this->getVersion());
		$request->setUser($user);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['userProfile'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 获取用户的全部属性：包括基础属性及扩展属性。
	 * @param $uid		int		表示要获取其属性的用户的ID。参数$uid和$account至少需提供一个。
	 * @param $account	string	表示要获取其属性的用户的帐号名：Email地址或者电话。参数$uid和$account至少需提供一个。
	 * @return			array|NULL	操作成功时返回一个由键值对组成的关联数组，表示该用户的属性。操作失败时返回NULL，并且可调用getLastError()方法获取错误信息。
	 */
	public function getUserWholeProfile($uid = 0, $account = '') {
		$request = new ACRequest($this->getName(), 'getWholeProfile', $this->getVersion());
		$data = array('userId' => $uid, 'account' => $account);
		$request->setPayloadAsJSON(json_encode($data));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['userProfile'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 根据用户uid列表查找用户的信息（基本信息+扩展信息）。一次最多可查询1000个用户的信息。
	 * @param $userIds	array	待查询的用户ID的数组。元素个数必须小于等于1000。
	 * @return			array	操作成功时返回用户属性数据的数组。数组中的每个元素是一个关联数组，对应于一个用户的属性。操作失败时返回NULL，并且可调用getLastError()方法获取错误信息。
	 */
	public function getProfilesByUserList($userIds) {
		$request = new ACRequest($this->getName(), 'getProfilesByUserList', $this->getVersion());
		$data = array('userList' => $userIds);
		$request->setPayloadAsJSON(json_encode($data));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$json = json_decode($response->getResponse(), true);
			if (is_array($json) && array_key_exists('userProfiles', $json)) {
				$this->resetLastError();
				return $json['userProfiles'];
			} else {
				$this->resetLastError(-1, 'Unknown response from getProfilesByUserList: ' . json_encode($json));
				return NULL;
			}
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}
	//@}

	/**
	 * 清除已注册的帐号信息。仅测试环境支持该方法。
	 * @return bool 操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function clearUsers() {
		if ($this->getContext()->isProductionMode()) {
			$this->resetLastError(-1, 'It is not allowed in production environment.');
            return FALSE;
		}
		$request = new ACRequest($this->getName(), 'cleanAllData', $this->getVersion());
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}
}