<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACDevice.php');
require_once(dirname(__FILE__) . '/ACHome.php');
require_once(dirname(__FILE__) . '/ACUser.php');
require_once(dirname(__FILE__) . '/ACDeviceShareCode.php');

/**
 * AbleCloud设备绑定管理服务。
 */
class ACBindMgr extends ACService {
	/**
	 * 构造函数。
	 * @param $name		string		AbleCloud设备绑定管理服务的名字。
	 * @param $version	int			AbleCloud设备绑定管理服务的版本。
	 * @param $context	ACContext	ACContext对象，表示访问该远程服务所依赖的环境信息。
	 */
	function __construct($name, $version, $context) {
		parent::__construct($name, $version, $context);
	}

	/// @name 独立设备
	//@{
	/**
	 * 将设备绑定至指定的用户。
	 * @param $subDomain	string	是要被绑定的设备所属的子域的名字。
	 * @param $physicalId	string	要被绑定的设备的物理ID。
	 * @param $name			string	设备的名字。
	 * @param $user			ACUser	表示要被绑定的用户。
	 * @return 				ACDevice	操作成功返回ACDevice对象，表示被绑定的设备的信息。操作失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function bindDevice($subDomain, $physicalId, $name, $user) {
		$request = new ACRequest($this->getName(), 'bindDevice', $this->getVersion(), $subDomain);
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('physicalDeviceId' => $physicalId, 'name' => $name)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return new ACDevice(
				intval($json['device']['deviceId']),
				$json['device']['physicalDeviceId'],
				$json['device']['name'],
				intval($json['device']['owner']),
				$json['device']['aesKey'],
				intval($json['device']['gatewayDeviceId']),
				intval($json['device']['subDomainId']),
				intval($json['device']['rootId']),
				intval($json['device']['status']),
				$subDomain
			);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 解除设备与用户的绑定关系。
	 * @param $subDomain	string	是要被解绑的设备所属的子域的名字。
	 * @param $deviceId		int		要被解除绑定关系的设备的逻辑ID。
	 * @param $user			ACUser	ACUser对象，表示要被解除绑定关系的用户。如果该用户是该设备的管理员，则解除这两者之间的绑定关系时，将解除设备与其它所有用户的绑定。
	 * @return				bool	操作成功返回TRUE，否则返回FALSE。失败时可调用getLastError()方法获取错误消息。
	 */
	public function unbindDevice($subDomain, $deviceId, $user) {
		$request = new ACRequest($this->getName(), 'unbindDevice', $this->getVersion(), $subDomain);
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => $deviceId)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
     * @deprecated 请使用方法 #fetchDeviceShareCode 及 #refreshDeviceShareCode 获取/更新设备的分享码。
     *
	 * 设备的管理员用户获取设备的分享码。
	 * @param $deviceId	int		要操作的对象的逻辑ID。
	 * @param $user		ACUser	ACUser对象，表示该设备的管理员用户。
	 * @param $timeout	int		生成的分享码的有效时长。以秒为单位。
	 * @return			string	操作成功后返回分享码字符串。操作失败，则返回空字符串，并且可调用getLastError()方法获取错误消息。
	 */
	public function getDeviceShareCode($deviceId, $user, $timeout) {
		$request = new ACRequest($this->getName(), 'getShareCode', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => $deviceId, 'timeout' => $timeout)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['shareCode'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

    /**
     * 取设备当前的分享码。
     * @param $deviceId int  设备的逻辑ID。
     * @param $user     ACUser ACUser对象，表示该设备的管理员用户。
     * @param $timeout  int  分享码的有效时长（以秒为单位）。此参数取值为0时表示仅取当前有效即未过期的分享码，若不存在有效的分享码则抛出分享码不存在的异常。
     *                       此参数值大于0时，若存在有效的分享码，则将其有效时长重置为此值后返回该分享码；若不存在有效的分享码则生成一个新的有效时长为timeout的分享码并返回之。
     *                       此参数的值不能超过3153600000秒（约100年），否则将被截取为此最大值。此参数小于0时也表示取此最大值。
     * @return ACDeviceShareCode 设备的分享码。操作失败时返回NULL，并且可调用getLastError()方法获取错误消息。
     */
	public function fetchDeviceShareCode($deviceId, $user, $timeout) {
        $request = new ACRequest($this->getName(), 'fetchDeviceShareCode', $this->getVersion());
        $request->setUser($user);
        $request->setPayloadAsJSON(json_encode(array('deviceId' => $deviceId, 'timeout' => $timeout)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            return new ACDeviceShareCode($json['shareCode'], $json['expireAtUTC']);
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 更新设备的分享分享码。此更新操作将删除已经存在的有效分享码。
     * @param $deviceId int 设备的逻辑ID。
     * @param $user     ACUser ACUser对象，表示该设备的管理员用户。
     * @param $timeout  int 分享码的有效时长（以秒为单位）。此参数的值为0时表示取值3600秒（1小时）。此参数的值大于0时不能超过3153600000秒（约100年），否则将被截取为此最大值。
     *                 此参数的值小于0时也表示取此最大值。
     * @return ACDeviceShareCode 设备的分享码。操作失败时返回NULL，并且可调用getLastError()方法获取错误消息。
     */
    public function refreshDeviceShareCode($deviceId, $user, $timeout) {
        $request = new ACRequest($this->getName(), 'refreshDeviceShareCode', $this->getVersion());
        $request->setUser($user);
        $request->setPayloadAsJSON(json_encode(array('deviceId' => $deviceId, 'timeout' => $timeout)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            return new ACDeviceShareCode($json['shareCode'], $json['expireAtUTC']);
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

	/**
	 * 将设备绑定至获取了分享码的用户。
	 * @param $user			ACUser	ACUser对象，表示获取了分享码的用户。设备将被绑定至该用户。
	 * @param $shareCode	string	绑定设备使用的分享码。
	 * @return				ACDevice	操作成功将返回ACDevice对象，表示被绑定的设备的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function bindDeviceWithShareCode($user, $shareCode) {
		$request = new ACRequest($this->getName(), 'bindDeviceWithShareCode', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('shareCode' => $shareCode)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return new ACDevice(
				intval($json['device']['deviceId']),
				$json['device']['physicalDeviceId'],
				$json['device']['name'],
				intval($json['device']['owner']),
				$json['device']['aesKey'],
				intval($json['device']['gatewayDeviceId']),
				intval($json['device']['subDomainId']),
				intval($json['device']['rootId']),
                intval($json['device']['status']),
				$json['device']['subDomainName']
			);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 将设备绑定至指定用户。
	 * @param $adminUser	ACUser	ACUser对象，是要被绑定的设备的管理员用户。
	 * @param $deviceId		int		要被绑定的设备的逻辑ID。
	 * @param $account		string	要绑定设备的用户的帐号：email或手机号。
	 * @return 				bool	绑定成功返回TURE，否则返回FALSE。绑定失败时可调用getLastError()方法获取错误信息。
	 */
	public function bindDeviceWithUser($adminUser, $deviceId, $account) {
		$request = new ACRequest($this->getName(), 'bindDeviceWithUser', $this->getVersion());
		$request->setUser($adminUser);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'account' => strval($account))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 将设备绑定至指定用户。
	 * @param $adminUser	ACUser	ACUser对象，是要被绑定的设备的管理员用户。
	 * @param $deviceId		int		要被绑定的设备的逻辑ID。
	 * @param $userId		int		要绑定设备的用户的ID。
	 * @return 				bool	绑定成功返回TURE，否则返回FALSE。绑定失败时可调用getLastError()方法获取错误信息。
	 */
	public function bindDeviceWithUserByID($adminUser, $deviceId, $userId) {
		$request = new ACRequest($this->getName(), 'bindDeviceWithUser', $this->getVersion());
		$request->setUser($adminUser);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'userId' => intval($userId))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 解邦设备与指定用户。
	 * @param $adminUser	ACUser	ACUser对象，是设备的管理员用户。
	 * @param $deviceId		int		要被解绑的设备的逻辑ID。
	 * @param $userId		int		要解绑设备的用户的ID。
	 * @return 				bool	解绑成功返回TURE，否则返回FALSE。解邦失败时可调用getLastError()方法获取错误信息。
	 */
	public function unbindDeviceWithUser($adminUser, $deviceId, $userId) {
		$request = new ACRequest($this->getName(), 'unbindDeviceWithUser', $this->getVersion());
		$request->setUser($adminUser);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'userId' => intval($userId))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	//@}

	/**
     * @deprecated 本方法限制了可查询的设备数目的上限。请使用方法 #getDeviceCountOfUser 及 #getDevicesOfUser 查询用户绑定的设备信息。
     *
	 * 查询用户绑定的所有设备（包括网关及其子设备）。
	 * @param $user	ACUser	ACUser对象，表示被查询的用户。
	 * @return		array	返回由ACDevice对象组成的数组，表示该用户所绑定的全部设备。操作失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function listDevices($user) {
		$request = new ACRequest($this->getName(), 'listDevices', $this->getVersion());
		$request->setUser($user);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$list = array();
			if (is_array($json) && array_key_exists('devices', $json)) {
				foreach ($json['devices'] as $dev) {
					array_push($list, new ACDevice(
						intval($dev['deviceId']),
						$dev['physicalDeviceId'],
						$dev['name'],
						intval($dev['owner']),
						$dev['aesKey'],
						intval($dev['gatewayDeviceId']),
						intval($dev['subDomainId']),
						intval($dev['rootId']),
						intval($dev['status']),
						$dev['subDomainName']
					));
				}
			}
			return $list;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 查询某设备的所有用户。
	 * @param $deviceId	int		是拟查询的设备的逻辑ID。
	 * @return			array	操作成功时返回一个由ACUser对象组成的数组，表示所有绑定了该设备的用户。操作失败时返回NULL，并且可调用getLastError()方法获取错误信息。
	 */
	public function listUsers($deviceId) {
		$request = new ACRequest($this->getName(), 'listUsers', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$list = array();
			if (is_array($json) && array_key_exists('users', $json)) {
				foreach ($json['users'] as $u) {
					array_push($list, new ACUser(
						intval($u['userId']),
						'',
						$u['nickName']
					));
				}
			}
			return $list;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 通过设备逻辑ID查询设备的在线状态。
	 * @param $deviceId	int		设备的逻辑ID。
	 * @return 			bool	返回TRUE表示设备在线；返回FALSE表示设备不在线或状态未知。返回FALSE时，需要调用getLastError()方法检查状态。如果errCode为0，则表示设备不在线；否则表示操作出错，设备的状态为未知。
	 */
	public function isDeviceOnline($deviceId) {
		$request = new ACRequest($this->getName(), 'isDeviceOnline', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('deviceId' => $deviceId)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['online'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 通过设备物理ID查询设备的在线状态。
	 * @param $subDomain	string	是设备所属的子域的名字。
	 * @param $physicalId	string	设备的物理ID。
	 * @return				bool	返回TRUE表示设备在线；返回FALSE表示设备不在线或状态未知。返回FALSE时，需要调用getLastError()方法检查状态。如果errCode为0，则表示设备不在线；否则表示操作出错，设备的状态为未知。
	 */
	public function isDeviceOnlineByPhysicalId($subDomain, $physicalId) {
		$request = new ACRequest($this->getName(), 'isDeviceOnline', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('physicalDeviceId' => $physicalId)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['online'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 检查设备是否已经被用户绑定了。
	 * @param $subDomain	string	是设备所属的子域的名字。
	 * @param $physicalId	string	要被检查的设备的物理ID。
	 * @return				bool	返回TRUE表示设备已被绑定；返回FALSE表示设备未被绑定或状态未知。返回FALSE时，需要调用getLastError()方法检查状态。如果errCode为0，则表示设备未被绑定；否则表示操作出错，设备的状态为未知。
	 */
	public function isDeviceBound($subDomain, $physicalId) {
		$request = new ACRequest($this->getName(), 'isDeviceBound', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('physicalDeviceId' => $physicalId)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return boolval($json['bound']);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

    /**
     * 检查指定用户是否绑定了指定设备。
     * @param $userId       int     用户ID。
     * @param $deviceId     int     设备的逻辑ID。
     * @return              bool    返回TRUE表示用户绑定了设备，否则表示没有绑定。
     */
	public function isDeviceBoundByUser($userId, $deviceId) {
        $request = new ACRequest($this->getName(), 'isDeviceBoundByUser', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('userId' => $userId, 'deviceId' => $deviceId)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            return boolval($json['bound']);
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 检查指定用户是否绑定了指定设备。
     * @param $userId       int     用户ID。
     * @param $subDomain    string  设备所属的子域的名字。
     * @param $physicalId   string  设备的物理ID。
     * @return              bool    返回TRUE表示用户绑定了设备，否则表示没有绑定。
     */
    public function isDeviceBoundByUserWithPhysicalId($userId, $subDomain, $physicalId) {
        $request = new ACRequest($this->getName(), 'isDeviceBoundByUser', $this->getVersion(), $subDomain);
        $request->setPayloadAsJSON(json_encode(array('userId' => $userId, 'physicalDeviceId' => $physicalId)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            return boolval($json['bound']);
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

	/**
	 * 取设备的逻辑ID。
	 * @param $subDomain	string	是设备所属的子域的名字。
	 * @param $physicalId	string	表示设备的物理ID。
	 * @return				int		返回设备的逻辑ID。有效的逻辑ID是正整数。返回0表示操作失败。可调用getLastError()方法获取错误消息。
	 */
	public function getDeviceId($subDomain, $physicalId) {
		$request = new ACRequest($this->getName(), 'getDeviceId', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('physicalDeviceId' => $physicalId)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return intval($json['deviceId']);
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return 0;
		}
	}

	/**
	 * 查询已被绑定的设备的总数，并提供其中在线的设备数。
	 * @param $subDomain	string 拟查询的设备所属的子域的名字。空字符串或者NULL表示忽略该参数。
	 * @return				array|FALSE 操作成功时返回一个包含两个键"count"和"online"的关联数组，分别表示设备总数和其中在线的设备数。操作失败时返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function getDeviceCount($subDomain = '') {
		$request = new ACRequest($this->getName(), 'getDeviceCount', $this->getVersion(), $subDomain);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 查询所有被绑定的设备。
	 *
	 * @param $subDomain	string	拟查询的设备所属的子域的名字。空字符串或NULL表示忽略该参数。
	 * @param $offset		int		offset与limit限制查询的范围。offset是非负整数，表示返回的记录在原始集合中的偏移量。
	 * @param $limit		int		limit与offset参数指定查询的范围。limit是正整数，表示最多返回的记录的数目，并且其最大值不能超过100。
	 * @return 				array|FALSE	操作成功时返回ACDevice对象的数组。操作失败时返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function getAllDevices($subDomain, $offset, $limit) {
		$request = new ACRequest($this->getName(), 'getAllDevices', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('offset' => intval($offset), 'limit' => intval($limit))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$devices = array();
			foreach ($json['devices'] as $item) {
				$deviceId     = intval($item['deviceId']);
				$physicalId   = strval($item['physicalDeviceId']);
				$name         = strval($item['name']);
				$ownerId      = intval($item['owner']);
				$subDomainId  = intval($item['subDomainId']);
				$onlineStatus = boolval($item['online']) ? 1 : 0;
				$userIds      = $item['users'];
				$d = new ACDevice($deviceId, $physicalId, $name, $ownerId, '', 0, $subDomainId, 0, $onlineStatus);
				$d->setUserIds($userIds);
				array_push($devices, $d);
			}
			return $devices;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 查询用户绑定的设备数目。
	 * @param int 	 $userId		拟查询的用户的ID。
	 * @param string $subDomain		拟查询的设备所属的子域的名字。空字符串或NULL表示忽略该参数。
	 * @return int					用户绑定的设备的数目。返回值小于0时表示操作失败，此时可调用getLastError()方法获取错误信息。
	 */
	public function getDeviceCountOfUser($userId, $subDomain = '') {
		$request = new ACRequest($this->getName(), 'getDeviceCountOfUser', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('userId' => intval($userId))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			if (array_key_exists('count', $json)) {
				return $json['count'];
			} else {
				$this->resetLastError(-1, 'unknown response: ' . $response->getResponse());
				return -1;
			}
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return -1;
		}
	}

	/**
	 * 查询用户已绑定的设备。
	 * @param int	 $userId		拟查询的用户的ID。
	 * @param string $subDomain		拟查询的设备所属的子域的名字。空字符串或NULL表示忽略该参数。
	 * @param int	 $offset		offset与limit限制查询的范围。offset是非负整数，表示返回的记录在原始集合中的偏移量。
	 * @param int	 $limit			limit与offset参数指定查询的范围。limit是正整数，表示最多返回的记录的数目，并且其最大值不能超过100。
	 * @return array | FALSE		操作成功时返回ACDevice对象的数组。操作失败时返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function getDevicesOfUser($userId, $subDomain, $offset, $limit) {
		$request = new ACRequest($this->getName(), 'getDevicesOfUser', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('userId' => intval($userId), 'offset' => intval($offset), 'limit' => intval($limit))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			$devices = array();
			foreach ($json['devices'] as $item) {
				$deviceId     = intval($item['deviceId']);
				$physicalId   = strval($item['physicalDeviceId']);
				$name         = strval($item['name']);
				$ownerId      = intval($item['owner']);
				$subDomainId  = intval($item['subDomainId']);
				$onlineStatus = boolval($item['online']) ? 1 : 0;
				$userIds      = $item['users'];
				$d = new ACDevice($deviceId, $physicalId, $name, $ownerId, '', 0, $subDomainId, 0, $onlineStatus);
				$d->setUserIds($userIds);
				array_push($devices, $d);
			}
			return $devices;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

    /**
     * 查询用户已绑定的设备及其扩展属性。
     * @param $userId    int    拟查询的用户的ID。
     * @param $subDomain string 拟查询的设备所属的子域的名字。空字符串或NULL表示忽略该参数。
     * @param $offset    int    $offset与$limit限制查询的范围。$offset是非负整数，表示返回的记录在原始集合中的偏移量。
     * @param $limit     int    $limit与$offset参数指定查询的范围。$limit是正整数，表示最多返回的记录的数目，并且其最大值不能超过100。
     * @return array | FALSE    返回设备列表，其中包含的设备属性包括设备的基础信息、绑定时间、扩展属性、故障信息等。
     */
	public function getDevicesOfUserExt($userId, $subDomain, $offset, $limit) {
        $request = new ACRequest($this->getName(), 'getDevicesOfUserExt', $this->getVersion(), $subDomain);
		$request->setPayloadAsJSON(json_encode(array('userId' => intval($userId), 'offset' => intval($offset), 'limit' => intval($limit))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			if (array_key_exists('devices', $json)) {
			    return $json['devices'];
            } else {
			    return array();
            }
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
    }

	/**
	 * 向设备发送消息。
	 * @param $user			ACUser	ACUser对象，表示向设备发送消息的用户。
	 * @param $deviceId		int		是目标设备的逻辑ID。
	 * @param $subDomain	string	是拟操作的设备所属的子域的名字。
	 * @param $messageCode	int		发送给设备的消息的码。
	 * @param $message		string	以string对象存储的拟发送给设备的二进制数据。
	 * @param $handset		string	表示调用本方法时用户所使用的终端工具的名字，如'weixin'表示微信终端。
	 * 								开发者也可以通过ACContext对象设置终端工具信息。如果指定了本参数，则以本参数指定的值为准。
	 * @param $handsetVersion	string	表示调用本方法时用户所使用的终端工具的版本信息。
	 * @return				ACResponse	返回ACResponse对象，表示设备或云端服务的响应。
	 */
	public function sendToDevice($user, $deviceId, $subDomain, $messageCode, $message, $handset = '', $handsetVersion = '') {
		$request = new ACRequest($this->getName(), 'sendToDevice', $this->getVersion(), $subDomain);
		$request->setUser($user);
		$request->addParameter('deviceId', $deviceId);
		$request->addParameter('messageCode', $messageCode);
		$request->setPayloadAsStream($message);
		$context = $this->getContext();
		if (is_string($handset) && strlen($handset) > 0) {
			$context->setHandset($handset, $handsetVersion, '');
		}
		return ACHttpClient::doRequest($request, $context);
	}

	/**
	 * 向设备发送消息。
	 * @param $user			ACUser	ACUser对象，表示向设备发送消息的用户。
	 * @param $physicalId	string	是目标设备的物理ID。
	 * @param $subDomain	string	是拟操作的设备所属的子域的名字。
	 * @param $messageCode	int		发送给设备的消息的码。
	 * @param $message		string	以string对象存储的拟发送给设备的二进制数据。
	 * @param $handset		string	表示调用本方法时用户所使用的终端工具的名字，如'weixin'表示微信终端。
	 * 								开发者也可以通过ACContext对象设置终端工具信息。如果指定了本参数，则以本参数指定的值为准。
	 * @param $handsetVersion	string	表示调用本方法时用户所使用的终端工具的版本信息。
	 * @return				ACResponse	返回ACResponse对象，表示设备或云端服务的响应。
	 */
	public function sendToDeviceByPhysicalId($user, $physicalId, $subDomain, $messageCode, $message, $handset = '', $handsetVersion = '') {
		$request = new ACRequest($this->getName(), 'sendToDevice', $this->getVersion(), $subDomain);
		$request->setUser($user);
		$request->addParameter('physicalDeviceId', $physicalId);
		$request->addParameter('messageCode', $messageCode);
		$request->setPayloadAsStream($message);
		$context = $this->getContext();
		if (is_string($handset) && strlen($handset) > 0) {
			$context->setHandset($handset, $handsetVersion, '');
		}
		return ACHttpClient::doRequest($request, $context);
	}

	/**
	 * 修改设备名称。
	 * @param $user		ACUser	ACUser对象，表示要修改设备名字的用户。
	 * @param $deviceId	int		是目标设备的逻辑ID。
	 * @param $name		string	是设备的新名字。
	 * @return			bool	返回TRUE表示操作成功；返回FALSE表示操作失败，此时可调用getLastError()获取错误信息。
	 */
	public function changeName($user, $deviceId, $name) {
		$request = new ACRequest($this->getName(), 'modifyDevice', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => $deviceId, 'name' => $name)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/// @name 设备的扩展属性
	//@{
	/**
	 * 设置用户所绑定设备的扩展属性。
	 * @param $user		ACUser	ACUser对象，是发起该操作的用户。
	 * @param $deviceId	int		是要设置其扩展属性的设备的逻辑ID。
	 * @param $profile	array	是由键值对组成的关联数组，表示设备的扩展属性值。
	 * @return			bool	操作成功时返回TRUE，否则返回FALSE。操作失败时可调用getLastError()获取错误信息。
	 */
	public function setDeviceProfile($user, $deviceId, $profile) {
		$request = new ACRequest($this->getName(), 'setDeviceProfile', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'deviceProfile' => $profile)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 取设备的扩展属性。
	 * @param $deviceId	int		是要查询其扩展属性的设备的逻辑ID。
	 * @param $user		ACUser	ACUser对象，是发起该操作的用户。如果不为NULL则要求该用户已经绑定了参数$deviceId表示的设备。为NULL时，直接查询设备的扩展属性。
	 * @return 			array	操作成功时返回一个由键值对组成的关联数组，表示设备的扩展属性。操作失败时返回NULL，并且可调用getLastError()方法获取错误信息。
	 */
	public function getDeviceProfile($deviceId, $user = NULL) {
		$request = new ACRequest($this->getName(), 'getDeviceProfile', $this->getVersion());
		$request->setUser($user);
		$request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId))));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$json = json_decode($response->getResponse(), true);
			return $json['deviceProfile'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}
	//@}
}
