<?php

require_once(dirname(__FILE__) . '/ACService.php');

/**
 * AbleCloud文件存储服务。
 */
class ACBlobStoreMgr extends ACService {
	/**
	 * 构造函数。
	 * @param $name AbleCloud文件存储服务的名字。
	 * @param $version AbleCloud文件存储服务的版本。
	 * @param $context ACContext对象，表示访问该远程服务所依赖的环境信息。
	 */
	function __construct($name, $version, $context) {
		parent::__construct($name, $version, $context);
	}

	/**
	 * 向BlobStore服务上传文件。
	 * @param $bucket 给定的类别名字。
	 * @param $filePath 要被上传的文件的本地路径。
	 * @param $name 可以指定文件被上传后在服务器端的存储名字。如果未指定，则使用从$filePath中提取到的文件名。
	 * @return 操作成功返回TRUE，否则返回FALSE。失败时可调用getLastError()方法获取错误消息。
	 */
	public function put($bucket, $filePath, $name = NULL) {
		$payload = file_get_contents($filePath);
		if ($payload === FALSE) {
			$this->resetLastError(-1, 'Can not read the file of ' . $filePath);
			return FALSE;
		}
		if (is_null($name))
			$name = basename($filePath);
		$request = new ACRequest($this->getName(), 'put', $this->getVersion());
		$request->addParameter('bucket', $bucket);
		$request->addParameter('name', $name);
		$request->setPayloadAsStream($payload);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 从BlobStore服务下载文件。
	 * @param $bucket 要下载的文件所属的类别名。
	 * @param $name 要下载的文件的名字。
	 * @param $filePath 文件下载后的本地存储路径。
	 * @return 下载文件成功时返回TRUE，否则返回FALSE。失败时可调用getLastError()方法获取错误消息。
	 */
	public function get($bucket, $name, $filePath) {
		$request = new ACRequest($this->getName(), 'get', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('bucket' => $bucket, 'name' => $name)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			if (file_put_contents($filePath, $response->getResponse()) !== FALSE) {
				$this->resetLastError();
				return TRUE;
			}
			else {
				$this->resetLastError(-1, 'Write file of ' . $filePath . ' failed.');
				return FALSE;
			}
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 从BlobStore服务撒上删除指定文件。
	 * @param $bucket 要被删除的文件所属的类别名。
	 * @param $name 要被删除的文件的名字。
	 * @return 操作成功返回TRUE，否则返回FALSE。失败时可调用getLastError()方法获取错误消息。
	 */
	public function delete($bucket, $name) {
		$request = new ACRequest($this->getName(), 'delete', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('bucket' => $bucket, 'name' => $name)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
        	return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 替换BlobStore服务中存储的文件。
	 * @param $bucket 要被替换的文件所属的类别名。
	 * @param $name 要被替换的文件的名字。
	 * @param $filePath 包含新内容的文件在本地的存储路径。
	 * @return 操作成功返回TRUE，否则返回FALSE。失败时可调用getLastError()方法获取错误消息。
	 */
	public function replace($bucket, $name, $filePath) {
		$payload = file_get_contents($filePath);
		if ($payload === FALSE) {
			$this->resetLastError(-1, 'Can not read the file of ' . $filePath);
			return FALSE;
		}
		$request = new ACRequest($this->getName(), 'replace', $this->getVersion());
		$request->addParameter('bucket', $bucket);
		$request->addParameter('name', $name);
		$request->setPayloadAsStream($payload);
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}
}