<?php

require_once(dirname(__FILE__) . '/ACDeveloper.php');

/**
 * 访问AbleCloud服务时的环境信息。
 */
class ACContext {
    /**
     * 构造函数。
     * @param $developerId      int       AbleCloud开发者帐号ID。
     * @param $accessKey        string    开发者的Access Key。
     * @param $secretKey        string    开发者的Secret Key。
     * @param $majorDomain      string    本地服务对应的主域的名字。
     * @param $runtimeMode      string    运行模式：test（测试模式），production（生产模式）。
     * @param $routerAddress    string    AbleCloud远程服务的入口地址。
     */
    function __construct($developerId, $accessKey, $secretKey, $majorDomain, $runtimeMode, $routerAddress) {
        $this->developer = new ACDeveloper($developerId, $accessKey, $secretKey);
        $this->majorDomain = $majorDomain;
        $this->runtimeMode = $runtimeMode;
        $this->routerAddress = $routerAddress;
        $this->extHttpHeaders = array();
        $this->traceId = self::newTraceId();
    }

    /**
     * 取AbleCloud开发者信息。
     * @return ACDeveloper    返回ACDeveloper对象。
     */
    public function getDeveloper() {
        return $this->developer;
    }

    /**
     * 取本地服务对应的主域的名字。
     * @return string    返回本地服务对应的主域的名字。
     */
    public function getMajorDomain() {
        return $this->majorDomain;
    }

    /**
     * 取AbleCloud远程服务的访问入口地址。
     * @return string    返回AbleCloud远程服务的访问入口地址。
     */
    public function getRouterAddress() {
        return $this->routerAddress;
    }

    /**
     * 检查是否配置为生产环境。
     * @return bool    返回TRUE表示当前配置为生产环境，否则表示为测试环境。
     */
    public function isProductionMode() {
        return $this->runtimeMode != 'test';
    }

    /**
     * 设置用户控制设备时所使用的终端工具的信息。
     * @param $name      string    是终端工具所使用的应用的名字，如'weixin'（表示微信终端）等。
     * @param $version   string    是终端工具所使用的应用的版本信息。缺省值为空字符串。
     * @param $id        string    是终端工具的ID，用于标识该工具。缺省值为空字符串。
     * @param $osName    string    是终端工具所使用的操作系统的名字，如android，ios等。
     */
    public function setHandset($name, $version = '', $id = '', $osName = '') {
        if (is_string($name) && strlen($name) > 0) {
            $this->addExtHttpHeader('X-Zc-Device-Name', $name);
            $this->addExtHttpHeader('X-Zc-Operation-Type', $name);
        }
        if (is_string($version) && strlen($version) > 0)
            $this->addExtHttpHeader('X-Zc-Device-Version', $version);
        if (is_string($id) && strlen($id) > 0)
            $this->addExtHttpHeader('X-Zc-Device-ID', $id);
        if (is_string($osName) && strlen($osName) > 0)
            $this->addExtHttpHeader('X-Zc-Device-OS', $osName);
    }

    /**
     * 显示设置请求中携带的客户端的IP地址。
     * @param $clientIP string    IP地址。本方法不检查该参数的值的格式，需要调用者保证参数的有效性。
     */
    public function setClientAddress($clientIP) {
        if (is_string($clientIP) && strlen($clientIP) > 0) {
            $this->addExtHttpHeader('X-Zc-Client-Addr', $clientIP);
        } else {
            if (array_key_exists('X-Zc-Client-Addr', $this->extHttpHeaders)) {
                unset($this->extHttpHeaders['X-Zc-Client-Addr']);
            }
        }
    }

    /**
     * 为发给AbleCloud云端的HTTP请求配置自定义的HTTP Header。
     * @param $header   string    是要配置的HTTP Header的名字。
     * @param $value    string    是$header对应的值。
     */
    public function addExtHttpHeader($header, $value) {
        $this->extHttpHeaders[$header] = $value;
    }

    /**
     * 取自定义的HTTP Header信息。
     * @return array    返回一个关联数组。数组的每一个键值对表示一项自定义的Header。
     */
    public function getExtHttpHeaders() {
        return $this->extHttpHeaders;
    }

    /**
     * @deprecated 不再使用。使用ACRequest::getTraceId。
     *
     * 取当前的Trace-Id。
     * @return string    当前的Trace-Id。
     */
    public function getTraceId() {
        return $this->traceId;
    }

    /**
     * @deprecated 不再使用。使用ACRequest::setTraceId。
     *
     * 设置Trace-Id。
     * @param $traceId string    是符合RFC 4122 - Section 4.4规范的UUID的十六进制表示形式，用来作为请求的Trace-Id。
     */
    public function setTraceId($traceId) {
        $this->traceId = $traceId;
    }

    /**
     * @deprecated 不再使用。使用ACRequest::newTraceId。
     *
     * 生成一个Trace-ID。
     * @return string    是符合RFC 4122 - Section 4.4规范的UUID的十六进制表示形式。
     */
    public static function newTraceId() {
        $data = openssl_random_pseudo_bytes(16);
        // RFC 4122 - Section 4.4
        $data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
        $data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10
        return bin2hex($data);
    }

    private $developer;            /// ACDeveloper对象，表示AbleCloud开发者。
    private $majorDomain;        /// 本地服务对应的主域的名字。
    private $runtimeMode;        /// 字符串：test（测试模式），production（生产模式）。
    private $routerAddress;        /// AbleCloud远程服务的访问入口地址，如：http://test.ablecloud.cn:5000。
    private $extHttpHeaders;    /// 自定义的HTTP请求Header。
    private $traceId;            /// Trace-ID。
}
