<?php

require_once(dirname(__FILE__) . '/ACService.php');

/**
 * 反馈消息的列定义。
 */
class ACFeedbackColumn {
    public $columnName = '';    ///< 列的名字。
    public $columnType = 0;     ///< 列的类型：1（整数），2（浮点数），3（布尔），4（字符串），5（图片）。
    public $columnLength = 0;   ///< 列的值的长度。
    public $description = '';   ///< 列的描述信息。

    public static function fromObject($obj) {
        if (!is_array($obj))
            return NULL;
        $col = new ACFeedbackColumn();
        if (array_key_exists('columnName', $obj))
            $col->columnName = $obj['columnName'];
        if (array_key_exists('columnType', $obj))
            $col->columnType = intval($obj['columnType']);
        if (array_key_exists('columnLength', $obj))
            $col->columnLength = intval($obj['columnLength']);
        if (array_key_exists('description', $obj))
            $col->description = $obj['description'];
        return $col;
    }
}

/**
 * AbleCloud客户反馈消息服务。
 */
class ACFeedbackMgr extends ACService {
    /**
     * 构造函数。
     * @param $name		string		AbleCloud产品服务的名字。
     * @param $version	int			AbleCloud产品服务的主版本号。
     * @param $context	ACContext	表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
    }

    /**
     * 取用户反馈信息的列信息。
     * @return array 操作成功时返回ACFeedbackColumn对象的数组。操作失败时返回NULL，并可调用方法getLastError()获取错误信息。
     */
    public function listColumns() {
        $request = new ACRequest($this->getName(), 'listColumns', $this->getVersion());
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            if (array_key_exists('columns', $json)) {
                $list = array();
                foreach ($json['columns'] as $c) {
                    $col = ACFeedbackColumn::fromObject($c);
                    if (!is_null($col)) {
                        array_push($list, $col);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from listColumns: ' . $response->getResponse());
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 查询用户反馈记录的数目。
     * @param $startTime            string  指定查询条件的起始时刻：YYYY-MM-DD HH:MM:SS。为空表示不指定该条件。
     * @param $endTime              string  指定查询条件的截止时刻：YYYY-MM-DD HH:MM:SS。为空表示不指定该条件。
     * @param $productSubDomainName string  所查询产品所属的子域的名字。为空表示不指定该条件。APP应用对应的名字为'app'。
     * @param $productModel         string  所查询产品的型号。为空表示不指定该条件。
     * @param $status               int     指定反馈记录的状态：1（开放）；2（关闭）。为0表示不指定该条件。
     * @return                      int     返回非负的整数表示符合条件的记录的数目。返回负数表示查询失败，可以调用方法getLastError()获取错误信息。
     */
    public function getFeedbacksCount($startTime = '', $endTime = '', $productSubDomainName = '', $productModel = '', $status = 0) {
        $data = array(
            'isCount' => true,
            'startTime' => strval($startTime),
            'endTime'   => strval($endTime),
            'product'   => strval($productSubDomainName),
            'model'     => strval($productModel),
            'status'    => intval($status)
        );
        $request = new ACRequest($this->getName(), 'scanFeedbacks', $this->getVersion());
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            if (array_key_exists('count', $json)) {
                return intval($json['count']);
            } else {
                $this->resetLastError(-1, 'Unknown response from scanFeedbacks: ' . $response->getResponse());
                return -1;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return -1;
        }
    }

    /**
     * 查询用户反馈记录。
     * @param $startTime            string  指定查询条件的起始时刻：YYYY-MM-DD HH:MM:SS。为空表示不指定该条件。
     * @param $endTime              string  指定查询条件的截止时刻：YYYY-MM-DD HH:MM:SS。为空表示不指定该条件。
     * @param $productSubDomainName string  所查询产品所属的子域的名字。为空表示不指定该条件。APP应用对应的名字为'app'。
     * @param $productModel         string  所查询产品的型号。为空表示不指定该条件。
     * @param $status               int     指定反馈记录的状态：1（开放）；2（关闭）。为0表示不指定该条件。
     * @param $offset               int     $offset与$limit参数用于实现分页查询的效果。$offset是从0开始的偏移量，表示返回记录集中从第offset位置开始的共limit条记录的信息。
     * @param $limit                int     $offset与$limit参数用于实现分页查询的效果。$limit是正整数，表示返回记录集中从第offset位置开始的共limit条记录的信息。
     * @param $orderByASC           bool    是否以记录的创建时间的升序排序。
     * @return                      array   操作成功时，返回一个数组表示查询的结果。数组中的每个元素为由键值对组成的关联数组，表示对应的反馈记录。
     *                                      操作失败时返回NULL，可以调用方法getLastError()获取错误信息。
     */
    public function scanFeedbacks($startTime = '', $endTime = '', $productSubDomainName = '', $productModel = '', $status = 0, $offset = 0, $limit = 0, $orderByASC = false) {
        $data = array(
            'isCount' => false,
            'startTime' => strval($startTime),
            'endTime'   => strval($endTime),
            'product'   => strval($productSubDomainName),
            'model'     => strval($productModel),
            'status'    => intval($status),
            'offset'    => intval($offset),
            'limit'     => intval($limit),
            'order'     => $orderByASC ? 'asc' : 'desc'
        );
        $request = new ACRequest($this->getName(), 'scanFeedbacks', $this->getVersion());
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            if (array_key_exists('result', $json)) {
                return $json['result'];
            } else {
                $this->resetLastError(-1, 'Unknown response from scanFeedbacks: ' . $response->getResponse());
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 设置指定反馈意见的状态为开启——对应的状态码的值为整数1。
     * @param $id   int     反馈意见的ID。
     * @return      bool    返回TRUE表示操作成功，否则表示操作失败。失败时可调用getLastError()方法获取错误信息。
     */
    public function openFeedback($id) {
        $request = new ACRequest($this->getName(), 'openFeedback', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('id' => intval($id))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 设置指定反馈意见的状态为关闭——对应的状态码的值为整数2。
     * @param $id   int     反馈意见的ID。
     * @return      bool    返回TRUE表示操作成功，否则表示操作失败。失败时可调用getLastError()方法获取错误信息。
     */
    public function closeFeedback($id) {
        $request = new ACRequest($this->getName(), 'closeFeedback', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('id' => intval($id))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }
}