<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(dirname(dirname(__FILE__))) . '/qiniu/autoload.php');

use Qiniu\Storage\UploadManager;

/**
 * AbleCloud文件存储服务中文件的访问权限。
 */
class ACACL {

    /**
     * 构造函数。
     * @param $allowPublicRead  bool    是否可读。不可读的文件不能被访问。
     * @param $allowPublicWrite bool    是否可写。不可写的文件上传后不能被修改。
     */
    function __construct($allowPublicRead = true, $allowPublicWrite = true) {
        $this->allowPublicRead  = $allowPublicRead;
        $this->allowPublicWrite = $allowPublicWrite;
    }

    /**
     * 设置是否可读。
     * @param $allow    bool    布尔值，是否可读。
     * @return $this            返回本对象。
     */
    public function allowPublicRead($allow) {
        $this->allowPublicRead = $allow;
        return $this;
    }

    /**
     * 检查是否允许读。
     * @return bool 返回TRUE表示可读，否则表示不可读。
     */
    public function isPublicReadAllowed() {
        return $this->allowPublicRead;
    }

    /**
     * 设置是否可写。
     * @param $allow    bool    布尔值，是否可写。
     * @return $this            返回本对象。
     */
    public function allowPublicWrite($allow) {
        $this->allowPublicWrite = $allow;
        return $this;
    }

    /**
     * 检查是否允许写。
     * @return bool 返回TRUE表示可写，否则表示不可写。
     */
    public function isPublicWriteAllowed() {
        return $this->allowPublicWrite;
    }

    private $allowPublicRead;
    private $allowPublicWrite;
}

/**
 * AbleCloud文件存储服务。
 */
class ACFileMgr extends ACService {
    /**
     * 构造函数。
     * @param $name     string      AbleCloud文件存储服务的名字。
     * @param $version  int         AbleCloud文件存储服务的版本。
     * @param $context  ACContext   ACContext对象，表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
    }

    /**
     * 获取文件的访问/下载URL。
     * @param $bucket       string  要访问/下载的文件在云端所属的类别的名字。
     * @param $name         string  要访问/下载的文件在云端的名字。
     * @param $expireTime   int     所获取的访问/下载URL的有效时长。单位为秒。如果取值为小于或等于0,表示不限定有效期。另，如果上传文件时指定文件上传至public bucket，则此参数会被忽略，返回的URL永久有效。
     * @return              string  返回指定文件的访问/下载URL。返回空字符串时表示操作失败，可以调用getLastError()获取错误信息。
     */
    public function getDownloadUrl($bucket, $name, $expireTime = 0) {
        $expireTime = intval($expireTime);
        if ($expireTime < 0)
            $expireTime = 0;
        $data = array(
            'bucket'     => $bucket,
            'name'       => $name,
            'expireTime' => $expireTime);
        $request = new ACRequest($this->getName(), 'getDownloadUrl', $this->getVersion());
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            return strval($json['downloadUrl']);
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return '';
        }
    }

    /**
     * 上传文件至云端。云端使用七牛或AWS由所对应的AC-BlobStore服务决定。
     * @param $filePath     string  要被上传的文件的本地路径。
     * @param $bucket       string  文件上传后在云端所属的类别的名字。
     * @param $name         string  文件上传后在云端所使用的文件名（包括文件扩展名）。如不指定（null或空字符串），则表示使用从filePath中提取的文件名字。
     * @param $acl          ACACL   文件的访问权限。如果为NULL，则使用缺省值。
     * @param $isPublic     bool    为true时，表示上传文件到public bucket，获取的文件下载URL是永久有效的；为false时，上传文件到private bucket，获取的文件下载URL是有有效期的，并且带有签名信息。
     * @return              bool    操作成功是返回TRUE，否则表示操作失败。失败时可以调用getLastError()获取错误信息。
     */
    public function uploadFile($filePath, $bucket, $name, $acl = NULL, $isPublic = false) {
        if (is_null($name) || empty($name))
            $name = basename($filePath);
        if (ACConfig::isACRuntime()) {
            return $this->put($bucket, $filePath, $name, $isPublic);
        }
        if (is_null($acl) || !is_a($acl, 'ACACL'))
            $acl = new ACACL();
        $aclOpt = array(
            'isPublicReadAllow'  => $acl->isPublicReadAllowed(),
            'isPublicWriteAllow' => $acl->isPublicWriteAllowed()
        );
        // uploadFileInfo
        $storeType   = '';
        $uploadToken = '';
		$new_bucket = $bucket;
        $new_name = $name;
		
        $data = array(
            'bucket' => $bucket,
            'name'   => $name,
            'acl'    => $aclOpt,
			 "version" => 2,
            'accessType' => $isPublic ? 'public' : 'private'
        );
		
        $request = new ACRequest($this->getName(), 'uploadFileInfo', $this->getVersion());
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            if (array_key_exists('storeType', $json)) {
                $storeType = $json['storeType'];
            } else {
                $this->resetLastError(-1, '获取storeType失败。');
                return FALSE;
            }
            if (array_key_exists('uptoken', $json)) {
                $uploadToken = $json['uptoken'];
            } else {
                $this->resetLastError(-1, '获取uptoken失败。');
                return FALSE;
            }
			
			if (array_key_exists('bucket', $json)) {
                $new_bucket = $json['bucket'];
            } else {
                $this->resetLastError(-1, '获取bucket失败。');
                return FALSE;
            }
			
			if (array_key_exists('name', $json)) {
                $new_name = $json['name'];
            } else {
                $this->resetLastError(-1, '获取name失败。');
                return FALSE;
            }
			
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
        // 上传文件
        if (strcasecmp($storeType, 'qiniu') == 0) {
            // 七牛
            $this->resetLastError();
            return $this->uploadFileToQiniu($filePath, $new_bucket . '/' . $new_name, $uploadToken);
        } else if (strcasecmp($storeType, 'aws') == 0) {
            // AWS
            $this->resetLastError();
            return $this->uploadFileToAWS($filePath, $uploadToken, $isPublic);
        } else {
            $this->resetLastError(-1, '不支持的存储类型：' . $storeType);
            return FALSE;
        }
    }

    /**
     * 上传文件至七牛。
     * @param $filePath string  要被上传的文件的本地路径。
     * @param $key      string  文件在七牛所使用的KEY：$bucket/$name。
     * @param $token    string  文件上传的Token。
     * @return          bool    操作是否成功。
     */
    private function uploadFileToQiniu($filePath, $key, $token) {
        $uploadMgr = new UploadManager();
        list($ret, $err) = $uploadMgr->putFile($token, $key, $filePath);
        if ($err !== null) {
            $this->resetLastError(-1, strval($err));
            return FALSE;
        } else {
            return TRUE;
        }
    }

    /**
     * 上传文件至AWS。
     * @param $filePath string  要被上传的文件的本地路径。
     * @param $url      string  文件上传的AWS URL地址。
     * @return          bool    操作是否成功。
     */
    private function uploadFileToAWS($filePath, $url, $isPublic) {
        $file = fopen($filePath, 'rb');
        if ($file === FALSE) {
            $this->resetLastError(-1, '读取文件' . $filePath . '失败。');
            return FALSE;
        }
        $stat = fstat($file);
        $size = $stat['size'];
        $data = fread($file, $size);
        fclose($file);
        if($data === FALSE) {
            $this->resetLastError(-1, '读取文件' . $filePath . '失败。');
            return FALSE;
        }

        // 上传文件
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');//用put方法
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_HEADER, true); // Get response HEADERs.
        curl_setopt($ch, CURLOPT_TIMEOUT, 1200);
        if ($isPublic) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('x-amz-acl: public-read'));
        }
        $result   = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if (strVal($httpCode) == "200") {
            return TRUE;
        } else {
            $this->resetLastError(-1, strval($result));
            return FALSE;
        }
    }

    /**
     * 通过AC-BlobStore直接上传文件。
     * @param $bucket       string  文件上传后在云端所属的类别的名字。
     * @param $filePath     string  要被上传的文件的本地路径。
     * @param $name         string  文件上传后在云端所使用的文件名（包括文件扩展名）。
     * @param $isPublic     bool    为true时，表示上传文件到public bucket，获取的文件下载URL是永久有效的；为false时，上传文件到private bucket，获取的文件下载URL是有有效期的，并且带有签名信息。
     * @return              bool    操作成功是返回TRUE，否则表示操作失败。失败时可以调用getLastError()获取错误信息。
     */
    private function put($bucket, $filePath, $name, $isPublic = false) {
        $payload = file_get_contents($filePath);
        if ($payload === FALSE) {
            $this->resetLastError(-1, 'Can not read the file of ' . $filePath);
            return FALSE;
        }
        $request = new ACRequest($this->getName(), 'put', $this->getVersion());
        $request->addParameter('bucket', $bucket);
        $request->addParameter('name', $name);
        $request->addParameter('accessType', $isPublic ? 'public' : 'private');
        $request->setPayloadAsStream($payload);
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }
}