<?php

require_once(dirname(__FILE__) . '/ACService.php');

/**
 * @brief 运营支持服务。
 */
class ACOSSQueryMgr extends ACService {
    /**
     * 构造函数。
     * @param $name		string		AbleCloud运营支持服务的名字。
     * @param $version	int			AbleCloud运营支持服务的主版本号。
     * @param $context	ACContext	表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
    }

    /**
     * @param $query        ACOSSQueryRequest   查询请求。
     * @param $fieldFactory ACOSSFieldFactory   属性列工厂对象。如果为 NULL ，则使用缺省的实现，即 ACOSSFieldFactory。
     * @return ACOSSQueryResult|null    返回查询结果集合。返回 NULL 时表示操作失败，此时可调用方法 getLastError() 获取错误信息。
     */
    public function query($query, $fieldFactory = NULL) {
        $request = new ACRequest($this->getName(), 'rawQuery', $this->getVersion(), $query->getSubDomain());
        $request->setPayloadAsJSON(json_encode($this->buildRawQueryParam($query)), 'text/json');
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            if (is_null($fieldFactory)) {
                $fieldFactory = new ACOSSFieldFactory();
            }
            $json = json_decode($response->getResponse(), true);
            $result = new ACOSSQueryResult();
            $result->total = $json['Total'];
            foreach ($json['Columns'] as $c) {
                $f = $fieldFactory->newFieldBySourceAndName($c['SourceName'], $c['FieldName']);
                if (is_null($f)) {
                    continue;
                }
                $f->setDisplayName($c['DisplayName']);
                array_push($result->fields, $f);
            }
            $result->rows = $json['Rows'];
            return $result;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 构造rawQuery的查询参数。
     * @param $query    ACOSSQueryRequest   查询请求。
     * @return array    查询参数。
     */
    private function buildRawQueryParam($query) {
        $param = array('Offset' => $query->getOffset(), 'Limit' => $query->getLimit());
        // Fields
        $sources = array();
        $fields = array();
        foreach ($query->getSelectedFields() as $f) {
            $sourceName = $f->getSourceName();
            array_push($fields, array(
                'SourceName' => $sourceName,
                'FieldName' => $f->getName(),
                'FieldType' => $f->getDataType(),
            ));
            $sources[$sourceName] = 1;
        }
        $param['Fields'] = $fields;
        // Conditions
        $conditions = array();
        foreach ($query->getConditions() as $c) {
            array_push($conditions, array(
                'SourceName' => $c->getTargetFieldSourceName(),
                'FieldName' => $c->getTargetFieldName(),
                'Cond' => $c->getOperator(),
                'Values' => $c->getValues(),
            ));
        }
        $param['Conditions'] = $conditions;
        // Links
        $param['Links'] = $this->buildSourcesLink();
        return array('Query' => $param);
    }

    private function buildSourcesLink() {
        // full link
        return array(
            // device_base_info - device_bind_info
            $this->linkDeviceBaseInfoAndDeviceBindInfo(),
            // device_base_info - device_attr_info
            $this->linkDeviceBaseInfoAndDeviceAttrInfo(),
            // device_bind_info - user_base_info
            $this->linkDeviceBindInfoAndUserBaseInfo(),
            // user_base_info - third_account_info
            $this->linkUserBaseInfoAndThirdAccountInfo(),
            // device_base_info - device_fault_info
            $this->linkDeviceBaseInfoAndDeviceFaultInfo()
        );
    }

    private function linkUserBaseInfoAndThirdAccountInfo() {
        $userUID = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::USER_UID);
        $userUIDInThirdAccount = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::USER_UID_IN_THIRD_ACCOUNT);
        return $this->linkSourcesByFields($userUID, $userUIDInThirdAccount);
    }

    private function linkDeviceBaseInfoAndDeviceBindInfo() {
        $devicePhysicalID = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::DEVICE_PHYSICAL_ID);
        $devicePhysicalIDInBindInfo = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::DEVICE_PHYSICAL_ID_IN_BIND_INFO);
        return $this->linkSourcesByFields($devicePhysicalID, $devicePhysicalIDInBindInfo);
    }

    private function linkDeviceBindInfoAndUserBaseInfo() {
        $deviceUserID = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::DEVICE_USER_ID);
        $userUID = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::USER_UID);
        return $this->linkSourcesByFields($deviceUserID, $userUID);
    }

    private function linkDeviceBaseInfoAndDeviceAttrInfo() {
        $deviceLogicID = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::DEVICE_LOGIC_ID);
        $deviceLogicIDInAttrInfo = ACOSSFieldFactory::newFieldBySourceAndName(ACOSSSourceName::DEVICE_ATTR_INFO, 'id');
        return $this->linkSourcesByFields($deviceLogicID, $deviceLogicIDInAttrInfo);
    }

    private function linkDeviceBaseInfoAndDeviceFaultInfo() {
        $deviceLogicID = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::DEVICE_LOGIC_ID);
        $deviceLogicIDInFaultInfo = ACOSSFieldFactory::newFieldByAlias(ACOSSFieldAlias::DEVICE_LOGIC_ID_IN_FAULT_INFO);
        return $this->linkSourcesByFields($deviceLogicID, $deviceLogicIDInFaultInfo);
    }

    /**
     * 按连接列定义两个数据源的连接关系。
     * @param $leftField    ACOSSField  左数据源中的连接列。
     * @param $rightField   ACOSSField  右数据源中的连接列。
     * @return array    连接关系。
     */
    private function linkSourcesByFields($leftField, $rightField) {
        return array(
            'LeftSource' => $leftField->getSourceName(),
            'LeftField' => $leftField->getName(),
            'RightSource' => $rightField->getSourceName(),
            'RightField' => $rightField->getName(),
        );
    }
}

/**
 * 运营支持服务的查询请求。
 */
class ACOSSQueryRequest {

    /**
     * 添加要查询的属性列。
     * @param $field ACOSSField 要查熏的属性列。
     * @return ACOSSQueryRequest    返回本对象。
     */
    public function appendSelectedField($field) {
        array_push($this->selectedFields, $field);
        return $this;
    }

    public function getSelectedFields() {
        return $this->selectedFields;
    }

    public function clearSelectedFields() {
        $this->selectedFields = array();
        return $this;
    }

    /**
     * 添加查询条件。各个条件之间的关系为“与”。
     * @param $cond ACOSSQueryCondition 查询条件。
     * @return ACOSSQueryRequest    返回本对象。
     */
    public function appendCondition($cond) {
        array_push($this->conditions, $cond);
        return $this;
    }

    public function getConditions() {
        return $this->conditions;
    }

    public function clearConditions() {
        $this->conditions = array();
        return $this;
    }

    /**
     * 设置要返回的记录在结果集合中的偏移量。缺省为0。
     * @param $n    int 非负整数。
     * @return ACOSSQueryRequest    返回本对象。
     */
    public function setOffset($n = 0) {
        $this->offset = intval($n) >= 0 ? intval($n) : 0;
        return $this;
    }

    public function getOffset() {
        return $this->offset;
    }

    /**
     * 指定返回记录的数目。最大不超过100。缺省为20。
     * @param $n    int 正整数。
     * @return  ACOSSQueryRequest   返回本对象。
     */
    public function setLimit($n = 20) {
        $v = intval($n);
        if ($v <= 0 || $v > 100) {
            return $this;
        }
        $this->limit = $v;
        return $this;
    }

    public function getLimit() {
        return $this->limit;
    }

    /**
     * 设置产品所属的子域的名字。涉及设备属性的查询需要指定设备所属的子域。仅查询用户属性时可忽略该参数。缺省为空字符串。
     * @param string $name
     * @return  ACOSSQueryRequest   返回本对象。
     */
    public function setSubDomain($name = '') {
        $this->subDomain = $name;
        return $this;
    }

    /**
     * 取产品所属的子域的名字。
     * @return string   产品所属的子的名字。
     */
    public function getSubDomain() {
        return $this->subDomain;
    }


    /// 要查询的属性列( ACOSSField 实例)组成的数组。
    private $selectedFields = array();
    /// 查询条件( ACOSSQueryCondition 实例)组成的数组。数组元素之间的关系为“与”。
    private $conditions = array();
    /// 非负整数，指定要返回的记录在结果集合中的偏移量。
    private $offset = 0;
    /// 正整数，指定返回记录的数目。最大不超过100。
    private $limit = 20;
    /// 字符串，子域的名字。涉及设备属性的查询需要指定设备所属的子域。仅查询用户属性时可忽略该参数。
    private $subDomain = '';
}

/**
 * 运营支持服务的查询结果。
 */
class ACOSSQueryResult {
    /// 整数。匹配查询条件的总记录数目。
    public $total = 0;
    /// ACOSSField 实例的数组，表示查询结果中包含的属性列。
    public $fields = array();
    /// 数组，表示此次返回的查询记录集合。数组中每个元素表示一条记录。
    public $rows = array();
}

/**
 * 运营支持服务的查询条件。
 */
class ACOSSQueryCondition {
    /// @name 操作符
    //@{
    /// 目标值属于值集合。
    const Operator_In = 'In';
    /// 目标值位于值集合的两个元素组成的闭区间内。
    const Operator_Range = 'Range';
    //@}

    /**
     * 设置条件关联的属性列。
     * @param $field    ACOSSField  属性列。
     * @return ACOSSQueryCondition  返回本对象。
     */
    public function setTargetField($field) {
        $this->field = $field;
        return $this;
    }

    public function getTargetField() {
        return $this->field;
    }

    /**
     * 条件作用的属性列的名字。
     * @return string
     */
    public function getTargetFieldName() {
        if (is_null($this->field)) {
            return '';
        } else {
            return $this->field->getName();
        }
    }

    /**
     * 条件作用的属性列所属的数据源的名字。
     * @return string
     */
    public function getTargetFieldSourceName() {
        if (is_null($this->field)) {
            return '';
        } else {
            return $this->field->getSourceName();
        }
    }

    /**
     * 设置条件的操作符。
     * @param $op   string  操作符。有效的操作符为 ACOSSQueryCondition::Operator_In 和 ACOSSQueryCondition::Operator_Range 。
     * @return ACOSSQueryCondition  返回本对象。
     */
    public function setOperator($op) {
        $this->operator = strval($op);
        return $this;
    }

    public function getOperator() {
        return $this->operator;
    }

    /**
     * 添加参考值。本条件指定的操作是将目标属性列的值与参考值的集合做比较。
     * @param $v    mixed   参考值。
     * @return ACOSSQueryCondition  返回本对象。
     */
    public function appendValue($v) {
        array_push($this->values, strval($v));
        return $this;
    }

    public function getValues() {
        return $this->values;
    }

    public function clearValues() {
        $this->values = array();
    }

    private $field = NULL;
    private $operator = '';
    private $values = array();
}

/**
 * 属性列的标准别名。
 */
class ACOSSFieldAlias {
    /// @name 用户属性列
    //@{
    /// 用户ID。值为整数。
    const USER_UID = 'user.uID';
    /// 用户的电子邮件。值为字符串。
    const USER_EMAIL = 'user.email';
    /// 用户的电话。值为字符串。
    const USER_PHONE = 'user.phone';
    /// 用户的注册时间。值为字符串。
    const USER_CREATE_TIME = 'user.createTime';
    /// 第三方平台用户的OpenID。值为字符串。
    const USER_OPEN_ID = 'user.openID';
    /// 第三方平台的标识。值为整数。
    const USER_OPEN_ID_TYPE = 'user.openIDType';
    /// 第三方平台用户表的关联ID：用户ID。
    const USER_UID_IN_THIRD_ACCOUNT = 'user.uIDInThirdAccount';
    //@}

    /// @name 设备属性列
    //@{
    /// 设备的物理ID。值为字符串。
    const DEVICE_PHYSICAL_ID = 'device.physicalID';
    /// 设备的逻辑ID。值为整数。
    const DEVICE_LOGIC_ID = 'device.logicID';
    /// 设备的在线状态。值为整数。
    const DEVICE_ONLINE_STATUS = 'device.onlineStatus';
    /// 设备的激活状态。值为整数。
    const DEVICE_ACTIVE_STATUS = 'device.activeStatus';
    /// 设备的MCU版本。值为字符串。
    const DEVICE_MCU_VERSION = 'device.MCUVersion';
    /// 设备的通信模块的版本。值为字符串。
    const DEVICE_MOD_VERSION = 'device.MODVersion';
    /// 设备的激活时间。值为字符串。
    const DEVICE_ACTIVE_TIME = 'device.activeTime';
    /// 设备最近一次在线时间。值为字符串。
    const DEVICE_LAST_ONLINE_TIME = 'device.lastOnlineTime';
    /// 设备的绑定状态。值为整数。
    const DEVICE_BIND_STATE = 'device.bindState';
    /// 设备的绑定时间。值为整数。
    const DEVICE_BIND_TIME = 'device.bindTime';
    /// 绑定了设备的用户的ID。该用户为管理员。值为整数。
    const DEVICE_ADMIN_ID = 'device.adminID';
    // 绑定了设备的用户的ID。
    const DEVICE_USER_ID = 'device.userID';
    /// 设备绑定信息的关联ID：设备物理ID。
    const DEVICE_PHYSICAL_ID_IN_BIND_INFO = 'device.physicalIDInBindInfo';
    //@}

    /// @name 设备故障信息
    //@{
    /// 设备故障信息的关联ID：设备逻辑ID。
    const DEVICE_LOGIC_ID_IN_FAULT_INFO = "device.logicIDInFaultInfo";
    /// 设备故障状态。
    const DEVICE_FAULT_CURRENT_STATUS = "device.currentFaultStatus";
    //@}
}

/**
 * @brief 运营支持服务的查询方法所使用的属性列的工厂类。
 * @details 开发者可以扩展该工厂类来支持扩展属性。
 */
class ACOSSFieldFactory {

    /**
     * 依据属性列的别名构造对应的 ACOSSField 实例。
     * @param $alias    string  属性列的别名。有效的别名见 ACOSSFieldAlias 中定义的常量。
     * @return ACOSSField|NULL  对应的属性列对象。返回 NULL 时表示别名无效。
     */
    public static function newFieldByAlias($alias) {
        switch ($alias) {
            case ACOSSFieldAlias::USER_UID:
                return new ACOSSFieldUserUID();
            case ACOSSFieldAlias::USER_EMAIL:
                return new ACOSSFieldUserEmail();
            case ACOSSFieldAlias::USER_PHONE:
                return new ACOSSFieldUserPhone();
            case ACOSSFieldAlias::USER_CREATE_TIME:
                return new ACOSSFieldUserCreateTime();
            case ACOSSFieldAlias::USER_OPEN_ID:
                return new ACOSSFieldUserOpenID();
            case ACOSSFieldAlias::USER_OPEN_ID_TYPE:
                return new ACOSSFieldUserOpenIDType();
            case ACOSSFieldAlias::USER_UID_IN_THIRD_ACCOUNT:
                return new ACOSSFieldUserUIDInThirdAccount();
            case ACOSSFieldAlias::DEVICE_PHYSICAL_ID:
                return new ACOSSFieldDevicePhysicalID();
            case ACOSSFieldAlias::DEVICE_LOGIC_ID:
                return new ACOSSFieldDeviceLogicID();
            case ACOSSFieldAlias::DEVICE_ONLINE_STATUS:
                return new ACOSSFieldDeviceOnlineStatus();
            case ACOSSFieldAlias::DEVICE_ACTIVE_STATUS:
                return new ACOSSFieldDeviceActiveStatus();
            case ACOSSFieldAlias::DEVICE_MCU_VERSION:
                return new ACOSSFieldDeviceMCUVersion();
            case ACOSSFieldAlias::DEVICE_MOD_VERSION:
                return new ACOSSFieldDeviceMODVersion();
            case ACOSSFieldAlias::DEVICE_ACTIVE_TIME:
                return new ACOSSFieldDeviceActiveTime();
            case ACOSSFieldAlias::DEVICE_LAST_ONLINE_TIME:
                return new ACOSSFieldDeviceLastOnlineTime();
            case ACOSSFieldAlias::DEVICE_BIND_STATE:
                return new ACOSSFieldDeviceBindState();
            case ACOSSFieldAlias::DEVICE_BIND_TIME:
                return new ACOSSFieldDeviceBindTime();
            case ACOSSFieldAlias::DEVICE_ADMIN_ID:
                return new ACOSSFieldDeviceAdminID();
            case ACOSSFieldAlias::DEVICE_USER_ID:
                return new ACOSSFieldDeviceUserID();
            case ACOSSFieldAlias::DEVICE_PHYSICAL_ID_IN_BIND_INFO:
                return new ACOSSFieldDevicePhysicalIDInBindInfo();
            case ACOSSFieldAlias::DEVICE_LOGIC_ID_IN_FAULT_INFO:
                return new ACOSSFieldDeviceLogicIDInFaultInfo();
            case ACOSSFieldAlias::DEVICE_FAULT_CURRENT_STATUS:
                return new ACOSSFieldDeviceCurrentFaultStatus();
            default:
                return NULL;
        }
    }

    /**
     * 依据属性列的名字及所属的数据源构造对应的 ACOSSField 实例。
     * @param $source   string  属性列所属的数据源的名字。
     * @param $name     string  属性列的名字。
     * @return ACOSSField|NULL  对应的 ACOSSField 实例。返回 NULL 时表示数据源及列名对无效。
     */
    public static function newFieldBySourceAndName($source, $name) {
        if (strcmp($source, ACOSSSourceName::USER_BASE_INFO) == 0) {
            if (strcmp($name, 'uid') == 0) {
                return new ACOSSFieldUserUID();
            } else if (strcmp($name, 'email') == 0) {
                return new ACOSSFieldUserEmail();
            } else if (strcmp($name, 'phone') == 0) {
                return new ACOSSFieldUserPhone();
            } else if (strcmp($name, 'create_time') == 0) {
                return new ACOSSFieldUserCreateTime();
            } else {
                return NULL;
            }
        } else if (strcmp($source, ACOSSSourceName::THIRD_ACCOUNT_INFO) == 0) {
            if (strcmp($name, 'uid') == 0) {
                return new ACOSSFieldUserUIDInThirdAccount();
            } else if (strcmp($name, 'openid') == 0) {
                return new ACOSSFieldUserOpenID();
            } else if (strcmp($name, 'openid_type') == 0) {
                return new ACOSSFieldUserOpenIDType();
            } else {
                return NULL;
            }
        } else if (strcmp($source, ACOSSSourceName::DEVICE_BASE_INFO) == 0) {
            if (strcmp($name, 'dev_physical_id') == 0) {
                return new ACOSSFieldDevicePhysicalID();
            } else if (strcmp($name, 'dev_logic_id') == 0) {
                return new ACOSSFieldDeviceLogicID();
            } else if (strcmp($name, 'online_status') == 0) {
                return new ACOSSFieldDeviceOnlineStatus();
            } else if (strcmp($name, 'active_status') == 0) {
                return new ACOSSFieldDeviceActiveStatus();
            } else if (strcmp($name, 'mcu_version') == 0) {
                return new ACOSSFieldDeviceMCUVersion();
            } else if (strcmp($name, 'mod_version') == 0) {
                return new ACOSSFieldDeviceMODVersion();
            } else if (strcmp($name, 'active_time') == 0) {
                return new ACOSSFieldDeviceActiveTime();
            } else if (strcmp($name, 'last_online_time') == 0) {
                return new ACOSSFieldDeviceLastOnlineTime();
            } else {
                return NULL;
            }
        } else if (strcmp($source, ACOSSSourceName::DEVICE_BIND_INFO) == 0) {
            if (strcmp($name, 'dev_physical_id') == 0) {
                return new ACOSSFieldDevicePhysicalIDInBindInfo();
            } else if (strcmp($name, 'bind_state') == 0) {
                return new ACOSSFieldDeviceBindState();
            } else if (strcmp($name, 'bind_time') == 0) {
                return new ACOSSFieldDeviceBindTime();
            } else if (strcmp($name, 'user_id') == 0) {
                return new ACOSSFieldDeviceAdminID();
            } else {
                return NULL;
            }
        } else if (strcmp($source, ACOSSSourceName::DEVICE_ATTR_INFO) == 0) {
            return new ACOSSFieldDeviceAttr($name);
        } else if (strcmp($source, ACOSSSourceName::DEVICE_FAULT_INFO) == 0) {
            if (strcmp($name, 'id') == 0) {
                return new ACOSSFieldDeviceLogicIDInFaultInfo();
            } else if (strcmp($name, '_cur_status') == 0) {
                return new ACOSSFieldDeviceCurrentFaultStatus();
            } else {
                return new ACOSSGeneralField($name, 'device.faultInfo.' . $name, ACOSSSourceName::DEVICE_FAULT_INFO, ACOSSFieldDataType::UNKNOWN);
            }
        } else {
            return NULL;
        }
    }
}

/**
 * 运营支持服务的查询方法中使用的属性列。
 */
abstract class ACOSSField {
    private $displayName = '';

    /**
     * 取列的别名。
     * @return string   列的别名。
     */
    abstract public function getAlias();

    /**
     * 列在数据源中的名字。
     * @return string   列在数据源中的名字。
     */
    abstract public function getName();

    /**
     * 列所属的数据源的名字。
     * @return string   列所属的数据源的名字。
     */
    abstract public function getSourceName();

    /**
     * 列的数据类型。
     * @return int      列的数据类型。
     */
    abstract public function getDataType();

    /**
     * 设置列的显示名。
     * @param $name string  显示名。
     */
    public function setDisplayName($name) {
        $this->displayName = $name;
    }

    /**
     * 取列的显示名。
     * @return string 列的显示名。
     */
    public function getDisplayName() {
        return $this->displayName;
    }
}

/**
 * 属性列的数据类型。
 */
class ACOSSFieldDataType {
    const UNKNOWN = 0;
    const INT = 1;
    const FLOAT = 2;
    const BOOL = 3;
    const STRING = 4;
}

/**
 * 属性列所属的数据源的名字。
 */
class ACOSSSourceName {
    const THIRD_ACCOUNT_INFO = 'ThirdAccountInfo';
    const USER_BASE_INFO = 'UserBaseInfo';
    const DEVICE_BASE_INFO = 'DeviceBaseInfo';
    const DEVICE_BIND_INFO = 'DeviceBindInfo';
    const DEVICE_ATTR_INFO = 'DeviceAttrInfo';
    const DEVICE_FAULT_INFO = 'DeviceFaultInfo';
}

/**
 * ThirdAccountInfo.openid
 */
class ACOSSFieldUserOpenID extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_OPEN_ID;
    }

    public function getName() {
        return 'openid';
    }

    public function getSourceName() {
        return ACOSSSourceName::THIRD_ACCOUNT_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * ThirdAccountInfo.openid_type
 */
class ACOSSFieldUserOpenIDType extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_OPEN_ID_TYPE;
    }

    public function getName() {
        return 'openid_type';
    }

    public function getSourceName() {
        return ACOSSSourceName::THIRD_ACCOUNT_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * ThirdAccountInfo.uid
 */
class ACOSSFieldUserUIDInThirdAccount extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_UID_IN_THIRD_ACCOUNT;
    }

    public function getName() {
        return 'uid';
    }

    public function getSourceName() {
        return ACOSSSourceName::THIRD_ACCOUNT_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * UserBaseInfo.uid
 */
class ACOSSFieldUserUID extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_UID;
    }

    public function getName() {
        return 'uid';
    }

    public function getSourceName() {
        return ACOSSSourceName::USER_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * UserBaseInfo.email
 */
class ACOSSFieldUserEmail extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_EMAIL;
    }

    public function getName() {
        return 'email';
    }

    public function getSourceName() {
        return ACOSSSourceName::USER_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * UserBaseInfo.phone
 */
class ACOSSFieldUserPhone extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_PHONE;
    }

    public function getName() {
        return 'phone';
    }

    public function getSourcename() {
        return ACOSSSourceName::USER_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * UserBaseInfo.create_time
 */
class ACOSSFieldUserCreateTime extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::USER_CREATE_TIME;
    }

    public function getName() {
        return 'create_time';
    }

    public function getSourceName() {
        return ACOSSSourceName::USER_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * DeviceBaseInfo.dev_physical_id
 */
class ACOSSFieldDevicePhysicalID extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_PHYSICAL_ID;
    }

    public function getName() {
        return 'dev_physical_id';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * DeviceBaseInfo.dev_logic_id
 */
class ACOSSFieldDeviceLogicID extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_LOGIC_ID;
    }

    public function getName() {
        return 'dev_logic_id';
    }

    public function getSourcename() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBaseInfo.online_status
 */
class ACOSSFieldDeviceOnlineStatus extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_ONLINE_STATUS;
    }

    public function getName() {
        return 'online_status';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBaseInfo.active_status
 */
class ACOSSFieldDeviceActiveStatus extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_ACTIVE_STATUS;
    }

    public function getName() {
        return 'active_status';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBaseInfo.mcu_version
 */
class ACOSSFieldDeviceMCUVersion extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_MCU_VERSION;
    }

    public function getName() {
        return 'mcu_version';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * DeviceBaseInfo.mod_version
 */
class ACOSSFieldDeviceMODVersion extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_MOD_VERSION;
    }

    public function getName() {
        return 'mod_version';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * DeviceBaseInfo.active_time
 */
class ACOSSFieldDeviceActiveTime extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_ACTIVE_TIME;
    }

    public function getName() {
        return 'active_time';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * DeviceBaseInfo.last_online_time
 */
class ACOSSFieldDeviceLastOnlineTime extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_LAST_ONLINE_TIME;
    }

    public function getName() {
        return 'last_online_time';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BASE_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

/**
 * DeviceBindInfo.bind_state
 */
class ACOSSFieldDeviceBindState extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_BIND_STATE;
    }

    public function getName() {
        return 'bind_state';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BIND_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBindInfo.bind_time
 */
class ACOSSFieldDeviceBindTime extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_BIND_TIME;
    }

    public function getName() {
        return 'bind_time';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BIND_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBindInfo.user_id
 */
class ACOSSFieldDeviceAdminID extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_ADMIN_ID;
    }

    public function getName() {
        return 'user_id';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BIND_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBindInfo.user_id
 */
class ACOSSFieldDeviceUserID extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_USER_ID;
    }

    public function getName() {
        return 'user_id';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BIND_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceBindInfo.dev_physical_id
 */
class ACOSSFieldDevicePhysicalIDInBindInfo extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_PHYSICAL_ID_IN_BIND_INFO;
    }

    public function getName() {
        return 'dev_physical_id';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_BIND_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::STRING;
    }
}

class ACOSSFieldDeviceAttr extends ACOSSField {
    private $name;

    function __construct($name) {
        $this->name = $name;
    }

    public function getAlias() {
		return "device_attr." . $this->name;
	}

	public function getName() {
		return $this->name;
	}

	public function getSourceName() {
		return ACOSSSourceName::DEVICE_ATTR_INFO;
	}

	public function getDataType(){
		return ACOSSFieldDataType::UNKNOWN;
	}
}

/**
 * DeviceFaultInfo.id
 */
class ACOSSFieldDeviceLogicIDInFaultInfo extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_LOGIC_ID_IN_FAULT_INFO;
    }

    public function getName() {
        return 'id';
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_FAULT_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * DeviceFaultInfo._cur_status
 * 设备故障状态：0-正常；1-故障。
 */
class ACOSSFieldDeviceCurrentFaultStatus extends ACOSSField {
    public function getAlias() {
        return ACOSSFieldAlias::DEVICE_FAULT_CURRENT_STATUS;
    }

    public function getName() {
        return "_cur_status";
    }

    public function getSourceName() {
        return ACOSSSourceName::DEVICE_FAULT_INFO;
    }

    public function getDataType() {
        return ACOSSFieldDataType::INT;
    }
}

/**
 * 通用字段。
 */
class ACOSSGeneralField extends ACOSSField {
    private $sourceName;
    private $name;
    private $alias;
    private $dataType;

    /**
     * 构造函数。
     * @param $name string 字段的名字。
     * @param $alias string 字段的别名
     * @param $dataSourceName string 字段所属的数据源的名字。
     * @param $dataType int 字段的数据类型。见ACOSSFieldDataType。
     */
    function __construct($name, $alias, $dataSourceName, $dataType) {
        $this->sourceName = $dataSourceName;
        $this->name = $name;
        $this->alias = $alias;
        $this->dataType = $dataType;
    }

    public function getAlias() {
        return $this->alias;
    }

    public function getName() {
        return $this->name;
    }

    public function getSourceName() {
        return $this->sourceName;
    }

    public function getDataType() {
        return $this->dataType;
    }
}
