<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACQETimeframe.php');
require_once(dirname(__FILE__) . '/ACQETimeInterval.php');
require_once(dirname(__FILE__) . '/ACQEFilter.php');
require_once(dirname(__FILE__) . '/ACQEPropertyFilters.php');
require_once(dirname(__FILE__) . '/ACQEAggFilters.php');

/**
 * AbleCloud数据分析服务的数据查询类方法的基类。
 */
abstract class ACQEReadInterface extends ACService {
    /**
     * 构造函数。
     * @param $name		string		AbleCloud数据分析服务的名字。
     * @param $version	int			AbleCloud数据分析服务的主版本号。
     * @param $context	ACContext	表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
        $this->collection    = '';
        $this->timeframe     = NULL;
        $this->timeInterval  = NULL;
        $this->objectFilters = NULL;
        $this->filters       = NULL;
        $this->groupBy       = NULL;
        $this->orderBy       = NULL;
    }

    /**
     * 设置拟操作的数据集的名字。是必要参数。
     * @param $name     string  拟操作的数据集的名字。
     * @return          ACQEReadInterface   返回本对象。
     */
    public function collection($name) {
        $this->collection = $name;
        return $this;
    }

    /**
     * 设置数分分析的时间段。是必要参数。
     * @param $frame    ACQETimeframe       时间段对象。
     * @return          ACQEReadInterface   返回本对象。
     */
    public function timeframe($frame) {
        $this->timeframe = $frame;
        return $this;
    }

    /**
     * 设置分析的时间间隔参数。可选参数。
     * @param $interval ACQETimeInterval    时间间隔对象。
     * @return          ACQEReadInterface   返回本对象。
     */
    public function timeInterval($interval) {
        $this->timeInterval = $interval;
        return $this;
    }

    /**
     * 设置object_filters参数。可选参数。
     * @param $aggFilters   ACQEAggFilters      查询条件。
     * @return              ACQEReadInterface   返回本对象。
     */
    public function objectFilters($aggFilters) {
        $this->objectFilters = $aggFilters;
        return $this;
    }

    /**
     * 设置filters参数。可选参数。
     * @param $aggFilters   ACQEAggFilters      查询条件。
     * @return              ACQEReadInterface   返回本对象。
     */
    public function filters($aggFilters) {
        $this->filters = $aggFilters;
        return $this;
    }

    /**
     * 设置group_by参数。可选参数。
     * @param $properties   array   指定group_by所依赖的属性列（可以是多个），以及每个属性列的性质，如：array(array('province', 'actor'), array('gender', 'collection'))。
     * 参数$properties是一个数组，数组的每个元素是一个由两个字符串组成的数组。由这两个字符串组成的数组可描述一个属性列：
     * 第一个字符串是属性列的名字；第二个字符串标记属性列的性质——actor，object，phone或者collection（缺省值）。
     * @return              ACQEReadInterface   返回本对象。
     */
    public function groupBy($properties) {
        $this->groupBy = $properties;
        return $this;
    }

    /**
     * 设置order_by参数。可选参数。
     * @param $property string              用来排序的属性列的名字。符号“#”表示依据聚合函数的结果排序。
     * @param $ascend   bool                为TRUE时表示按升序排序，否则表示按降序排序。
     * @return          ACQEReadInterface   返回本对象。
     */
    public function orderBy($property, $ascend) {
        $this->orderBy = array($property, $ascend);
        return $this;
    }

    /**
     * 执行数据分析查询。
     * @return array|bool   返回服务端返回的结果。返回结果为数组时表示访问成功。
     * 返回FALSE时，表示有错误发生。此时可调用getLastError()方法获取错误消息。
     */
    abstract public function execute();

    /**
     * 取数组格式的查询参数。
     * @return array
     */
    protected function getParamInArray() {
        if (empty($this->collection)) {
            $this->resetLastError(-1, '参数collection不能为空。');
            return array();
        }
        if (!is_a($this->timeframe, 'ACQETimeframe')) {
            $this->resetLastError(-1, '参数timeframe不能为无效值。');
            return array();
        }
        $param = array(
            'collection' => $this->collection,
            'timezone'   => $this->timeframe->getTimezone(),
            'timeframe'  => $this->timeframe->toParamInArray()
        );
        if (is_a($this->timeInterval, 'ACQETimeInterval')) {
            $param['interval'] = $this->timeInterval->toParamInArray();
        }
        if (is_a($this->objectFilters, 'ACQEAggFilters')) {
            $param['object_filters'] = $this->objectFilters->toParamInArray();
        }
        if (is_a($this->filters, 'ACQEAggFilters')) {
            $param['filters'] = $this->filters->toParamInArray();
        }
        if (is_array($this->groupBy)) {
            $groupBy     = array();
            $groupByType = array();
            foreach ($this->groupBy as $k => $g) {
                if (is_array($g)) {
                    array_push($groupBy, $g[0]);
                    if (count($g) > 1) {
                        array_push($groupByType, $g[1]);
                    } else {
                        array_push($groupByType, 'collection');
                    }
                }
            }
            $param['group_by']      = $groupBy;
            $param['group_by_type'] = $groupByType;
        }
        if (is_array($this->orderBy)) {
            $param['order_by'] = array('property' => $this->orderBy[0]);
            if ($this->orderBy[1]) {
                $param['order_by']['reverse'] = FALSE;
            } else {
                $param['order_by']['reverse'] = TRUE;
            }
        }
        return $param;
    }

    /// 操作的目标数据集的名字。
    private $collection;
    /// 分析的时间范围。
    private $timeframe;
    /// 时间间隔参数。
    private $timeInterval;
    /// 过滤条件。
    private $objectFilters;
    /// 过滤条件
    private $filters;
    /// group_by参数
    private $groupBy;
    /// order_by参数。
    private $orderBy;
}