<?php
/**
 * AbleCloud数据分析服务的时间段参数。可以是绝对时间，或者是相对时间。
 */
class ACQETimeframe {
    /// @name 时间单位
    //@{
    public static $Unit_Second  = 'seconds';    ///< 秒
    public static $Unit_Minute  = 'minutes';    ///< 分钟
    public static $Unit_Hour    = 'hours';      ///< 小时
    public static $Unit_Day     = 'days';       ///< 天
    public static $Unit_Week    = 'weeks';      ///< 周
    public static $Unit_Month   = 'months';     ///< 月
    public static $Unit_Year    = 'years';      ///< 年
    //@}

    /**
     * 构造函数。
     */
    function __construct() {
        $this->start = NULL;
        $this->end   = NULL;
        $this->relativeTimeframe = '';
        $this->timezone = 'Asia/Shanghai';
    }

    /// @name 绝对时间
    //@{
    /**
     * 设置本对象为绝对时间段。
     * @param $start   DateTime 表示时间段的起始时刻的DateTime对象。$start与$end必须使用相同的时区。
     * @param $end     DateTime 表示时间段的结束时间的DateTime对象。$start与$end必须使用相同的时区。
     */
    public function setAbsoluteFrame($start, $end) {
        if (is_a($start, 'DateTime') && is_a($end, 'DateTime')) {
            $this->start = $start;
            $this->end   = $end;
            $this->timezone = $start->getTimezone()->getName();
            $this->relativeTimeframe = '';
        }
    }
    //@}

    /// @name 相对时间段
    //@{
    /**
     * 设置时间段为“当前N个单位”。
     * “当前”与“之前”的区别是，“当前3分钟”是包含当前时刻所在的这一分钟在内往前3分钟，“之前3分钟”是不包含当前时刻所在的这一分钟在内往前3分钟。
     * @param $n        int     正整数，表示时间量。
     * @param $unit     string  时间单位，如ACQETimeframe::$Unit_Second，ACQETimeframe::$Unit_Minute，……，ACQETimeframe::$Unit_Year等。
     * @param $timezone string  时区的名字。缺省值为'Asia/Shanghai'。
     */
    public function thisRelativeFrame($n, $unit, $timezone = 'Asia/Shanghai') {
        $this->setRelativeFrame(self::$Rel_This, $n, $unit);
        $this->timezone = $timezone;
    }

    /**
     * 设置时间段为“之前N个单位”。
     * “当前”与“之前”的区别是，“当前3分钟”是包含当前时刻所在的这一分钟在内往前3分钟，“之前3分钟”是不包含当前时刻所在的这一分钟在内往前3分钟。
     * @param $n        int     正整数，表示时间量。
     * @param $unit     string  时间单位，如ACQETimeframe::$Unit_Second，ACQETimeframe::$Unit_Minute，……，ACQETimeframe::$Unit_Year等。
     * @param $timezone string  时区的名字。缺省值为'Asia/Shanghai'。
     */
    public function previousRelativeFrame($n, $unit, $timezone = 'Asia/Shanghai') {
        $this->setRelativeFrame(self::$Rel_Previous, $n, $unit);
        $this->timezone = $timezone;
    }
    //@}

    /**
     * 取时间段的时区。
     * @return string   返回时区的名字。
     */
    public function getTimezone() {
        return $this->timezone;
    }

    /**
     * 将时间段转换为AC-QueryEngine要求的参数格式。
     * @return          array|string   以数组格式记录的绝对时间段，或者以字符串记录的相对时间段。
     */
    public function toParamInArray() {
        if (is_null($this->start) || is_null($this->end)) {
            // 相对时间段
            return $this->relativeTimeframe;
        } else {
            // 绝对时间段
            return array(
                'start' => $this->start->format('Y-m-d H:i:s.uO'),
                'end'   => $this->end->format('Y-m-d H:i:s.uO')
            );
        }
    }

    /**
     * 设置相对时间段。
     * @param $rel  string  相对时间段的前缀。
     * @param $n    int     数量。
     * @param $unit string  时间单位。
     */
    private function setRelativeFrame($rel, $n, $unit) {
        $this->relativeTimeframe = $rel . '_' . strval($n) . '_' . $unit;
        $this->start = NULL;
        $this->end   = NULL;
    }

    /// 整数。表示绝对时间段的起止时间：使用举例UTC基准时间的微秒数表示。
    private $start, $end;
    /// 相对时间，如：previous_5_days等。
    private $relativeTimeframe;
    /// 时间段使用的时区。
    private $timezone = 'Asia/Shanghai';
    /// “当前”时间段的前缀。
    private static $Rel_This     = 'this';
    /// “之前”时间段的前缀。
    private static $Rel_Previous = 'previous';
}
