<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACStoreClass.php');
require_once(dirname(__FILE__) . '/ACStoreIterator.php');
require_once(dirname(__FILE__) . '/ACStoreBatchDelete.php');
require_once(dirname(__FILE__) . '/ACStoreBatchUpdate.php');
require_once(dirname(__FILE__) . '/ACStoreModify.php');

/**
 * AbleCloud数据存储服务。
 */
class ACStore extends ACService {
	/**
	 * 构造函数。
	 * @param $name 	string	数据存储服务的名字。
	 * @param $version	int		数据存储服务的版本。
	 * @param $context	ACContext	ACContext对象，表示访问该远程服务所依赖的环境信息。
	 */
	function __construct($name, $version, $context) {
		parent::__construct($name, $version, $context);
	}

	/**
	 * 创建数据集。仅测试环境支持该方法。
	 * @param $classDef ACStoreClass	ACStoreClass对象，表示数据集的定义。
	 * @return			bool			操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function createClass($classDef) {
		if ($this->getContext()->isProductionMode()) {
			$this->resetLastError(-1, 'It is not allowed in production environment.');
			return FALSE;
		}
		$request = new ACRequest($this->getName(), 'createClass', $this->getVersion());
		$request->setPayloadAsJSON($classDef->jsonEncode());
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 查询已创建的数据集。
	 * @return array 返回ACStoreClass数组，表示已定义的数据集。操作失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function listClasses() {
		$request = new ACRequest($this->getName(), 'listClasses', $this->getVersion());
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$payload = json_decode($response->getResponse(), true);
			$list = array();
			foreach ($payload['classes'] as $item) {
				$class = new ACStoreClass($item['domain'], $item['className']);
				foreach ($item['primaryKeys'] as $key) {
					$column = new ACStoreClassColumn($key['attributeName'], (int)$key['attributeType']);
					array_push($class->primaryKeys, $column);
				}
				foreach ($item['entityGroupKeys'] as $key) {
					$column = new ACStoreClassColumn($key['attributeName'], (int)$key['attributeType']);
					array_push($class->entityGroupKeys, $column);
				}
				array_push($list, $class);
			}
			return $list;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 删除指定的数据集。仅测试环境支持该方法。
	 * @param $name string	字符串，表示要被删除的数据集的名字。
	 * @return 		bool	操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function dropClass($name) {
		if ($this->getContext()->isProductionMode()) {
			$this->resetLastError(-1, 'It is not allowed in production environment.');
			return FALSE;
		}
		$request = new ACRequest($this->getName(), 'dropClass', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('className' => $name)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 清除指定数据集中的数据。仅测试环境支持该方法。
	 * @param $name string	要清除其数据的数据集的名字。
	 * @return 		bool	操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function clearClass($name) {
		if ($this->getContext()->isProductionMode()) {
			$this->resetLastError(-1, 'It is not allowed in production environment.');
			return FALSE;
		}
		$request = new ACRequest($this->getName(), 'clearClass', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('className' => $name)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 在数据集中添加一条数据记录。
	 * @param $name string	要添加数据的数据集的名字。
	 * @param $row	array	以键值对的方式（关联数组）描述的要添加的数据记录。其中，至少应包含所有主键的值。
	 * @return 		bool	操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function create($name, $row) {
		$request = new ACRequest($this->getName(), 'create', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('className' => $name, 'object' => $row)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 从数据集中删除指定的记录。
	 * @param $name string	要删除数据的数据集的名字。
	 * @param $row	array	以键值对的方式（关联数组）描述的要被删除的记录。$row参数应指定所有主键的值，删除操作将删除主键值与$row匹配的记录。
	 * @return 		bool	操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function delete($name, $row) {
		$request = new ACRequest($this->getName(), 'delete', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('className' => $name, 'object' => $row)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 更新数据集中指定的记录。
	 * @param $name string	要更新的数据所属的数据集的名字。
	 * @param $row	array	以键值对的方式（关联数组）描述的要被更新的记录，以及更新后的值。$row参数应指定所有主键的值，用于定位要被更新的记录；$row中记录的其它列的值用于更新该记录：只更新原数据记录中已经存在的字段。如果主键指定的记录不存在，本方法将会在数据集中新建一条对应的记录。
	 * @return		bool	操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function update($name, $row) {
		$request = new ACRequest($this->getName(), 'update', $this->getVersion());
		$request->setPayloadAsJSON(json_encode(array('className' => $name, 'object' => $row)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			return TRUE;
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return FALSE;
		}
	}

	/**
	 * 查询指定的记录。
	 * @param $name				string	要查询的数据集的名字。
	 * @param $primaryKeyValues array	以键值对方式（如关联数组等）描述的要查询的记录的主键值。
	 * @param $select 			array	字符串数组，记录了查询结果中应显示的数据列的集合。可选。如为NULL，则表示要查询所有数据列。
	 * @return 					array	操作成功时返回一个关联数组，记录查询结果。操作失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function find($name, $primaryKeyValues, $select = NULL) {
		$request = new ACRequest($this->getName(), 'find', $this->getVersion());
		$findParam = array('object' => $primaryKeyValues);
		if ($select != NULL)
			$findParam = array_merge($findParam, array('select' => $select));
		$request->setPayloadAsJSON(json_encode(array('className' => $name, 'findParam' => $findParam)));
		$response = ACHttpClient::doRequest($request, $this->getContext());
		if ($response->isAck()) {
			$this->resetLastError();
			$payload = json_decode($response->getResponse(), true);
			return $payload['object'];
		} else {
			$this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
			return NULL;
		}
	}

	/**
	 * 查询分区内的数据。兼容v1.3.x之前的版本。
	 * @param $scanner	ACStoreScanner	ACStoreScanner对象，表示查询条件。
	 * @return 			ACStoreIterator	返回一个ACStoreIterator对象，用于遍历查询结果集合中的数据。返回NULL时表示操作失败，此时可调用getLastError()方法获取错误消息。
	 */
	public function scan($scanner) {
		return new ACStoreIterator($this->getName(), $this->getVersion(), $this->getContext(), $scanner);
	}

	/**
	 * 查询数据。
	 * @param $name					string	字符串。要被查询的数据集的名字。
	 * @param $partitionKeyValues	array	以键值对的方式（如关联数组等）描述的查询数据集时所使用的分区键的值。如果数据集没有分区，则使用NULL。
	 * @return						ACStoreScanner	返回一个ACStoreScanner对象，以便于设置查询参数，执行查询，获取查询结果。
	 */
	public function scanExt($name, $partitionKeyValues = NULL) {
		return new ACStoreScanner($name, $partitionKeyValues, $this->getName(), $this->getVersion(), $this->getContext());
	}
	
	/**
	 * 批量删除数据。
	 * @param $name 				string	字符串。要被操作的数据集的名字。
	 * @param $partitionKeyValues	array	以键值对的方式（关联数组）描述的操作数据集时所使用的分区键的值。如果数据集没有分区，则使用NULL。
	 * @return 						ACStoreBatchDelete	返回一个ACStoreBatchDelete对象，以便于设置删除参数，执行操作。
	 */
	public function batchDelete($name, $partitionKeyValues = NULL) {
		return new ACStoreBatchDelete($this->getName(), $this->getVersion(), $this->getContext(), $name, $partitionKeyValues);
	}
	
	/**
	 * 批量更新数据。
	 * @param $name 				string	字符串。要被操作的数据集的名字。
	 * @param $partitionKeyValues	array	以键值对的方式（关联数组）描述的操作数据集时所使用的分区键的值。如果数据集没有分区，则使用NULL。
	 * @return 						ACStoreBatchUpdate	返回一个ACStoreBatchUpdate对象，以便于设置更新参数，执行操作。
	 */
	public function batchUpdate($name, $partitionKeyValues = NULL) {
		return new ACStoreBatchUpdate($this->getName(), $this->getVersion(), $this->getContext(), $name, $partitionKeyValues);
	}

	/**
	 * 更新单条记录。
	 * @param $name				string	字符串，是要被操作的数据集的名字。
	 * @return					ACStoreModify	返回一个ACStoreModify对象，以便于设置更新参数，执行操作。
	 */
	public function modify($name) {
		return new ACStoreModify($this->getName(), $this->getVersion(), $this->getContext(), $name);
	}
}
