<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACStoreFilter.php');

/**
 * AbleCloud数据存储服务的批量删除工具。
 */
class ACStoreBatchDelete extends ACService {
	/**
     * 构造函数。
     * @param $name                 string      数据存储服务的名字。
	 * @param $version              int         数据存储服务的版本。
	 * @param $context              ACContext   ACContext对象，表示访问该远程服务所依赖的环境信息。
     * @param $className            string      字符串。要操作的数据集的名字。
     * @param $partitionKeyValues   array       以键值对的方式（关联数组）描述的操作数据集时所使用的分区键的值。如果数据集没有分区，则使用NULL。
     */
    function __construct($name, $version, $context, $className, $partitionKeyValues = NULL) {
	    parent::__construct($name, $version, $context);
        $this->className = $className;
        $this->entityGroupKeyValues = $partitionKeyValues;
        $this->where = NULL;
    }
    
    /**
     * 取要操作的数据集的名字。
     * @return string   要操作的数据集的名字。
     */
    public function getClassName() {
        return $this->className;
    }

    /**
     * 设置删除条件。该方法如果被多次调用，则后续调用传入的参数将覆盖之前设置的所有删除条件。
     * @param $filter ACStoreFilter | ACStoreComplicatedFilter    ACStoreFilter或ACStoreComplicatedFilter对象，表示删除条件。
     * @return ACStoreBatchDelete   本ACStoreBatchDelete对象。
     */
    public function where($filter) {
        if ($filter == NULL) {
            $this->where = NULL;
            return NULL;
        }
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where = new ACStoreComplicatedFilter($filter);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where = $filter;
        }
        return $this;
    }

    /**
     * 以逻辑“与”的关系添加一个删除条件或条件的组合。
     * @details 该方法应该在调用了ACStoreBatchDelete::where方法之后再调用。
     * @param $filter ACStoreFilter | ACStoreComplicatedFilter    ACStoreFilter或ACStoreComplicatedFilter对象，表示新添加的删除条件或条件的组合。如果是组合条件，则组合条件将会被括号组合在一起。
     * @return ACStoreBatchDelete   本ACStoreBatchDelete对象。
     */
    public function andWhere($filter) {
        if ($this->where == NULL)
            return $this;
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where->appendFilter($filter, TRUE);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where->linkTo($filter, TRUE);
        }
        return $this;
    }

    /**
     * 以逻辑“或”的关系添加一个删除条件或条件的组合。
     * @details 该方法应该在调用了ACStoreBatchDelete::where方法之后再调用。
     * @param $filter ACStoreFilter | ACStoreComplicatedFilter    ACStoreFilter或ACStoreComplicatedFilter对象，表示新添加的删除条件或条件的组合。如果是组合条件，则组合条件将会被括号组合在一起。
     * @return ACStoreBatchDelete   本ACStoreBatchDelete对象。
     */
    public function orWhere($filter) {
        if ($this->where == NULL) {
            return $this;
        }
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where->appendFilter($filter, FALSE);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where->linkTo($filter, FALSE);
        }
        return $this;
    }
    
    /**
     * 清除删除条件。
     * @return ACStoreBatchDelete   本ACStoreBatchDelete对象。
     */
    public function clearWhere() {
    	$this->where = NULL;
    	return $this;
    }
    
    /**
     * 执行删除操作。
     * @return bool 返回TRUE或FALSE。返回FALSE表示有错误发生。此时可调用getLastError()方法获取错误消息。
     */
    public function execute() {
	    $request = new ACRequest($this->getName(), 'batchDelete', $this->getVersion());
        $deleteParam = $this->toBatchDeleteParam();
        $request->setPayloadAsJSON(json_encode(array('className' => $this->getClassName(), 'batchDeleteParam' => $deleteParam)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
	        $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }
    
    /**
     * 将本对象转换为AbleCloud存储服务的scanParam结构。
     * @return array 返回一个关联数组，代表存储服务的scanParam参数。
     */
    private function toBatchDeleteParam() {
        $deleteParam = array();
        // entityGroupKeys
        if ($this->entityGroupKeyValues != NULL && count($this->entityGroupKeyValues) > 0)
            $deleteParam = array_merge($deleteParam, array('entityGroupKeys' => $this->entityGroupKeyValues));
        // where
        if ($this->where != NULL)
            $deleteParam = array_merge($deleteParam, array('filters' => $this->where->toArray()));
        return $deleteParam;
    }
    
    /// 要操作的数据集的名字。
    private $className;
    // 分区键。为NULL或空数组时，表示不分区。
    private $entityGroupKeyValues;
    /// 操作条件。
    private $where;
}
