<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACStoreFilter.php');

/**
 * AbleCloud数据存储服务的批量更新工具。
 */
class ACStoreBatchUpdate extends ACService {
	/**
     * 构造函数。
     * @param $name                 string      数据存储服务的名字。
	 * @param $version              int         数据存储服务的版本。
	 * @param $context              ACContext   ACContext对象，表示访问该远程服务所依赖的环境信息。
     * @param $className            string      字符串。要操作的数据集的名字。
     * @param $partitionKeyValues   array       以键值对的方式（关联数组）描述的操作数据集时所使用的分区键的值。如果数据集没有分区，则使用NULL。
     */
    function __construct($name, $version, $context, $className, $partitionKeyValues = NULL) {
	    parent::__construct($name, $version, $context);
        $this->className = $className;
        $this->entityGroupKeyValues = $partitionKeyValues;
        $this->where = NULL;
        $this->expressions = array();
    }
    
    /**
     * 取要操作的数据集的名字。
     * @return string   要操作的数据集的名字。
     */
    public function getClassName() {
        return $this->className;
    }

    /**
     * 设置更新条件。该方法如果被多次调用，则后续调用传入的参数将覆盖之前设置的所有删除条件。
     * @param $filter   ACStoreFilter | ACStoreComplicatedFilter    ACStoreFilter或ACStoreComplicatedFilter对象，表示删除条件。
     * @return          ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function where($filter) {
        if ($filter == NULL) {
            $this->where = NULL;
            return NULL;
        }
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where = new ACStoreComplicatedFilter($filter);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where = $filter;
        }
        return $this;
    }

    /**
     * 以逻辑“与”的关系添加一个更新条件或条件的组合。
     * @details 该方法应该在调用了ACStoreBatchDelete::where方法之后再调用。
     * @param $filter   ACStoreFilter | ACStoreComplicatedFilter    ACStoreFilter或ACStoreComplicatedFilter对象，表示新添加的更新条件或条件的组合。如果是组合条件，则组合条件将会被括号组合在一起。
     * @return          ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function andWhere($filter) {
        if ($this->where == NULL)
            return $this;
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where->appendFilter($filter, TRUE);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where->linkTo($filter, TRUE);
        }
        return $this;
    }

    /**
     * 以逻辑“或”的关系添加一个更新条件或条件的组合。
     * @details 该方法应该在调用了ACStoreBatchDelete::where方法之后再调用。
     * @param $filter   ACStoreFilter | ACStoreComplicatedFilter  ACStoreFilter或ACStoreComplicatedFilter对象，表示新添加的更新条件或条件的组合。如果是组合条件，则组合条件将会被括号组合在一起。
     * @return          ACStoreBatchUpdate  本ACStoreBatchUpdate对象。
     */
    public function orWhere($filter) {
        if ($this->where == NULL) {
            return $this;
        }
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where->appendFilter($filter, FALSE);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where->linkTo($filter, FALSE);
        }
        return $this;
    }
    
    /**
     * 清除更新条件。
     * @return ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function clearWhere() {
    	$this->where = NULL;
    	return $this;
    }
    
    /**
     * 将某一列设置为一个值。
     * @param $column string 是列名。如果之前设置过该列，则之前设置的值会被覆盖。
     * @param $value  mixed	 是值，其类型与列的类型要匹配。
     * @return ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function set($column, $value) {
        $this->expressions[$column] = array('key' => $column, 'operator' => self::$Oper_SET, 'value' => $value);
        return $this;
    }
    
    /**
     * 将某一列加上一个值。
     * @param $column string 是列名。如果之前设置过该列，则之前设置的值会被覆盖。
     * @param $value  mixed	 是值，其类型与列的类型要匹配。
     * @return ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function inc($column, $value) {
        $this->expressions[$column] = array('key' => $column, 'operator' => self::$Oper_ADD, 'value' => $value);
        return $this;
    }
    
    /**
     * 将某一列减去一个值。
     * @param $column string 是列名。如果之前设置过该列，则之前设置的值会被覆盖。
     * @param $value  mixed	 是值，其类型与列的类型要匹配。
     * @return ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function dec($column, $value) {
        $this->expressions[$column] = array('key' => $column, 'operator' => self::$Oper_DEC, 'value' => $value);
        return $this;
    }
    
    /**
     * 清除更新值。
     * @return ACStoreBatchUpdate   本ACStoreBatchUpdate对象。
     */
    public function clearColumns() {
        $this->expressions = array();
        return $this;
    }
    
    /**
     * 执行更新操作。
     * @return bool 返回TRUE或FALSE。返回FALSE表示有错误发生。此时可调用getLastError()方法获取错误消息。
     */
    public function execute() {
	    $request = new ACRequest($this->getName(), 'batchUpdate', $this->getVersion());
        $updateParam = $this->toBatchUpdateParam();
        $request->setPayloadAsJSON(json_encode(array('className' => $this->getClassName(), 'batchUpdateParam' => $updateParam)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
	        $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }
    
    /**
     * 将本对象转换为AbleCloud存储服务的batchUpdateParam结构。
     * @return array    返回一个关联数组，代表存储服务的batchUpdateParam参数。
     */
    private function toBatchUpdateParam() {
        $batchUpdateParam = array();
        // entityGroupKeys
        if (!empty($this->entityGroupKeyValues) && count($this->entityGroupKeyValues) > 0)
            $batchUpdateParam = array_merge($batchUpdateParam, array('entityGroupKeys' => $this->entityGroupKeyValues));
        // where
        if (!empty($this->where))
            $batchUpdateParam = array_merge($batchUpdateParam, array('filters' => $this->where->toArray()));
        // expressions
        if (count($this->expressions) > 0) {
            $a = array();
            foreach ($this->expressions as $k => $v) {
                array_push($a, $v);
            }
            $batchUpdateParam = array_merge($batchUpdateParam, array('exprs' => $a));
        }
        return $batchUpdateParam;
    }
    
    /// 要操作的数据集的名字。
    private $className;
    // 分区键。为NULL或空数组时，表示不分区。
    private $entityGroupKeyValues;
    /// 操作条件。
    private $where;
    /// 要修改其值的列。
    private $expressions;
    
    private static $Oper_SET = 1;
    private static $Oper_ADD = 2;
    private static $Oper_DEC = 3;
}
