<?php

require_once(dirname(__FILE__) . '/ACStoreFilter.php');

/**
 * 复杂查询条件：单个或多个ACStoreFilter通过逻辑操作符组合起来形成复杂查询条件。如：“time < 1436170756857177”，“time > 1436170756857177 AND name = '张三'”等都是复杂查询条件。
 * 多个复杂条件连接在一起时，同一个复杂条件内的条件会被包括在一个括弧内，如：(time > 1436170756857177 AND name = '张三') OR (deviceId = 12 AND name = '张三')。
 * 其中，条件“time > 1436170756857177 AND name = '张三'”是一个负载查询条件的内容，“deviceId = 12 AND name = '张三'”是另一个复杂查询条件的内容。
 * 这两个复杂查询条件通过逻辑或（OR）连接起来。
 */
class ACStoreComplicatedFilter {
    /**
     * 构造函数。
     * @param $filter ACStoreFitler对象，是初始化复杂查询条件的简单条件。
     */
    function __construct($filter) {
        $this->headFilter = $filter;
        $this->nextComplicatedFilter = NULL;
        $this->andToPrevious         = true;
    }

    /**
     * 将一个简单查询条件ACStoreFilter链接至本对象所包含的简单查询条件链表的末尾。
     * @param $filter       ACStoreFilter   ACStoreFilter对象，表示要被连接的查询条件。
     * @param $logicalAnd   bool            为true时，表示以逻辑“与”的关系连接$filter，否则表示以逻辑“或”的关系连接$filter。
     * @return              ACStoreComplicatedFilter 本ACStoreComplicatedFilter对象。
     */
    public function appendFilter($filter, $logicalAnd = true) {
        if ($filter == NULL)
            return $this;
        $tail = $this->headFilter;
        while ($tail->nextFilter != NULL && $tail->nextFilter !== $this->headFilter)
            $tail = $tail->nextFilter;
        $filter->andToPrevious = $logicalAnd;
        $tail->nextFilter = $filter;
        return $this;
    }

    /**
     * 计算本对象所含的简单查询对象（ACStoreFilter）链表中元素的数目。
     * @return int 返回链表中的简单查询对象（ACStoreFilter）的数目。
     */
    public function countFilters() {
        $num = 0;
        $iter = $this->headFilter;
        while ($iter != NULL) {
            $iter = $iter->nextFilter;
            if ($iter === $this)
                break;
            $num += 1;
        }
        return $num;
    }

    /**
     * 将另一个复杂查询条件（ACStoreComplicatedFilter）链接至本对象所属复杂查询条件链表的末尾。
     * @param $complicatedFilter ACStoreComplicatedFilter对象，表示要链接的复杂查询条件。
     * @param $logicalAnd 为true时，表示以逻辑与操作链接$complicatedFilter对象，否则表示以逻辑或操作链接$complicatedFilter对象。
     * @return 位于链表末尾的ACStoreComplicatedFilter对象。
     */
    public function linkTo($complicatedFilter, $logicalAnd) {
        $tail = $this;
        while ($tail->nextComplicatedFilter != NULL && $tail->nextComplicatedFilter !== $this) {
            $tail = $tail->nextComplicatedFilter;
        }
        if ($complicatedFilter != NULL) {
            $complicatedFilter->andToPrevious = $logicalAnd;
            $tail->nextComplicatedFilter = $complicatedFilter;
            $tail = $complicatedFilter;
        }
        return $tail;
    }

    /**
     * 假设以本复杂查询对象为链表的头，计算该链表中所包含的复杂查询条件对象（ACStoreComplicatedFilter）的数目。
     * @return 返回链表中复杂查询条件对象（ACStoreComplicatedFilter）的数目。
     */
    public function countComplicatedFilters() {
        $num = 0;
        $iter = $this;
        while ($iter != NULL) {
            $iter = $iter->nextComplicatedFilter;
            if ($iter === $this)
                break;
            $num += 1;
        }
        return $num;
    }

    /**
     * 将以本对象为头元素的复杂查询对象链表转化为数组结构。
     * @return 以数组结构表达的复杂查询条件。
     */
    public function toArray() {
        $filters = array();
        $iter = $this;
        $count = 0;
        while ($iter != NULL) {
            if ($iter->countFilters() > 0) {
                $filter = array('connector' => 1);
                if (!$iter->andToPrevious)
                    $filter['connector'] = 2;
                $nodes = array();
                $head = $iter->headFilter;
                while ($head != NULL) {
                    $object = array('key' => $head->columnName, 'operator' => $head->operator, 'value' => $head->value);
                    if ($head->andToPrevious)
                        $object['connector'] = 1;
                    else
                        $object['connector'] = 2;
                    array_push($nodes, $object);
                    $head = $head->nextFilter;
                    if ($head === $iter->headFilter)
                        break;
                }
                $filter['filter'] = $nodes;
                array_push($filters, $filter);
            }
            $iter = $iter->nextComplicatedFilter;
            if ($iter === $this)
                break;
        }
        return $filters;
    }

    /// ACStoreFilter对象链表的头。
    private $headFilter;
    // ACStoreComplicatedFilter对象链表
    public $nextComplicatedFilter;     /// 所链接的后一个ACStoreComplicatedFilter对象。缺省为NULL，表示没有下一个复杂查询条件。
    public $andToPrevious;             /// 与后一个ACStoreComplicatedFilter对象的链接关系：true-逻辑与；false-逻辑或。
}
