<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACStoreComplicatedFilter.php');

/**
 * AbleCloud数据存储服务的修改单条数据记录的方法。
 * 本方法先检查符合条件的记录是否存在。如存在则更新该记录，否则不执行操作。
 */
class ACStoreModify extends ACService {

    /**
     * 构造函数。
     * @param $name     string      数据存储服务的名字。
     * @param $version  int         数据存储服务的版本。
     * @param $context  ACContext   ACContext对象，表示访问该远程服务所依赖的环境信息。
     * @param $className    string  要修改的记录所属的数据集的名字。
     */
    function __construct($name, $version, $context, $className) {
        parent::__construct($name, $version, $context);
        $this->className = $className;
        $this->filter    = array();
        $this->exprs     = array();
    }

    /**
     * 设置要更新的记录
     * @param $row      array   以键值对的方式（关联数组）描述的要被更新的记录。$row参数应指定所有主键的值，用于定位要被更新的记录。
     * 如果主键指定的记录不存在，则不执行操作。
     * @return          ACStoreModify   返回本对象。
     */
    public function where($row) {
        $this->filter = $row;
        return $this;
    }

    /**
     * 设置要更新的列的值：将记录中该列的值修改为指定的值。
     * @param $column   string  要修改的列的名字。
     * @param $value    mixed   要修改的目标值。
     * @return          ACStoreModify   返回本对象。
     */
    public function set($column, $value) {
        array_push($this->exprs, array(self::$KEY => $column, self::$OPERATOR => self::$SET, self::$VALUE => $value));
        return $this;
    }

    /**
     * 设置更新某列的值为当前值加上$value后的结果。
     * @param $column   string  要修改的列的名字。
     * @param $value    int|float|double    修改的增量值。
     * @return          ACStoreModify       返回本对象。
     */
    public function inc($column, $value) {
        array_push($this->exprs, array(self::$KEY => $column, self::$OPERATOR => self::$INC, self::$VALUE => $value));
        return $this;
    }

    /**
     * 设置更新某列的值为当前值减去$value后的结果。
     * @param $column   string  要修改的列的名字。
     * @param $value    int|float|double    修改的减量值。
     * @return          ACStoreModify       返回本对象。
     */
    public function dec($column, $value) {
        array_push($this->exprs, array(self::$KEY => $column, self::$OPERATOR => self::$DEC, self::$VALUE => $value));
        return $this;
    }

    /**
     * 执行查询，返回查询结果。
     * @return bool 操作成功返回TRUE；否则返回FALSE，并且可调用getLastError()方法获取错误消息。
     */
    public function execute() {
        $request = new ACRequest($this->getName(), 'modify', $this->getVersion());
        $data = array(
            'className'   => $this->className,
            'modifyParam' => array(
                'filter'  => $this->filter,
                'exprs'   => $this->exprs
            )
        );
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    private $className;
    private $filter;
    private $exprs;
    static private $SET = 1;
    static private $INC = 2;
    static private $DEC = 3;
    static private $KEY = 'key';
    static private $OPERATOR = 'operator';
    static private $VALUE    = 'value';
}