<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACStoreComplicatedFilter.php');

/**
 * AbleCloud数据存储服务的查询工具。
 */
class ACStoreScanner extends ACService {
    /**
     * 构造函数。
     * @param $name                 string      字符串。要被查询的数据集的名字。
     * @param $partitionKeyValues   array       以键值对的方式（如关联数组等）描述的查询数据集时所使用的分区键的值。如果数据集没有分区，则使用NULL。
     * @param $storeName            string      数据存储服务的名字。
	 * @param $storeVersion         int         数据存储服务的版本。
	 * @param $context              ACContext   ACContext对象，表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $partitionKeyValues = NULL, $storeName = '', $storeVersion = 1, $context = NULL) {
        $this->className = $name;
        $this->entityGroupKeyValues = $partitionKeyValues;
        $this->selectColumns  = array();
        $this->groupByColumns = array();
        $this->aggregates     = array();
        $this->orderByColumns = array();
        $this->limit = 0;
        $this->offset = 0;
        $this->where = NULL;
        $this->acStoreName    = $storeName;
        $this->acStoreVersion = empty($storeVersion) ? 1 : intval($storeVersion);
        $this->acContext      = $context;
    }
    
    /**
	 * 取服务的名字。
	 * @return 返回服务的名字。
	 */
	public function getName() {
		return $this->acStoreName;
	}

	/**
	 * 取服务的主版本值。
	 * @return 返回一个整数，表示服务的主版本值。
	 */
	public function getVersion() {
		return $this->acStoreVersion;
	}

	/**
	 * 取访问该服务所依赖的环境信息。
	 * @return 返回ACContext对象，表示环境信息。
	 */
	public function getContext() {
		return $this->acContext;
	}

    /**
     * 取要查询的数据集的名字。
     * @return 要查询的数据集的名字。
     */
    public function getClassName() {
        return $this->className;
    }

    /**
     * 设置要被查询的数据列。该方法可被多次调用，每次调用指定的数据列将被添加至之前设置的集合中。
     * 该方法的调用方式，如查询单列：$scanner->select('deviceId')；或查询多列：$scanner->select('deviceId', 'time', 'message')。
     * @param $columnNames 字符串，表示要被查询的数据列的名字。
     * @return 本ACStoreScanner对象。
     */
    public function select(...$columnNames) {
        foreach ($columnNames as $col)
            array_push($this->selectColumns, $col);
        return $this;
    }
    
    /**
     * 清除要查询的目标列。
     * @return 本ACStoreScanner对象。
     */
    public function clearSelection() {
    	$this->selectColumns = array();
    	return $this;
    }

    /**
     * 设置查询条件。该方法如果被多次调用，则后续调用传入的参数将覆盖之前设置的所有查询条件。
     * @param $complicatedFilter ACStoreComplicatedFilter对象，表示查询的过滤条件。
     * @return 本ACStoreScanner对象。
     */
    public function where($complicatedFilter) {
        if (!is_a($complicatedFilter, 'ACStoreComplicatedFilter')) {
            return $this;
        }
        $this->where = $complicatedFilter;
        return $this;
    }

    /**
     * 设置查询条件。该方法如果被多次调用，则后续调用传入的参数将覆盖之前设置的所有查询条件。
     * @param $filter ACStoreFilter或ACStoreComplicatedFilter对象，表示查询条件。
     * 如果是ACStoreFilter对象，将先生成一个组合条件，并将该对象放置在组合条件中，然后再设置为过滤条件。
     * @return 本ACStoreScanner对象。
     */
    public function whereExt($filter) {
        if ($filter == NULL) {
            $this->where = NULL;
            return NULL;
        }
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where = new ACStoreComplicatedFilter($filter);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where = $filter;
        }
        return $this;
    }

    /**
     * 以逻辑“与”的关系添加一个查询条件或条件的组合。
     * @details 该方法应该在调用了ACStoreScanner::where方法或ACStoreScanner::whereExt之后再调用。
     * @param $filter ACStoreFilter或ACStoreComplicatedFilter对象，表示新添加的查询条件或条件的组合。
     * 如果是ACStoreFilter，那该条件将会被添加至已存在的第一个组合条件中；如果是组合条件，则组合条件将会跟已存在的组合条件并列组合起来。
     * @return 本ACStoreScanner对象。
     */
    public function andWhere($filter) {
        if ($this->where == NULL)
            return $this;
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where->appendFilter($filter, TRUE);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where->linkTo($filter, TRUE);
        }
        return $this;
    }

    /**
     * 以逻辑“或”的关系添加一个查询条件或条件的组合。
     * @details 该方法应该在调用了ACStoreScanner::where方法或ACStoreScanner::whereExt之后再调用。
     * @param $filter ACStoreFilter或ACStoreComplicatedFilter对象，表示新添加的查询条件或条件的组合。
     * 如果是ACStoreFilter，那该条件将会被添加至已存在的第一个组合条件中；如果是组合条件，则组合条件将会跟已存在的组合条件并列组合起来。
     * @return 本ACStoreScanner对象。
     */
    public function orWhere($filter) {
        if ($this->where == NULL) {
            return $this;
        }
        if (is_a($filter, 'ACStoreFilter')) {
            $this->where->appendFilter($filter, FALSE);
        } else if (is_a($filter, 'ACStoreComplicatedFilter')) {
            $this->where->linkTo($filter, FALSE);
        }
        return $this;
    }
    
    /**
     * 清除查询条件。
     * @return 本ACStoreScanner对象。
     */
    public function clearWhere() {
    	$this->where = NULL;
    	return $this;
    }

    /**
     * 增加对查询结果进行排序的列及方式。
     * @param $columnName 字符串，是用作排序的列的名字。如该名字与之前添加过的列重名，则以最后一次设置的排序方式为准。
     * @param $asc 布尔值，为true时表示依据$columnName升序排序，否则表示以降序排序。
     * @return 本ACStoreScanner对象。
     */
    public function addOrderBy($columnName, $asc) {
        if ($asc)
            $this->orderByColumns[$columnName] = 1;
        else
            $this->orderByColumns[$columnName] = 2;
        return $this;
    }
    
    /**
     * 清除排序条件。
     * @return 本ACStoreScanner对象。
     */
    public function clearOrderBy() {
    	$this->orderByColumns = array();
    	return $this;
    }

    /**
     * 设置查询结果集的分组数据列。该方法可被多次调用，每次调用指定的数据列将被添加至之前设置的集合中。
     * @param $columnNames 字符串，是分组所依据的数据列的名字。
     * @return 本ACStoreScanner对象。
     */
    public function groupBy(...$columnNames) {
        foreach ($columnNames as $col)
            array_push($this->groupByColumns, $col);
        $this->groupByColumns = array_unique($this->groupByColumns, SORT_STRING);
        return $this;
    }
    
    /**
     * 清除分组参数。
     * @return 本ACStoreScanner对象。
     */
    public function clearGroupBy() {
    	$this->groupByColumns = array();
    	return $this;
    }
    
    /**
     * 设置置扫描的offset，默认为0。
     * @param $number 非负整数。
     * @return 本ACStoreScanner对象。
     */
    public function offset($number) {
    	$number = intval($number);
    	if ($number < 0)
    		$number = 0;
    	$this->offset = $number;
    	return $this;
    }

    /**
     * 设置查询结果集的最大记录数。
     * @param $number 非负整数，指定查询结果集中的最大记录数。其取值范围限制为闭区间[0, 1000]。如果为0（缺省值），其效果等于取值为1000。
     * @return 本ACStoreScanner对象。
     */
    public function limit($number) {
        $number = intval($number);
        if ($number < 0)
            $number = 0;
        if ($number > 1000)
            $number = 1000;
        $this->limit = $number;
        return $this;
    }
    
    /**
     * 清除Offset及Limit参数。
     * @return 本ACStoreScanner对象。
     */
    public function clearOffsetAndLimit() {
    	$this->offset = 0;
    	$this->limit  = 0;
    	return $this;
    }

    /**
     * 添加聚集函数COUNT()为查询结果列。
     * @return 本ACStoreScanner对象。
     */
    public function count() {
        $this->aggregates['count'] = array('*');
        return $this;
    }

    /**
     * 添加聚集函数SUM()为查询结果列。该方法可被多次调用，每次调用指定的数据列将被添加至之前设置的集合中。
     * @param $columnName SUM()函数作用的目标数据列。
     * @return 本ACStoreScanner对象。
     */
    public function sum($columnName) {
        if (!array_key_exists('sum', $this->aggregates)) {
            $this->aggregates['sum'] = array($columnName);
        } else {
            array_push($this->aggregates['sum'], $columnName);
        }
        return $this;
    }

    /**
     * 添加聚集函数AVG()为查询结果列。该方法可被多次调用，每次调用指定的数据列将被添加至之前设置的集合中。
     * @param $columnName AVG()函数作用的目标数据列。
     * @return 本ACStoreScanner对象。
     */
    public function avg($columnName) {
        if (!array_key_exists('avg', $this->aggregates)) {
            $this->aggregates['avg'] = array($columnName);
        } else {
            array_push($this->aggregates['avg'], $columnName);
        }
        return $this;
    }

    /**
     * 添加聚集函数MAX()为查询结果列。该方法可被多次调用，每次调用指定的数据列将被添加至之前设置的集合中。
     * @param $columnName MAX()函数作用的目标数据列。
     * @return 本ACStoreScanner对象。
     */
    public function max($columnName) {
        if (!array_key_exists('max', $this->aggregates)) {
            $this->aggregates['max'] = array($columnName);
        } else {
            array_push($this->aggregates['max'], $columnName);
        }
        return $this;
    }

    /**
     * 添加聚集函数MIN()为查询结果列。该方法可被多次调用，每次调用指定的数据列将被添加至之前设置的集合中。
     * @param $columnName MIN()函数作用的目标数据列。
     * @return 本ACStoreScanner对象。
     */
    public function min($columnName) {
        if (!array_key_exists('min', $this->aggregates)) {
            $this->aggregates['min'] = array($columnName);
        } else {
            array_push($this->aggregates['min'], $columnName);
        }
        return $this;
    }
    
    /**
     * 清除Aggregate参数。
     * @return 本ACStoreScanner对象。
     */
    public function clearAggregates() {
    	$this->aggregates = array();
    	return $this;
    }

    /**
     * 执行查询，返回查询结果。
     * @return - 返回一个数组。数组中的每个元素是一个关联数组，表示一条数据记录。
     *  - 返回FALSE时，表示有错误发生。此时可调用getLastError()方法获取错误消息。
     */
    public function execute() {
    	$request = new ACRequest($this->getName(), 'scan', $this->getVersion());
        $scanParam = $this->toScanParam();
        $request->setPayloadAsJSON(json_encode(array('className' => $this->getClassName(), 'scanParam' => $scanParam)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $payload = json_decode($response->getResponse(), true);
            if (is_array($payload['objects'])) {
                return $payload['objects'];
            } else {
                return array();
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 设置访问ACStore服务的参数。
     * @details 调用ACStoreScanner::execute之前需要调用本方法配置ACStore的访问信息。
     * @param $serviceName 字符串，是ACStore服务的名字。
     * @param $serviceVersion 整数，是ACStore服务的主版本号。
     * @param $acContext ACContext对象，是访问ACStore服务的上下文环境。
     * @return 本ACStoreScanner对象。
     */
    public function setACStore($serviceName, $serviceVersion, $acContext) {
        $this->acStoreName    = $serviceName;
        $this->acStoreVersion = $serviceVersion;
        $this->acContext      = $acContext;
        return $this;
    }

    /**
     * 将本对象转换为AbleCloud存储服务的scanParam结构。
     * @return 返回一个关联数组，代表存储服务的scanParam参数。
     */
    public function toScanParam() {
        $scanParam = array();
        // entityGroupKeys
        if ($this->entityGroupKeyValues != NULL && count($this->entityGroupKeyValues) > 0)
            $scanParam = array_merge($scanParam, array('entityGroupKeys' => $this->entityGroupKeyValues));
        // select
        if (count($this->selectColumns) > 0)
            $scanParam = array_merge($scanParam, array('select' => $this->selectColumns));
        // where
        if ($this->where != NULL)
            $scanParam = array_merge($scanParam, array('filters' => $this->where->toArray()));
        // orderBy
        if (count($this->orderByColumns) > 0) {
            $byKeys = array();
            foreach ($this->orderByColumns as $key => $val)
                array_push($byKeys, array('key' => $key, 'order' => $val));
            $scanParam = array_merge($scanParam, array('orderBy' => $byKeys));
        }
        // groupBy
        if (count($this->groupByColumns) > 0)
            $scanParam = array_merge($scanParam, array('groupBy' => $this->groupByColumns));
        // offset
        if ($this->offset > 0)
            $scanParam = array_merge($scanParam, array('offset' => $this->offset));
        // limit
        if ($this->limit > 0)
            $scanParam = array_merge($scanParam, array('limit' => $this->limit));
        else if ($this->limit == 0)
            $scanParam = array_merge($scanParam, array('limit' => 1000));
        // aggregate
        if (count($this->aggregates) > 0)
            $scanParam = array_merge($scanParam, array('aggregate' => $this->aggregates));
        return $scanParam;
    }

	/// 要查询的数据集的名字。
    private $className;
    /// 分区键。为NULL或空数组时，表示不分区。
    private $entityGroupKeyValues;
    /// 要查询的目标数据列。
    private $selectColumns;
    /// 查询条件。
    private $where;
    /// 用于排序的列。
    private $orderByColumns;
    /// 用作分组的列。
    private $groupByColumns;
    /// 查询条件的 offset 参数。
    private $offset;
    /// 查询条件的 limit 参数。
    private $limit;
    /// 聚合函数。
    private $aggregates;
    /// 字符串，是ACStore服务的名字。
    private $acStoreName;
    /// 字符串，是ACStore服务的主版本号。
    private $acStoreVersion;
    /// ACContext对象，是访问ACStore服务的上下文环境。
    private $acContext;
}
