<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACTimerTask.php');

/**
 * APP端定时任务服务。
 */
class ACTimerTaskMgr extends ACService {
    /**
     * 构造函数。
     * @param $name     string      AbleCloud APP端定时任务管理服务的名字。
     * @param $version  int         AbleCloud APP端定时任务管理服务的版本。
     * @param $context  ACContext   ACContext对象，表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
    }

    /**
     * 添加一个新定时任务。
     * @param $task ACTimerTask对象，表示要新添加的任务。
     * @param $user ACUser对象，表示定义该任务的用户。
     * @return 返回TRUE表示操作成功，否则表示操作失败。失败时，可调用getLastError()获取错误信息。
     */
    public function addTask($task, $user) {
        $request = new ACRequest($this->getName(), 'addTask', $this->getVersion());
        $request->setUser($user);
        $request->addParameter('name', $task->name);
        $request->addParameter('description', $task->description);
        $request->addParameter('timePoint', $task->timePoint->format('Y-m-d H:i:s'));
        $request->addParameter('timeZone', $task->timePoint->getTimezone()->getName());
        $request->addParameter('timeCycle', $task->timeCycle);
        $request->addParameter('deviceId', intval($task->deviceId));
        $request->addParameter('messageCode', intval($task->messageCode));
        $request->addParameter('taskFlag', intval($task->taskFlag));
        $request->setPayloadAsStream($task->message);
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 修改指定的定时任务。
     * @param $taskId 整数，表示要被修改的任务的ID。
     * @param $task ACTimerTask对象，表示任务的新内容。
     * @param $user ACUser对象，表示定义$taskId这个任务的用户。
     * @return 返回TRUE表示操作成功，否则表示操作失败。失败时，可调用getLastError()获取错误信息。
     */
    public function modifyTask($taskId, $task, $user) {
        $request = new ACRequest($this->getName(), 'modifyTask', $this->getVersion());
        $request->setUser($user);
        $request->addParameter('taskId', $taskId);
        $request->addParameter('name', $task->name);
        $request->addParameter('description', $task->description);
        $request->addParameter('timePoint', $task->timePoint->format('Y-m-d H:i:s'));
        $request->addParameter('timeZone', $task->timePoint->getTimezone()->getName());
        $request->addParameter('timeCycle', $task->timeCycle);
        $request->addParameter('deviceId', intval($task->deviceId));
        $request->addParameter('messageCode', intval($task->messageCode));
        $request->addParameter('taskFlag', intval($task->taskFlag));
        $request->setPayloadAsStream($task->message);
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 查询用户针对某设备定制的定时任务。
     * @param $user ACUser对象，表示创建该定时任务的用户。
     * @param $deviceId 整数，是设备的逻辑ID。
     * @return 操作成功时，返回一个数组，数组的元素是ACTimerTask对象。操作失败时返回NULL，并且可调用getLastError()获取错误信息。
     */
    public function listTasks($user, $deviceId) {
        $request = new ACRequest($this->getName(), 'listTasks', $this->getVersion());
        $request->setUser($user);
        $request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            $json = json_decode($response->getResponse(), true);
            $list = array();
            foreach ($json['tasks'] as $t) {
                $task = new ACTimerTask();
                $task->userId   = $user->getId();
                $task->deviceId = intval($deviceId);
                $task->taskId   = intval($t['taskId']);
                $task->status   = intval($t['status']);
                $task->taskFlag = intval($t['taskFlag']);
                $task->createTime  = new DateTime($t['createTime']);
                $task->modifyTime  = new DateTime($t['modifyTime']);
                $task->timeCycle   = $t['timeCycle'];
                $task->description = $t['description'];
                $task->name        = $t['name'];
                $task->timePoint   = DateTime::createFromFormat('Y-m-d H:i:s', $t['timePoint'], new DateTimeZone($t['timeZone']));
                array_push($list, $task);
            }
            return $list;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 删除一个定时任务。
     * @param $user ACUser对象，是要被删除的任务所关联的用户。
     * @param $deviceId 整数，是要被删除的任务所关联的设备的逻辑ID。
     * @param $taskId 整数，是要被删除的任务的ID。
     * @return 操作成功时返回TRUE；操作失败时返回FALSE，并且可调用getLastError()获取错误信息。
     */
    public function deleteTask($user, $deviceId, $taskId) {
        $request = new ACRequest($this->getName(), 'deleteTask', $this->getVersion());
        $request->setUser($user);
        $request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'taskId' => intval($taskId))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 停止一个定时任务。
     * @param $user ACUser对象，是要被停止的任务所关联的用户。
     * @param $deviceId 整数，是要被停止的任务所关联的设备的逻辑ID。
     * @param $taskId 整数，是要被停止的任务的ID。
     * @return 操作成功时返回TRUE；操作失败时返回FALSE，并且可调用getLastError()获取错误信息。
     */
    public function stopTask($user, $deviceId, $taskId) {
        $request = new ACRequest($this->getName(), 'stopTask', $this->getVersion());
        $request->setUser($user);
        $request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'taskId' => intval($taskId))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }

    /**
     * 启动一个定时任务。
     * @param $user ACUser对象，是要被启动的任务所关联的用户。
     * @param $deviceId 整数，是要被启动的任务所关联的设备的逻辑ID。
     * @param $taskId 整数，是要被启动的任务的ID。
     * @return 操作成功时返回TRUE；操作失败时返回FALSE，并且可调用getLastError()获取错误信息。
     */
    public function startTask($user, $deviceId, $taskId) {
        $request = new ACRequest($this->getName(), 'startTask', $this->getVersion());
        $request->setUser($user);
        $request->setPayloadAsJSON(json_encode(array('deviceId' => intval($deviceId), 'taskId' => intval($taskId))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $this->resetLastError();
            return TRUE;
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return FALSE;
        }
    }
}