<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACDeviceInfo.php');

/**
 * AbleCloud设备入库服务。
 */
class ACWarehouseMgr extends ACService {
    /**
     * 构造函数。
     * @param $name		string		AbleCloud设备入库管理服务的名字。
     * @param $version	int			AbleCloud设备入库管理服务的版本。
     * @param $context	ACContext	ACContext对象，表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
    }

    /**
     * 查询已入库设备的数目。
     * @param $subDomain    string  指定要查询的设备所属的子域的名字。如为NULL或空字符串表示不区分子域。
     * @return              int     返回非负整数表示设备数目。返回负整数表示查询失败，可调用getLastError()方法获取错误信息。
     */
    public function getDeviceCount($subDomain = '') {
        $data = $this->getDeviceCountExt($subDomain);
        if (is_array($data)) {
            if (array_key_exists('count', $data)) {
                return $data['count'];
            } else {
                $this->resetLastError(-1, 'Unknown response from getDeviceCount: ' . json_encode($data));
                return -1;
            }
        } else {
            return -1;
        }
    }
    
    /**
     * 统计入库设备的信息。
     * @param $subDomain    string  指定要统计的设备所属的子域的名字。如为NULL或空字符串表示不区分子域。
     * @return              array|NULL  操作成功时返回一个包含如下三个键的关联数组：count、activeCount、onlineCount，分别表示入库的设备总数、已激活设备数，以及当前在线设备数。
     *                                  操作失败时返回NULL，并且可以调用getLastError()方法获取错误信息。
     */
    public function getDeviceCountExt($subDomain = '') {
        if (is_null($subDomain))
            $subDomain = '';
        $request = new ACRequest($this->getName(), 'getDeviceCount', $this->getVersion(), strval($subDomain));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json)) {
                $this->resetLastError();
                return $json;
            } else {
                $this->resetLastError(-1, 'Unknown response from getDeviceCount: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 批量查询设备信息。
     * @param $subDomain    string          指定要查询的设备所属的子域的名字。如为NULL或空字符串表示不区分子域。
     * @param $offset       int             $offset与$limit参数用于实现分页查询的效果。$offset是从0开始的偏移量，表示返回设备列表中从第offset位置开始的共limit个设备的信息。
     * @param $limit        int             $offset与$limit参数用于实现分页查询的效果。$limit是正整数，表示返回设备列表中从第offset位置开始的共limit个设备的信息。
     * @return              array | NULL    操作成功时返回ACDeviceInfo对象的数组，否则返回NULL。失败时可调用getLastError()方法获取错误信息。
     */
    public function listDevices($subDomain, $offset, $limit) {
        if (is_null($subDomain))
            $subDomain = '';
        $data = array('offset' => intval($offset), 'count' => intval($limit), 'order' => 0);
        $request = new ACRequest($this->getName(), 'listDevices', $this->getVersion(), strval($subDomain));
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('devices', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['devices'] as $v) {
                    $dev = ACDeviceInfo::fromObject($v);
                    if (!is_null($dev)) {
                        array_push($list, $dev);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from listDevices: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 查询设备信息。
     * @param $physicalDeviceId string              拟查询的设备的物理ID。
     * @return                  ACDeviceInfo | NULL 操作成功时返回ACDeviceInfo对象，失败时返回NULL。失败时可调用getLastError()方法获取错误信息。
     */
    public function getDeviceInfo($physicalDeviceId) {
        $data = array('physicalDeviceId' => strval($physicalDeviceId));
        $request = new ACRequest($this->getName(), 'getDeviceInfo', $this->getVersion());
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json)) {
                $this->resetLastError();
                $dev = ACDeviceInfo::fromObject($json);
                if (is_a($dev, 'ACDeviceInfo')) {
                    $dev->physicalDeviceId = strval($physicalDeviceId);
                }
                return $dev;
            } else {
                $this->resetLastError(-1, 'Unknown response from getDeviceInfo: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 批量查询设备信息。
     * @param $physicalDeviceIds    array           拟查询的设备的物理ID组成的数组。
     * @return                      array | NULL    操作成功时返回ACDeviceInfo对象的数组，否则返回NULL。失败时可调用getLastError()方法获取错误信息。
     */
    public function getDevicesInfo($physicalDeviceIds) {
        if (!is_array($physicalDeviceIds)) {
            $this->resetLastError(-1, 'invalid parameter');
            return NULL;
        }
        $data = array();
        foreach ($physicalDeviceIds as $pid) {
            array_push($data, array('subDomain' => '', 'physicalDeviceId' => $pid));
        }
        $request = new ACRequest($this->getName(), 'getDevicesInfo', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('devices' => $data)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('devices', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['devices'] as $v) {
                    $dev = ACDeviceInfo::fromObject($v);
                    if (!is_null($dev)) {
                        array_push($list, $dev);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getDevicesInfo: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 取设备的逻辑ID。
     * @param $subDomain        string  拟查询的设备所属的子域的名字。
     * @param $physicalDeviceId string  拟查询的设备的物理ID。
     * @return                  int     返回设备的逻辑ID。有效的逻辑ID是正整数。返回值小于0时表示操作失败，并可调用getLastError()方法获取错误信息。
     */
    public function getDeviceLogicalId($subDomain, $physicalDeviceId) {
        $data = array('physicalId' => strval($physicalDeviceId));
        $request = new ACRequest($this->getName(), 'getDeviceLogicalId', $this->getVersion(), $subDomain);
        $request->setPayloadAsJSON(json_encode($data));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json)) {
                $this->resetLastError();
                return $json['logicalId'];
            } else {
                $this->resetLastError(-1, 'Unknown response from getDeviceLogicalId: ' . json_encode($json));
                return -1;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return -1;
        }
    }
}
