<?php

require_once(dirname(__FILE__) . '/ACService.php');
require_once(dirname(__FILE__) . '/ACPM25.php');
require_once(dirname(__FILE__) . '/ACAQI.php');
require_once(dirname(__FILE__) . '/ACWeather.php');

/**
 * 天气服务。
 */
class ACWeatherMgr extends ACService {
    /**
     * 构造函数。
     * @param $name		string		AbleCloud天气服务的名字。
     * @param $version	int			AbleCloud天气服务的版本。
     * @param $context	ACContext	ACContext对象，表示访问该远程服务所依赖的环境信息。
     */
    function __construct($name, $version, $context) {
        parent::__construct($name, $version, $context);
    }

    /**
     * 获取指定地区的最新的PM2.5值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @return          ACPM25  操作成功时返回ACPM25对象；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLatestPM25($area) {
        $request = new ACRequest($this->getName(), 'getLatestPM25Data', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json)) {
                $this->resetLastError();
                return ACPM25::fromObject($json);
            } else {
                $this->resetLastError(-1, 'Unknown response from getLatestPM25Data: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最近N天的PM2.5值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @param   $day    int     指定要查询的天数范围，表示查询最近$day天以来的数据。取值范围为区间[0,7]，0也表示7天。
     * @return          array   操作成功时返回ACPM25对象组成的数组；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLastDaysPM25($area, $day) {
        $request = new ACRequest($this->getName(), 'getLastDaysPM25Data', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area, 'days' => intval($day))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('history', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['history'] as $v) {
                    $pm = ACPM25::fromObject($v);
                    if (!is_null($pm)) {
                        array_push($list, $pm);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getLastDaysPM25Data: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最近N小时的PM2.5值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @param   $hour   int     指定要查询的小时范围，表示查询最近$hour小时以来的数据。取值范围是区间[0,24]，0也表示24小时。
     * @return          array   操作成功时返回ACPM25对象组成的数组；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLastHoursPM25($area, $hour) {
        $request = new ACRequest($this->getName(), 'getLastHoursPM25Data', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area, 'hours' => intval($hour))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('history', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['history'] as $v) {
                    $pm = ACPM25::fromObject($v);
                    if (!is_null($pm)) {
                        array_push($list, $pm);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getLastHoursPM25Data: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最新的空气质量值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @return          ACAQI   操作成功时返回ACAQI对象；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLatestAqi($area) {
        $request = new ACRequest($this->getName(), 'getLatestAqi', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json)) {
                $this->resetLastError();
                return ACAQI::fromObject($json);
            } else {
                $this->resetLastError(-1, 'Unknown response from getLatestAqi: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最近N天的空气质量值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @param   $day    int     指定要查询的天数范围，表示查询最近$day天以来的数据。取值范围为区间[0,7]，0也表示7天。
     * @return          array   操作成功时返回ACAQI对象组成的数组；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLastDaysAqi($area, $day) {
        $request = new ACRequest($this->getName(), 'getLastDaysAqi', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area, 'days' => intval($day))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('history', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['history'] as $v) {
                    $aqi = ACAQI::fromObject($v);
                    if (!is_null($aqi)) {
                        array_push($list, $aqi);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getLastDaysAqi: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最近N小时的空气质量值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @param   $hour   int     指定要查询的小时范围，表示查询最近$hour小时以来的数据。取值范围是区间[0,24]，0也表示24小时。
     * @return          array   操作成功时返回ACAQI对象组成的数组；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLastHoursAqi($area, $hour) {
        $request = new ACRequest($this->getName(), 'getLastHoursAqi', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area, 'hours' => intval($hour))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('history', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['history'] as $v) {
                    $aqi = ACAQI::fromObject($v);
                    if (!is_null($aqi)) {
                        array_push($list, $aqi);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getLastDaysAqi: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最新的温度湿度值。
     * @param   $area   string      要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @return          ACWeather   操作成功时返回ACWeather对象；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLatestWeather($area) {
        $request = new ACRequest($this->getName(), 'getLatestWeatherData', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area)));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json)) {
                $this->resetLastError();
                return ACWeather::fromObject($json);
            } else {
                $this->resetLastError(-1, 'Unknown response from getLatestWeatherData: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最近N天的温度湿度值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @param   $day    int     指定要查询的天数范围，表示查询最近$day天以来的数据。取值范围为区间[0,7]，0也表示7天。
     * @return          array   操作成功时返回ACWeather对象组成的数组；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLastDaysWeather($area, $day) {
        $request = new ACRequest($this->getName(), 'getLastDaysWeatherData', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area, 'days' => intval($day))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('history', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['history'] as $v) {
                    $aqi = ACWeather::fromObject($v);
                    if (!is_null($aqi)) {
                        array_push($list, $aqi);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getLastDaysWeatherData: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }

    /**
     * 获取指定地区最近N天的温度湿度值。
     * @param   $area   string  要查询的地区的名字，如"北京"。只支持到地级市级别。
     * @param   $hour   int     指定要查询的小时范围，表示查询最近$hour小时以来的数据。取值范围是区间[0,24]，0也表示24小时。
     * @return          array   操作成功时返回ACWeather对象组成的数组；操作失败时返回NULL，并且可以调用方法getLastError()获取错误信息。
     */
    public function getLastHoursWeather($area, $hour) {
        $request = new ACRequest($this->getName(), 'getLastHoursWeatherData', $this->getVersion());
        $request->setPayloadAsJSON(json_encode(array('area' => $area, 'hours' => intval($hour))));
        $response = ACHttpClient::doRequest($request, $this->getContext());
        if ($response->isAck()) {
            $json = json_decode($response->getResponse(), true);
            if (is_array($json) && array_key_exists('history', $json)) {
                $this->resetLastError();
                $list = array();
                foreach ($json['history'] as $v) {
                    $aqi = ACWeather::fromObject($v);
                    if (!is_null($aqi)) {
                        array_push($list, $aqi);
                    }
                }
                return $list;
            } else {
                $this->resetLastError(-1, 'Unknown response from getLastHoursWeatherData: ' . json_encode($json));
                return NULL;
            }
        } else {
            $this->resetLastError($response->getErrorCode(), $response->getErrorMessage());
            return NULL;
        }
    }
}