<?php

require_once(dirname(dirname(__FILE__)) . '/ablecloud/ACClient.php');

/**
 * AbleCloud-微信公众号服务接口。
 * 主要用于同步用户信息及用户与设备的绑定关系。
 */
class ACBridgeWeChat {
	/**
	 * 构造函数。
	 * @param $accessToken	string	是微信公众号的Access Token。注意：微信的Access Token的有效期是有期限限制的。
	 * @param $jsTicket		string	字符串，是微信公众号的JS API Ticket。注意：微信公众号的JS API Ticket的有效期是有期限限制的。
	 */
	function __construct($accessToken, $jsTicket) {
		$this->accessToken = $accessToken;
		$this->jsTicket    = $jsTicket;
		$this->lastError   = array('errCode' => 0, 'errMessage' => '');
	}

	/**
	 * 设置/更新微信公众号的Access Token。
	 * @param $accessToken	string	是新的Access Token。
	 */
	public function setAccessToken($accessToken) {
		$this->accessToken = $accessToken;
	}

	/**
	 * 设置/更新微信公众号的JS API Ticket。
	 * @param $jsTicket		string	是新的JS API Ticket。
	 */
	public function setJsTicket($jsTicket) {
		$this->jsTicket = $jsTicket;
	}

	/// @name 微信推送的事件的处理方法
	//@{
	/**
	 * 微信推送消息：MsgType为"event"，Event为"subscribe"时的响应函数：将微信用户注册为开发者所提供服务的用户。
	 * @param $xmlMsg	string	微信推送的原始XML消息内容。
	 * @param $unionId	string	字符串。是关注公众号的用户在微信平台对应的UnionID。如果不提供该参数，则无法识别同一个用户关注开发者的多个微信公众号的情况。
	 * @return 			ACUser	操作成功时返回ACUser对象，表示新注册的用户信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function onEventSubscribe($xmlMsg, $unionId = '') {
		$message = simplexml_load_string($xmlMsg);
		if ($message === FALSE) {
			$this->resetLastError(-1, '解析XML消息失败。');
			return NULL;
		}
        $account = ACClient::getAccountMgr();
		$user = $account->registerByOpenId(strval($message->FromUserName), 'weixin', $unionId);
		if ($user === NULL) {
			$err = $account->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
		}
		return $user;
	}

	/**
	 * 微信推送消息：MsgType为"event"，Event为"unsubscribe"时的响应函数：在AbleCloud平台中解除该用户与所有设备的绑定关系。
	 * @param $xmlMsg	string	微信推送的原始XML消息内容。
	 * @return 			bool	返回TRUE表示操作成功，否则返回FALSE。失败时，可调用getLastError()方法获取错误消息。
	 */
	public function onEventUnsubscribe($xmlMsg) {
		$message = simplexml_load_string($xmlMsg);
		if ($message === FALSE) {
			$this->resetLastError(-1, '解析XML消息失败。');
			return FALSE;
		}
		$user = $this->getUser(strval($message->FromUserName));
		if ($user === NULL)
			return FALSE;
		$bindMgr = ACClient::getBindMgr();
		$devices = $bindMgr->listDevices($user);
		if ($devices === NULL) {
			$err = $bindMgr->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
			return FALSE;
		}
		foreach ($devices as $d) {
			$subDomain = $d->getSubDomainName();
			$bindMgr->unbindDevice($subDomain, $d->getId(), $user);
			$this->syncBindingsByDevice($d->getPhysicalId(), strval($message->ToUserName), $subDomain);
		}
		return TRUE;
	}

	/**
	 * 微信推送消息：MsgType为"device_event"，Event为"bind"时的响应函数：将微信记录的用户与设备的绑定关系同步到AbleCloud平台。
	 * @param $xmlMsg		string	微信推送的原始XML消息内容。
	 * @param $deviceName	string	字符串，表示设备的显示名。
	 * @param $subDomain	string	字符串，是将要绑定的设备在AbleCloud平台上所属的子域的名字。如果设备的二维码信息中包含了其所属子域的名字，则以二维码中的信息为准。
	 * @return 				ACDevice	操作成功时返回ACDevice对象，表示绑定后设备的信息。失败时返回NULL，并且可调用getLastError()方法获取错误消息。
	 */
	public function onDeviceEventBind($xmlMsg, $deviceName, $subDomain) {
		$message = simplexml_load_string($xmlMsg);
		if ($message === FALSE) {
			$this->resetLastError(-1, '解析XML消息失败。');
			return NULL;
		}
		$openId = strval($message->FromUserName);
		$user = $this->getUser($openId);
		if ($user === NULL)
			return NULL;
		$device = NULL;
		$shareCode = '';
		$content = strval($message->Content);
		if (isset($content) && strlen($content) > 0) {
			$content = base64_decode($content);
			// 处理设备管理员将设备分享给其他用户的情况。
			$contentInfo = json_decode($content, true);
			if (array_key_exists('sub_domain', $contentInfo) && strlen($contentInfo['sub_domain']) > 0)
				$subDomain = $contentInfo['sub_domain'];
			if (array_key_exists('share_code', $contentInfo))
				$shareCode = $contentInfo['share_code'];
		}
		$bindMgr = ACClient::getBindMgr();
		if (strlen($shareCode) > 0) {
			// 处理设备管理员将设备分享给其他用户的情况。
			$device = $bindMgr->bindDeviceWithShareCode($user, $shareCode);
			if ($device != NULL) {
				$this->syncBindings($openId);
			}
		} else {
			// 处理用户直接扫描设备的微信二维码绑定设备的情况。
			$device = $bindMgr->bindDevice($subDomain, strval($message->DeviceID), $deviceName, $user);
		}
		if ($device === NULL) {
			// 在AbleCloud平台中绑定设备失败。
			$err = $bindMgr->getLastError();
			error_log(json_encode($err));
			$this->resetLastError($err['errCode'], $err['errMessage']);
			// 通知微信解除设备绑定。
			$status = $this->wxUnbindDevice(strval($message->FromUserName), strval($message->DeviceID));
			if (!$status) {
				$err = $this->getLastError();
				error_log(json_encode($err));
			}
		}
		return $device;
	}

	/**
	 * 微信推送消息：MsgType为"device_event"，Event为"unbind"时的响应函数：将微信记录的用户与设备解除绑定的关系同步到AbleCloud平台。
	 * @details 本方法解绑设备后会自动通知微信平台执行额外的设备绑定关系同步。
	 * @param $xmlMsg		string	微信推送的原始XML消息内容。
	 * @param $subDomain	string	字符串，是要解绑的设备在AbleCloud平台上所属的子域的名字。
	 * @return 				bool	操作成功时返回TRUE；失败时返回FALSE，并且可调用getLastError()方法获取错误消息。
	 */
	public function onDeviceEventUnbind($xmlMsg, $subDomain) {
		$message = simplexml_load_string($xmlMsg);
		if ($message === FALSE) {
			$this->resetLastError(-1, '解析XML消息失败。');
			return FALSE;
		}
		$openId = strval($message->FromUserName);
		$user = $this->getUser($openId);
		if ($user === NULL)
			return FALSE;
		$bindMgr  = ACClient::getBindMgr();
		$deviceId = $bindMgr->getDeviceId($subDomain, strval($message->DeviceID));
		if ($deviceId == 0) {
			$err = $bindMgr->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
			return FALSE;
		}
		$retVal = $bindMgr->unbindDevice($subDomain, $deviceId, $user);
		if ($retVal) {
			$this->syncBindingsByDevice(strval($message->DeviceID), strval($message->DeviceType), $subDomain);
		} else {
			$err = $bindMgr->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
		}
		return $retVal;
	}

	/**
	 * 微信推送消息：MsgType为"device_event"，Event为"subscribe_status"时的响应函数：微信订阅设备状态信息（目前微信仅查询WIFI设备的状态信息）。
	 * @param $xmlMsg	string	微信推送的原始XML消息内容。
	 * @param $doSync	bool	标记是否执行设备绑定状态的同步操作。同步设备绑定状态是指同步该设备在AbleCloud平台与微信平台上分别记录的绑定状态。该过程可能需要耗费一定的时间。缺省值为TRUE，表示需要执行同步操作。
	 * @return			string	返回XML格式的文本内容，表示可回复给微信的设备状态信息。返回空字符串表示操作失败，并且可调用getLastError()方法获取错误消息。
	 */
	public function onDeviceEventSubscribeStatus($xmlMsg, $doSync = TRUE) {
		$message = simplexml_load_string($xmlMsg);
		if ($message === FALSE) {
			$this->resetLastError(-1, '解析XML消息失败。');
			return '';
		}
		$openId = strval($message->FromUserName);
		$physicalId = strval($message->DeviceID);
		$status = FALSE;
		if ($doSync) {
			$status = $this->syncBindings($openId, $physicalId);
		} else {
			$user = $this->getUser($openId);
			if ($user != NULL) {
				$bindMgr = ACClient::getBindMgr();
				$deviceList = $bindMgr->listDevices($user);
				if ($deviceList != NULL) {
					foreach ($deviceList as $dev) {
						if (strcmp($dev->getPhysicalId(), $physicalId) == 0) {
							$status = $bindMgr->isDeviceOnline($dev->getId());
							break;
						}
					}
				}
			}
		}
		$resp = '<xml>';
		$resp .= '<ToUserName><![CDATA[' . strval($message->FromUserName) . ']]></ToUserName>';
		$resp .= '<FromUserName><![CDATA[' . strval($message->ToUserName) . ']]></FromUserName>';
		$resp .= '<CreateTime>' . time() . '</CreateTime>';
		$resp .= '<MsgType><![CDATA[device_status]]></MsgType>';
		$resp .= '<DeviceType><![CDATA[' . strval($message->DeviceType) . ']]></DeviceType>';
		$resp .= '<DeviceID><![CDATA[' . strval($message->DeviceID) . ']]></DeviceID>';
		if ($status)
			$resp .= '<DeviceStatus>1</DeviceStatus>';
		else
			$resp .= '<DeviceStatus>0</DeviceStatus>';
		$resp .= '</xml>';
		return $resp;
	}

	/**
	 * 微信推送消息：MsgType为"device_event"，Event为"unsubscribe_status"时的响应函数：微信退订设备状态信息（目前微信仅查询WIFI设备的状态信息）。
	 * @param $xmlMsg	string	微信推送的原始XML消息内容。
	 * @param $doSync	bool	标记是否执行设备绑定状态的同步操作。同步设备绑定状态是指同步该设备在AbleCloud平台与微信平台上分别记录的绑定状态。该过程可能需要耗费一定的时间。缺省值为TRUE，表示需要执行同步操作。
	 * @return			string	返回XML格式的文本内容，表示可回复给微信的设备状态信息。返回空字符串表示操作失败，并且可调用getLastError()方法获取错误消息。
	 */
	public function onDeviceEventUnsubscribeStatus($xmlMsg, $doSync = TRUE) {
		return $this->onDeviceEventSubscribeStatus($xmlMsg, $doSync);
	}
	//@}

	/**
	 * 获取开发者用户信息。
	 * @param $openId	string	字符串，是要检查的微信用户的OpenID。
	 * @return			ACUser	操作成功时返回ACUser对象，表示AbleCloud平台存储的用户信息。返回NULL表示失败，并且可调用getLastError()方法获取错误消息。
	 */
	public function getUser($openId) {
		$account = ACClient::getAccountMgr();
		$user = $account->getUserByOpenId($openId, 'weixin');
		if ($user === NULL) {
			$err = $account->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
		}
		return $user;
	}

	/**
	 * 获取用户的OpenId。
	 * @param $userId	int		整数，是用户在AbleCloud平台上的ID。
	 * @return 			string	返回一个字符串，表示用户在微信系统中对应的OpenID。如果返回值为空字符串，表示操作失败。此时，可调用getLastError()方法获取错误信息。
	 */
	public function getUserOpenId($userId) {
		$account = ACClient::getAccountMgr();
		$openId = $account->getUserOpenId($userId, 'weixin');
		if (strlen($openId) == 0) {
			$err = $account->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
		}
		return $openId;
	}

	/**
	 * 设置用户的手机号。
	 * @param $openId	string	字符串，是微信用户的OpenID。
	 * @param $phone	string	字符串，是用户的新手机号。
	 * @param $verifyCode	string	字符串，是用户修改手机号码的验证码。
	 * @return			bool	操作成功时返回TRUE；否则返回FALSE，并且可调用getLastError()获取错误信息。
	 */
	public function setPhone($openId, $phone, $verifyCode) {
		$user = $this->getUser($openId);
		if ($user === NULL)
			return FALSE;
		$account = ACClient::getAccountMgr();
		if ($account->changePhone($user, $phone, $verifyCode, 'NA')) {
			$this->resetLastError();
			return TRUE;
		} else {
			$err = $account->getLastError();
			$this->resetLastError($err['errCode'], $err['errMessage']);
			return FALSE;
		}
	}

	/**
	 * 获取设备的二维码：微信二维码（附加设备在AbleCloud平台上所属的子域的信息），还可选择是否附加AbleCloud分享码。附加的信息是以JSON格式的字符串表示的。
	 * @param $physicalId		string	表示设备的物理ID。
	 * @param $subDomain		string	表示设备在AbleCloud平台中所属的子域的名字。如果值不为空字符串，则会以第三方自定义数据的方式在微信标准二维码的末尾附加该子域信息。
	 * @param $withACShareCode	bool	为TRUE时表示需要附加设备的AbleCloud分享码，否则表示不需附加该信息。
	 * @param $openId			string	$withShareCode为TRUE时，需要指定获取该分享码的用户的微信OpenID。仅设备的管理员用户有权限获取设备的分享码。
	 * @param $timeout			int		$withShareCode为TRUE时，需要指定分享码的有效时长。该参数的类型为整数，是以秒为单位指定分享码的有效时长。
	 * @return					string	操作成功时返回制作设备二维码的字符串。依据参数设置，该字符串中可能包含AbleCloud平台的分享码。操作失败时返回空字符串，此时可调用getLastError()方法获取错误消息。
	 */
	public function getDeviceQRCode($physicalId, $subDomain, $withACShareCode = FALSE, $openId = '', $timeout = 300) {
		// 取微信平台的二维码
		$qrCode = '';
		$data = array('device_num' => 1, 'device_id_list' => array($physicalId));
		$resp = $this->doHttpPost(self::WXQRCodeUrl, json_encode($data));
		if ($resp === NULL)
			return '';
		$resp = json_decode($resp, true);
		if (intval($resp['errcode']) == 0) {
			$qrCode = $resp['code_list'][0]['ticket'];
		} else {
			$this->resetLastError(intval($resp['errcode']), $resp['errmsg']);
			return '';
		}
		$contentArray = array();
		if (strlen($subDomain) > 0)
			$contentArray['sub_domain'] = $subDomain;
		if ($withACShareCode) {
			// 取AbleCloud平台的设备分享码。
			$user = $this->getUser($openId);
			if ($user === NULL)
				return '';
			$bindMgr = ACClient::getBindMgr();
			$deviceId = $bindMgr->getDeviceId($subDomain, $physicalId);
			if ($deviceId == 0) {
				$err = $bindMgr->getLastError();
				$this->resetLastError($err['errCode'], $err['errMessage']);
				return '';
			}
			$shareCode = $bindMgr->getDeviceShareCode($deviceId, $user, $timeout);
			if (strlen($shareCode) == 0) {
				$err = $bindMgr->getLastError();
				$this->resetLastError($err['errCode'], $err['errMessage']);
				return '';
			}
			$contentArray['share_code'] = $shareCode;
		}
		if (count($contentArray) > 0)
			$qrCode .= '#' . json_encode($contentArray);
		return $qrCode;
	}

	/// @name 信息同步方法。
	//@{
	/**
	 * 针对指定用户，同步AbleCloud平台与微信硬件平台记录的用户-设备绑定信息。同时检查指定设备的在线状态。
	 * @details 在响应微信硬件平台推送的subscribe_status/unsubscribe_status事件（比如用户打开/关闭公众号主界面）的方法 #onDeviceEventSubscribeStatus 及 #onDeviceEventUnsubscribeStatus 中，
	 * 可选择性地调用本方法同步数据。开发者也可根据实际情况主动调用本方法同步数据，比如处理蓝牙设备绑定关系的同步，或者将用户从“家”或“房间”里移除之后。
	 * @param $openId				string	待检查的用户的微信OpenID。
	 * @param $physicalIdOfStatus	string	是要查询其在线状态的设备的物理ID。
	 * @return						bool	返回TRUE表示待查询的设备在线；返回FALSE表示待查询的设备不在线。
	 */
	public function syncBindings($openId, $physicalIdOfStatus = '') {
		$onlineStatus  = FALSE;
		$missingFromAC = array();	// 在微信平台上绑定了，但未在AbleCloud平台上绑定的设备的物理ID。
		$missingFromWC = array();	// 在AbleCloud平台上绑定了，但未在微信平台上绑定的设备的物理ID。

		// 检查微信硬件平台记录的用户绑定的设备
		$wcDevListIsValid = FALSE;
		$wcDevList = array();
		$url = str_replace('OPENID', $openId, self::WXListDevicesUrl);
		$resp = $this->doHttpGet($url);
		if ($resp != NULL) {
        	$resp = json_decode($resp, true);
			if (is_array($resp) && array_key_exists('resp_msg', $resp)) {
				if ($resp['resp_msg']['ret_code'] == 0 && array_key_exists('device_list', $resp)) {
					$wcDevListIsValid = TRUE;
					foreach ($resp['device_list'] as $wcDev) {
						array_push($wcDevList, $wcDev['device_id']);
					}
				} else {
					if ($resp['resp_msg']['ret_code'] == 1 && strcasecmp($resp['resp_msg']['error_info'], 'not exist') == 0) {
						$wcDevListIsValid = TRUE;
					}
				}
			}
        }

        // 检查AbleCloud平台记录的用户绑定的设备
        $acDevListIsValid = FALSE;
        $acDevList = array();
        $user = $this->getUser($openId);
        if ($user != NULL) {
        	$bindMgr = ACClient::getBindMgr();
        	$deviceList = $bindMgr->listDevices($user);
        	if ($deviceList != NULL) {
        		$acDevListIsValid = TRUE;
        		foreach ($deviceList as $dev) {
        			array_push($acDevList, $dev->getPhysicalId());
        			if (strcmp($dev->getPhysicalId(), $physicalIdOfStatus) == 0) {
        				$onlineStatus = $bindMgr->isDeviceOnline($dev->getId());
        			}
				}
        	}
        }

        // 同步数据：以AbleCloud平台记录的数据为准，向微信平台同步数据。
        if ($wcDevListIsValid && $acDevListIsValid) {
        	foreach ($wcDevList as $wcDevId) {
        		$found = FALSE;
        		foreach ($acDevList as $acDevId) {
        			if (strcmp($acDevId, $wcDevId) == 0) {
        				$found = TRUE;
        				break;
        			}
        		}
        		if (!$found) {
        			array_push($missingFromAC, $wcDevId);
        		}
        	}
        	foreach ($acDevList as $acDevId) {
        		$found = FALSE;
        		foreach ($wcDevList as $wcDevId) {
        			if (strcmp($wcDevId, $acDevId) == 0) {
        				$found = TRUE;
        				break;
        			}
        		}
        		if (!$found) {
        			array_push($missingFromWC, $acDevId);
        		}
        	}
        	// 通知微信平台解邦设备
        	foreach ($missingFromAC as $devId) {
				$status = $this->wxUnbindDevice(strval($openId), $devId);
				if (!$status) {
					$err = $this->getLastError();
					error_log(json_encode($err));
				}
        	}
        	// 通知微信平台绑定设备
        	foreach ($missingFromWC as $devId) {
				$status = $this->wxBindDevice(strval($openId), $devId);
				if (!$status) {
					$err = $this->getLastError();
					error_log(json_encode($err));
				}
        	}
        }

		return $onlineStatus;
	}

	/**
	 * 针对指定设备，同步AbleCloud平台与微信硬件平台记录的用户-设备绑定信息。
	 * @details 在调用AbleCloud平台提供的API解除了某用户与设备的绑定关系后，或者将设备从“房间”或“家”中移除后，开发者需要调用本方法在AbleCloud平台与微信平台之间同步设备与用户的绑定关系。
	 * 在其它情况下，开发者也可根据实际情况主动调用本方法同步数据。
	 * @param $physicalId string 是设备的物理ID。
	 * @param $deviceType string 是本设备在微信公众号平台上的设备类型。
	 * @param $subDomain  string 是本设备在AbleCloud平台上所属的子域的名字。
	 * @return 			  bool	 操作成功时返回TRUE；操作失败时返回FALSE，同时可调用方法getLastError()获取错误信息。
	 */
	public function syncBindingsByDevice($physicalId, $deviceType, $subDomain) {
		$missingFromAC = array();	// 在微信平台上绑定了，但未在AbleCloud平台上绑定该设备的用户的OpenID。
		$missingFromWC = array();	// 在AbleCloud平台上绑定了，但未在微信平台上绑定该设备的用户的OpenID。

		// 检查微信硬件平台记录的用户与设备的绑定信息。
		$wcUserList = array();
		$url = str_replace('DEVICE_TYPE', $deviceType, self::WXListUserUrl);
		$url = str_replace('DEVICE_ID', $physicalId, $url);
		$resp = $this->doHttpGet($url);
		if ($resp != NULL) {
			$resp = json_decode($resp, true);
			if (is_array($resp) && array_key_exists('resp_msg', $resp)) {
				if ($resp['resp_msg']['ret_code'] == 0 && array_key_exists('open_id', $resp)) {
					foreach ($resp['open_id'] as $openId) {
						array_push($wcUserList, $openId);
					}
				} else {
					$this->resetLastError($resp['resp_msg']['ret_code'], $resp['resp_msg']['error_info']);
					return FALSE;
				}
			} else {
				$this->resetLastError(-1, 'Unknown response from WeChat API: ' . json_encode($resp));
				return FALSE;
			}
		} else {
			return FALSE;
		}

		// 检查AbleCloud平台记录的用户与设备的绑定信息。
		$acUserList = array();
		$bindMgr  = ACClient::getBindMgr();
		$deviceId = $bindMgr->getDeviceId($subDomain, $physicalId);
		if ($deviceId > 0) {
			$userList = $bindMgr->listUsers($deviceId);
			if ($userList != NULL) {
				$accountMgr = ACClient::getAccountMgr();
				foreach ($userList as $user) {
					$openId = $accountMgr->getUserOpenId($user->getId(), 'weixin');
					if (strlen($openId) > 0) {
						// 不一定所有的用户都来自微信。
						array_push($acUserList, $openId);
					}
				}
			} else {
				$err = $bindMgr->getLastError();
				$this->resetLastError($err['errCode'], $err['errMessage']);
				return FALSE;
			}
		} else {
			$err = $bindMgr->getLastError();
			if (intval($err['errCode']) != 3802 && strcasecmp($err['errMessage'], 'device not exist') != 0) {
				$this->resetLastError($err['errCode'], $err['errMessage']);
				return FALSE;
			}
		}

		// 同步数据：以AbleCloud平台记录的数据为准，向微信平台同步数据。
		foreach ($wcUserList as $wcOpenId) {
			$found = FALSE;
			foreach ($acUserList as $acOpenId) {
				if (strcmp($acOpenId, $wcOpenId) == 0) {
					$found = TRUE;
					break;
				}
			}
			if (!$found) {
				array_push($missingFromAC, $wcOpenId);
			}
		}
		foreach ($acUserList as $acOpenId) {
			$found = FALSE;
			foreach ($wcUserList as $wcOpenId) {
				if (strcmp($wcOpenId, $acOpenId) == 0) {
					$found = TRUE;
					break;
				}
			}
			if (!$found) {
				array_push($missingFromWC, $acOpenId);
			}
		}
		// 通知微信平台解邦设备
		foreach ($missingFromAC as $openId) {
			$status = $this->wxUnbindDevice(strval($openId), $physicalId);
			if (!$status) {
				$err = $this->getLastError();
				error_log(json_encode($err));
			}
		}
		// 通知微信平台绑定设备
		foreach ($missingFromWC as $openId) {
			$status = $this->wxBindDevice(strval($openId), $physicalId);
			if (!$status) {
				$err = $this->getLastError();
				error_log(json_encode($err));
			}
		}

		$this->resetLastError();
		return TRUE;
	}

	/**
	 * 检查设备在微信平台的授权及绑定状态，并尝试绑定设备与指定的用户。
	 * @details 使用微信客户端扫描设备二维码绑定设备却无反应时，可能是因为：（1）设备未在微信平台授权；（2）或用户已经绑定了该设备。
	 * 本方法首先检查设备在微信平台的授权状态，其次检查设备是否已在微信平台绑定了该用户，之后尝试在AbleCloud平台绑定设备，并与微信平台同步用户绑定设备的信息。
	 * @param $openId		string	拟要绑定设备的用户在微信平台的OpenID。
	 * @param $physicalId	string	要检查其状态并被绑定的设备在微信平台上的ID，即设备的物理ID。
	 * @param $deviceName	string	绑定设备和用户时，设备的显示名称。
	 * @param $subDomain	string	设备在AbleCloud平台上所属的子域的名字。
	 * @return				bool	操作成功时返回TRUE，否则返回FALSE。操作失败时可调用getLastError()方法获取错误信息。
	 */
	public function checkAndBindDevice($openId, $physicalId, $deviceName, $subDomain) {
		// 检查设备在微信平台的状态：是否已经授权。
		$url = str_replace('DEVICE_ID', $physicalId, self::WXGetStatUrl);
		$resp = $this->doHttpGet($url);
		if ($resp != NULL) {
			$resp = json_decode($resp, true);
			if (is_array($resp)) {
				if ($resp['errcode'] == 0 && array_key_exists('status', $resp)) {
					$devStatus = intval($resp['status']);
					if ($devStatus == 0) {
						// 设备未授权。
						$this->resetLastError(-1, '设备未在微信平台授权。');
						return FALSE;
					}
				} else {
					$this->resetLastError(-1 , 'Unknown response from WeChat API: ' . json_encode($resp));
					return FALSE;
				}
			} else {
				$this->resetLastError(-1, 'Unknown response from WeChat API: ' . json_encode($resp));
				return FALSE;
			}
		} else {
			return FALSE;
		}

		// 检查微信平台记录的设备与用户的绑定关系。
		$wcBoundWithUser = FALSE;
		$url = str_replace('OPENID', $openId, self::WXListDevicesUrl);
		$resp = $this->doHttpGet($url);
		if (!is_null($resp)) {
			$resp = json_decode($resp, true);
			if (is_array($resp) && array_key_exists('resp_msg', $resp)) {
				if ($resp['resp_msg']['ret_code'] == 0 && array_key_exists('device_list', $resp)) {
					foreach ($resp['device_list'] as $wcDev) {
						if (strcmp($wcDev['device_id'], $physicalId) == 0) {
							$wcBoundWithUser = TRUE;
						}
					}
				} else {
					if ($resp['resp_msg']['ret_code'] == 1 && strcasecmp($resp['resp_msg']['error_info'], 'not exist') == 0) {
						$wcBoundWithUser = FALSE;
					} else {
						$this->resetLastError($resp['resp_msg']['ret_code'], $resp['resp_msg']['error_info']);
						return FALSE;
					}
				}
			}
		} else {
			return FALSE;
		}

		// 在AbleCloud平台绑定用户和设备
		$user = $this->getUser($openId);
		if (is_null($user)) {
			return FALSE;
		}
		$bindMgr = ACClient::getBindMgr();
		// 处理用户直接扫描设备的微信二维码绑定设备的情况。
		$device = $bindMgr->bindDevice($subDomain, $physicalId, $deviceName, $user);
		if (!is_null($device)) {
			// 在AbleCloud平台绑定成功，则通知微信平台同步用户的绑定设备。
			$this->syncBindings($openId);
			return TRUE;
		} else {
			// 在AbleCloud平台中绑定设备失败。
			if ($wcBoundWithUser) {
				// 通知微信解除设备绑定。
				$status = $this->wxUnbindDevice($openId, $physicalId);
				if (!$status) {
					$err = $this->getLastError();
					error_log(json_encode($err));
				}
			}
			$err = $bindMgr->getLastError();
			error_log(json_encode($err));
			$this->resetLastError($err['errCode'], $err['errMessage']);
			return FALSE;
		}
	}
	//@}

	/**
	 * 通知微信绑定设备与用户。普通绑定失败时尝试强制绑定。
	 * @param $openId		string	要绑定设备的用户的OpenID。
	 * @param $physicalId	string	要被绑定的设备在微信平台上的ID，即设备的物理ID。
	 * @return				bool	成功时返回TRUE，否则可调用getLastError()方法获取错误信息。
	 */
	public function wxBindDevice($openId, $physicalId) {
		// 普通绑定
		$data = array('ticket' => $this->jsTicket, 'openid' => strval($openId), 'device_id' => $physicalId);
		$resp = $this->doHttpPost(self::WXBindUrl, json_encode($data));
		$json = json_decode($resp, TRUE);
		if (is_array($json) && array_key_exists('base_resp', $json)) {
			if (intval($json['base_resp']['errcode']) == 0) {
				$this->resetLastError();
				return TRUE;
			} else {
				$this->resetLastError(intval($json['base_resp']['errcode']), $json['base_resp']['errmsg']);
			}
		}
		// 强制绑定
		$data = array('openid' => strval($openId), 'device_id' => $physicalId);
		$resp = $this->doHttpPost(self::WXCompelBindUrl, json_encode($data));
		$json = json_decode($resp, TRUE);
		if (is_array($json) && array_key_exists('base_resp', $json)) {
			if (intval($json['base_resp']['errcode']) == 0) {
				$this->resetLastError();
				return TRUE;
			} else {
				$this->resetLastError(intval($json['base_resp']['errcode']), $json['base_resp']['errmsg']);
				return FALSE;
			}
		} else {
			return FALSE;
		}
	}

	/**
	 * 通知微信解绑设备。普通解绑失败时尝试强制解绑。
	 * @param $openId		string	要解绑设备的用户的OpenID。
	 * @param $physicalId	string	要被解绑的设备在微信平台上的ID，即设备的物理ID。
	 * @return				bool	成功时返回TRUE，否则可调用getLastError()方法获取错误信息。
	 */
	public function wxUnbindDevice($openId, $physicalId) {
		// 普通解绑
		$data = array('ticket' => $this->jsTicket, 'openid' => strval($openId), 'device_id' => $physicalId);
		$resp = $this->doHttpPost(self::WXUnbindUrl, json_encode($data));
		$json = json_decode($resp, TRUE);
		if (is_array($json) && array_key_exists('base_resp', $json)) {
			if (intval($json['base_resp']['errcode']) == 0) {
				$this->resetLastError();
				return TRUE;
			} else {
				$this->resetLastError(intval($json['base_resp']['errcode']), $json['base_resp']['errmsg']);
			}
		}
		// 强制解绑
		$data = array('openid' => strval($openId), 'device_id' => $physicalId);
		$resp = $this->doHttpPost(self::WXCompelUnbindUrl, json_encode($data));
		$json = json_decode($resp, TRUE);
		if (is_array($json) && array_key_exists('base_resp', $json)) {
			if (intval($json['base_resp']['errcode']) == 0) {
				$this->resetLastError();
				return TRUE;
			} else {
			  if (strcasecmp($json['base_resp']['errmsg'], 'not exist') == 0) {
			    $this->resetLastError();
				  return TRUE;
			  } else {
			    $this->resetLastError(intval($json['base_resp']['errcode']), $json['base_resp']['errmsg']);
				  return FALSE;
			  }
			}
		} else {
			return FALSE;
		}
	}

	/**
	 * 取最近一次错误消息。
	 * @return array	返回一个包含错误码和消息的关联数组：['errCode': 0, 'errMessage': '']。errCode为0时表示没有错误发生。
	 */
	public function getLastError() {
		return $this->lastError;
	}

	/**
	 * 重置错误信息。
	 * @param $errCode		int		错误消息码。
	 * @param $errMessage	string	消息内容。
	 */
	private function resetLastError($errCode = 0, $errMessage = '') {
		$this->lastError = array('errCode' => $errCode, 'errMessage' => $errMessage);
	}

	/**
	 * 执行HTTP POST请求。
	 * @param $url		string	POST请求的目标地址。
	 * @param $jsonData string	POST请求拟发送的数据。
	 * @return 			string	返回远程服务的响应消息。返回NULL时表示访问出错，此时可以调用getLastError()方法获取错误信息。
	 */
	private function doHttpPost($url, $jsonData) {
		$url = str_replace('ACCESS_TOKEN', $this->accessToken, $url);
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 60);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
		$response = curl_exec($ch);
		if (curl_errno($ch) != 0) {
			$this->resetLastError(intval(curl_errno($ch)), curl_error($ch));
			$response = NULL;
		}
		curl_close($ch);
		return $response;
	}

	/**
	 * 执行HTTP GET请求。
	 * @param $url	string	GET请求的目标地址。
	 * @return 		string	返回远程服务的响应消息。返回NULL时表示访问出错，此时可以调用getLastError()方法获取错误信息。
	 */
	private function doHttpGet($url) {
		$url = str_replace('ACCESS_TOKEN', $this->accessToken, $url);
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 60);
		curl_setopt($ch, CURLOPT_HTTPGET, 1);
		$response = curl_exec($ch);
		if (curl_errno($ch) != 0) {
			$this->resetLastError(intval(curl_errno($ch)), curl_error($ch));
			$response = NULL;
		}
		curl_close($ch);
		return $response;
	}

	private $accessToken;	// 微信公众号的Access Token。
	private $jsTicket;		// 微信公众号的JS API Ticket。
	private $lastError;		// 最近一次错误信息。
	const WXBindUrl			= "https://api.weixin.qq.com/device/bind?access_token=ACCESS_TOKEN";
	const WXCompelBindUrl	= "https://api.weixin.qq.com/device/compel_bind?access_token=ACCESS_TOKEN";
	const WXUnbindUrl		= "https://api.weixin.qq.com/device/unbind?access_token=ACCESS_TOKEN";
	const WXCompelUnbindUrl = "https://api.weixin.qq.com/device/compel_unbind?access_token=ACCESS_TOKEN";
	const WXQRCodeUrl		= "https://api.weixin.qq.com/device/create_qrcode?access_token=ACCESS_TOKEN";
	const WXListDevicesUrl	= "https://api.weixin.qq.com/device/get_bind_device?access_token=ACCESS_TOKEN&openid=OPENID";
	const WXListUserUrl		= "https://api.weixin.qq.com/device/get_openid?access_token=ACCESS_TOKEN&device_type=DEVICE_TYPE&device_id=DEVICE_ID";
	const WXGetStatUrl		= "https://api.weixin.qq.com/device/get_stat?access_token=ACCESS_TOKEN&device_id=DEVICE_ID";
}
